--[[

Titan Panel [All Trade]
Plug-in for Titan Panel that displays tradeskills for all characters. 
You will need to log in to each character to get it added to the display


Author: Riboflavin on Bloodhoof
v2.0.4 - Camisen
- modified toc file to version 20003 to update to latest version.
v2.0.3 - Camisen
- modified toc file to version 20003 to update to latest version.
v1.0 1/18/2006
- Initial Release

Requires Titan Panel!  (As if the name didn't clue you in)
Includes support for myAddOns

]]--

--debug output
local function
  tatDebug(message) DEFAULT_CHAT_FRAME:AddMessage(message, 0.4, 0.4, 0.4); 
end

--Initialize our variables
TitanAllTradeCharacters = {};
TitanAllTradeSettings = {};

TAT_UNIQUE_ID = "AllTrade";
TAT_VERSION_ID = "2.0.4";
TAT_FREQUENCY = 10;

TAT_MINUTE_SECONDS = 60;
TAT_HOUR_SECONDS = 3600;
TAT_DAY_SECONDS = 86400;

TAT_ABOUT_TEXT = TitanUtils_GetGreenText(TAT_MENU_TEXT)  .. "\n" .. 
                 TitanUtils_GetNormalText(TAT_STRING_VERSION) .. TitanUtils_GetHighlightText(TAT_VERSION_ID) .. "\n" ..
		 TitanUtils_GetNormalText(TAT_STRING_AUTHOR) .. TitanUtils_GetHighlightText("Riboflavin - Bloodhoof");

--Table of my help messages to be displayed
--myAddOns can't handle scrollbars for long text so we need to 
--manually break up the messages
helpTable = 
{
  TAT_HELP_LINE_1,
  TAT_HELP_LINE_2,
  TAT_HELP_LINE_3,
  TAT_HELP_LINE_4,
  TAT_HELP_LINE_5,
  TAT_HELP_LINE_6,
  TAT_HELP_LINE_7
 };

--myAddons support
TitanAllTradeDetails = 
{
  name = "TitanAllTrade",
  version = TAT_VERSION_ID,
  releaseDate = "January 12,2007",
  author = "Riboflavin - Bloodhoof",
  category = MYADDONS_CATEGORY_PROFESSIONS,
};

TitanAllTradeHelp = {};
TitanAllTradeHelp[1] = table.concat(helpTable, "\n\n");

--OnLoad
function TitanAllTradeButton_OnLoad()
  --Register for the events we care about
  this:RegisterEvent("UNIT_NAME_UPDATE");
  this:RegisterEvent("ADDON_LOADED");
  this:RegisterEvent("TRADE_SKILL_UPDATE");
  this:RegisterEvent("SKILL_LINES_CHANGED");
  
  --Register for Titan
  this.registry =
  {
    id = TAT_UNIQUE_ID,
    menuText = TAT_MENU_TEXT,
    buttonTextFunction = "TitanPanelAllTradeButton_GetButtonText",
    tooltipTitle = TAT_TOOLTIP_TITLE,
    tooltipTextFunction = "TitanPanelAllTradeButton_GetTooltipText",
    category = "Profession",
    version = TAT_VERSION_ID,
    frequency = TAT_FREQUENCY,
    icon = "Interface\\Icons\\Ability_Repair.blp";
    iconWidth = 16,
    savedVariables = 
    {
      ShowIcon = 1,
      ShowLabelText = 1,
      ShowLevel = TITAN_NIL,
    }
  }

  --lets see if I can get this hook thing to work
  --TitanAllTradeLogout = Logout;
  --Logout = function() RequestTimePlayed(); TitanAllTradeLogout(); end
  -- quitting too
  --TitanAllTradeQuit = Quit;
  --Quit = function() RequestTimePlayed(); TitanAllTradeQuit(); end;

end

--the main event!
function TitanAllTradeButton_OnEvent()
  if (event == "TRADE_SKILL_UPDATE") then
    --local wtf = GetNumTradeSkills();
  elseif (event == "UNIT_NAME_UPDATE" or event == "SKILL_LINES_CHANGED") then
    --make sure this player is up to date
    getCurrentPlayerTradeStats();
  elseif ((event == "ADDON_LOADED") and (arg1 == "TitanAllTrade"))then
    --See if we have settings..
    if (TitanAllTradeSettings["playerOptions"] == nil) then
      --Create our options table
      createTATOptionsTable();
    end
    
    --myAddOns Support
    if (myAddOnsFrame_Register) then
      myAddOnsFrame_Register(TitanAllTradeDetails, TitanAllTradeHelp);
    end

    --Might actually help to call this so people upgrading don't get errors (luckily most of my new additions default to false)
    upgradeTATVersion();
  end
end

--
-- Titan functions
--
function TitanPanelAllTradeButton_GetButtonText(id)

  if (TitanGetVar(TAT_UNIQUE_ID, "ShowLabelText")) then
    return TAT_BUTTON_LABEL;
  end
end

function TitanPanelAllTradeButton_GetTooltipText()

  --Here I need to loop through my players and list out what we know.
  local tooltipDataTAT = "";

  --Do some crazy ass sorting thing to actually sort. 
  sortedServers = sortByTATName(TitanAllTradeCharacters);
  for k, v in ipairs (sortedServers) do
    tooltipDataTAT = tooltipDataTAT .. displayServerCharactersTAT(v, TitanAllTradeCharacters[v]);
  end 
  
  return tooltipDataTAT;
end

--
-- create menus
--
function TitanPanelRightClickMenu_PrepareAllTradeMenu()

  local info = {};
  
  if (UIDROPDOWNMENU_MENU_LEVEL == 3) then
    
    if (UIDROPDOWNMENU_MENU_VALUE == "ListOptions") then
      
      --toggle all servers
      info = {};
      info.text = TAT_MENU_ALLSERVERS_OPTION;
      info.value = "ShowAllServersToggle";
      info.func = showAllTATServersToggle;
      info.checked = (TitanAllTradeSettings["playerOptions"].listScope == TAT_ARG_ALLSERVERS);
      UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);

      --toggle current server display
      info = {};
      info.text = TAT_MENU_CURRENTSERVER_OPTION;
      info.value = "ShowCurrentServerToggle";
      info.func = showCurrentTATServerToggle;
      info.checked = (TitanAllTradeSettings["playerOptions"].listScope == TAT_ARG_CURRENTSERVER);
      UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);
    end
    return;
  end
  
  if (UIDROPDOWNMENU_MENU_LEVEL == 2) then
    if ( UIDROPDOWNMENU_MENU_VALUE == "DisplayAbout" ) then
      info = {};
      info.text = TAT_ABOUT_TEXT;
      info.value = "AboutTextPopUP";
      info.notClickable = 1;
      info.isTitle = 0;
      UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);
    
    elseif ( UIDROPDOWNMENU_MENU_VALUE == "DisplayOptions" ) then
      --toggle level display
      info = {};
      info.text = TAT_MENU_LEVEL_OPTION;
      info.value = "ShowLevelToggle";
      info.func = showTATLevelToggle;
      info.checked = TitanAllTradeSettings["playerOptions"].showLevel;
      info.keepShownOnClick = 1;
      UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);

      --include this character in our calculations?
      local thisRealm = GetRealmName();
      local thisPlayer = UnitName("player");
      info = {};
      info.text = TAT_MENU_IGNORE_OPTION;
      info.value = "ignoreCharacter";
      info.func = ignoreTATCharacterToggle;
      info.checked = TitanAllTradeCharacters[thisRealm][thisPlayer].ignore;
      info.keepShownOnClick = 1;
      UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);
       
      --secondary skills
      info = {};
      info.text = TAT_MENU_SECONDARY_OPTION;
      info.value = "ShowSecondarySkillsToggle";
      info.func = showTATSecondarySkillsToggle;
      info.checked = TitanAllTradeSettings["playerOptions"].showSecondary;
      info.keepShownOnClick = 1;
      UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);

      --Short text
      info = {};
      info.text = TAT_MENU_SHORTTEXT_OPTION;
      info.value = "ShortTextTATToggle";
      info.func = shortTextTATToggle;
      info.checked = TitanAllTradeSettings["playerOptions"].shortText;
      info.keepShownOnClick = 1;
      UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);

      --color maxxed out skill
      info = {};
      info.text = TAT_MENU_MAXSKILL_OPTION;
      info.value = "ShowTATCompleteSkillToggle";
      info.func = showTATCompleteSkillToggle;
      info.checked = TitanAllTradeSettings["playerOptions"].showCompleteSkill;
      info.keepShownOnClick = 1;
      UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);

      --toggle level sort
      info = {};
      info.text = TAT_MENU_SORT_LEVEL_OPTION;
      info.value = "SortLevelToggle";
      info.func = sortTATLevelToggle;
      info.checked = (TitanAllTradeSettings["playerOptions"].listSort == TAT_ARG_LEVELSORT);
      info.keepShownOnClick = 1;
      UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);
      
      
      --Create a list scope options menu
      info = {};
      info.text = TAT_MENU_LISTSCOPE_OPTIONS;
      info.value = "ListOptions";
      info.hasArrow = 1;
      UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);
      
      --reset button (shouldn't be necessary once I stop changing the saved vars)
      info = {};
      info.text = TAT_MENU_RESET_OPTION;
      info.value = "resetData";
      info.func = resetTATData;
      UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);
      
    end
    return;
  end

  TitanPanelRightClickMenu_AddTitle(TitanPlugins[TAT_UNIQUE_ID].menuText);
  
  --TitanPanelRightClickMenu_AddToggleIcon(TAT_UNIQUE_ID);
  TitanPanelRightClickMenu_AddToggleLabelText(TAT_UNIQUE_ID);
  TitanPanelRightClickMenu_AddCommand(TITAN_PANEL_MENU_HIDE, TAT_UNIQUE_ID, TITAN_PANEL_MENU_FUNC_HIDE);

  --Create an options menu
  info = {};
  info.text = TAT_MENU_OPTIONS;
  info.value = "DisplayOptions";
  info.hasArrow = 1;
  UIDropDownMenu_AddButton(info);

  -- info about plugin
  info = {};
  info.text = TAT_MENU_ABOUT_TEXT;
  info.value = "DisplayAbout";
  info.hasArrow = 1;
  UIDropDownMenu_AddButton(info);
end

function showTATLevelToggle()
  TitanAllTradeSettings["playerOptions"].showLevel = not TitanAllTradeSettings["playerOptions"].showLevel;

  TitanPanelButton_UpdateButton(TAT_UNIQUE_ID);
end

function showAllTATServersToggle()
  if (TitanAllTradeSettings["playerOptions"].listScope == TAT_ARG_ALLSERVERS) then
    --if we turn off an option, we will automatically set the value to be current char
    TitanAllTradeSettings["playerOptions"].listScope = TAT_ARG_CURRENTPLAYER;
  else
    --set it to be all servers
    TitanAllTradeSettings["playerOptions"].listScope = TAT_ARG_ALLSERVERS
  end
end

function showCurrentTATServerToggle()
	  if (TitanAllTradeSettings["playerOptions"].listScope == TAT_ARG_CURRENTSERVER) then
	    --if we turn off an option, we will automatically set the value to be current char
    TitanAllTradeSettings["playerOptions"].listScope = TAT_ARG_CURRENTPLAYER;
  else
    TitanAllTradeSettings["playerOptions"].listScope = TAT_ARG_CURRENTSERVER
  end
end

function resetTATData()
  TitanAllTradeCharacters = {};
  TitanAllTradeSettings = {};
  createTATOptionsTable();
  --if they do a reset, repopulate with current char
  --RequestTimePlayed();
  getCurrentPlayerTradeStats();
end

function createTATOptionsTable()
  TitanAllTradeSettings["playerOptions"] = {listScope, showLevel, showLabelValue, showCurrentOnlyLabel, listSort, showCompleteSkill};
  --Give it some default options
  TitanAllTradeSettings["playerOptions"].listScope = TAT_ARG_ALLSERVERS;
  TitanAllTradeSettings["playerOptions"].showLevel = true;
  TitanAllTradeSettings["playerOptions"].showLabelValue = true;
  TitanAllTradeSettings["playerOptions"].showCurrentOnlyLabel = false;
  TitanAllTradeSettings["playerOptions"].listSort = TAT_ARG_CHARACTER;
  TitanAllTradeSettings["playerOptions"].showCompleteSkill = true;
end

function displayServerCharactersTAT(index, characters)
  local tooltip = "";
  local indicies;
  
  --How does the user want to sort?
  if (TitanAllTradeSettings["playerOptions"].listSort == TAT_ARG_LEVELSORT) then
    indicies = sortByTATLevel(characters);
  else
    indicies = sortByTATName(characters);
  end

  --display "headings" in a different color
  if (TitanAllTradeSettings["playerOptions"].listScope == TAT_ARG_ALLSERVERS) then
    tooltip = tooltip .. "\n" .. TitanUtils_GetNormalText(index .. " " .. TAT_STRING_CHARACTERS .. ":\n");
  end
  
  --I still need to loop through everything to get my info
  --beats the hell out of me what this does, but it seems to work.  I mean really, what's wrong with
  --normal for loops.  
  for k, v in ipairs (indicies) do
    tooltip = tooltip .. displayAllTrade(v, characters[v]);  
  end

  return tooltip;
end

function displayAllTrade(index, data)
  local tooltip = "";
  local factionColor = {r=0.0, g=0.5, b=1.0};
  if (data.faction == "Horde") then
    factionColor = {r=1.0, g=0.0, b=0.0};
  end
  --someone tell me why the ! I have a 1... I don't understand
  if (index ~= 1 and not data.ignore) then
    --build the output for this character
    --Here is the format (showing everything)
    --Player (level)  /t  TS (x) / TS (x) [/ TS (x) / TS (x) / TS (x)] 
    
    --attempt at some sort of formatting (output was original tooltip)
    local frontput = data.name;
    local backput = "";  

    --if we want to see level, show that here
    if (TitanAllTradeSettings["playerOptions"].showLevel) then
      frontput = frontput .. " (" .. data.level .. ")";
    end

    local needSpecialSeparator = false;

    --does this user have tradeskills?
    if (data.firstTS == nill and data.secondTS == nill) then
      backput = backput .. TitanUtils_GetColoredText(TAT_STRING_NO_TRADESKILLS, factionColor);
    else
      --show some tradeskills
      if (data.firstTS ~= nil) then
        backput = backput .. TitanUtils_GetColoredText(getTATDisplayStringForSkill(data.firstTS) .. " (", factionColor) .. getTATSkillColor(data.firstTSLevel, data.firstTSMax, factionColor) .. TitanUtils_GetColoredText(")", factionColor);
	needSpecialSeparator = true;
      end

      if (data.secondTS ~= nil) then
        backput = backput .. TitanUtils_GetColoredText(" / " .. getTATDisplayStringForSkill(data.secondTS) .. " (", factionColor) .. getTATSkillColor(data.secondTSLevel, data.secondTSMax, factionColor) .. TitanUtils_GetColoredText(")", factionColor);
	needSpecialSeparator = true;
      end
    end

    if (TitanAllTradeSettings["playerOptions"].showSecondary) then
      if (data.firstAidLevel ~= nil) then
        --some chars may have secondary skills, but no professions.  do we need a separator
        if (backput ~= "") then
          --it's not empty, so lets separate, but which?
	  if (needSpecialSeparator) then
	    backput = backput .. " / ";
	    --only need one of these
	    needSpecialSeparator = false;
	  else
	    backput = backput .. TitanUtils_GetColoredText(" / ", factionColor);
	  end
        end
        backput = backput .. TitanUtils_GetColoredText(getTATDisplayStringForSkill(TAT_SKILL_FIRSTAID) .. " (", factionColor) .. getTATSkillColor(data.firstAidLevel, data.firstAidMax, factionColor) .. TitanUtils_GetColoredText(")", factionColor);
      end

      if (data.cookingLevel ~= nil) then
        --some chars may have secondary skills, but no professions.  do we need a separator
        if (backput ~= "") then
	  --it's not empty, so lets separate, but which?
	  if (needSpecialSeparator) then
	    backput = backput .. " / ";
	    --only need one of these
	    needSpecialSeparator = false;
	  else
	    backput = backput .. TitanUtils_GetColoredText(" / ", factionColor);
	  end
        end
        backput = backput .. TitanUtils_GetColoredText(getTATDisplayStringForSkill(TAT_SKILL_COOKING) .. " (", factionColor) .. getTATSkillColor(data.cookingLevel, data.cookingMax, factionColor) .. TitanUtils_GetColoredText(")", factionColor);
      end

      if (data.fishingLevel ~= nil) then
        --some chars may have secondary skills, but no professions.  do we need a separator
        if (backput ~= "") then
	  --it's not empty, so lets separate, but which?
	  if (needSpecialSeparator) then
	    backput = backput .. " / ";
	    --only need one of these
	    needSpecialSeparator = false;
	  else
	    backput = backput .. TitanUtils_GetColoredText(" / ", factionColor);
	  end
        end
        backput = backput .. TitanUtils_GetColoredText(getTATDisplayStringForSkill(TAT_SKILL_FISHING) .. " (", factionColor) .. getTATSkillColor(data.fishingLevel, data.fishingMax, factionColor) .. TitanUtils_GetColoredText(")", factionColor);
      end
    end

    --check our list options
    if (printTATOutput(data)) then
      --Indent player
      tooltip = "  " .. TitanUtils_GetColoredText(frontput, factionColor) .. "\t" .. backput  .."\n";
    end;
  end
  return tooltip;
end

function printTATOutput(data)
  --don't print anything unless we determine we should
  local retValue = false;
  if (TitanAllTradeSettings["playerOptions"].listScope == TAT_ARG_CURRENTPLAYER and
      data.name == UnitName("player") and data.realm == GetRealmName()) then
    retValue = true;
  elseif (TitanAllTradeSettings["playerOptions"].listScope == TAT_ARG_CURRENTSERVER and
          data.realm == GetRealmName()) then
    retValue = true;
  elseif (TitanAllTradeSettings["playerOptions"].listScope == TAT_ARG_ALLSERVERS) then
    retValue = true;
  end
  return retValue;
end

function sortByTATName(characters)
  --this is a table of characters, I copied this sort from a MUD website because 
  --I could not figure out a better way:
  t2 = {}
  
  table.foreach (characters, function (k) if (k ~= 1) then table.insert (t2, k) end end )

  table.sort (t2)
  return t2;
end

--Might be a better way to do this, but as I add features and items to the saved variables, I want
--to make sure we have valid data for all versions.
function upgradeTATVersion()
  TitanAllTradeSettings["playerOptions"].showCompleteSkill = true;
end

function ignoreTATCharacterToggle()
  local thisRealm = GetRealmName();
  local thisPlayer = UnitName("player");

  TitanAllTradeCharacters[thisRealm][thisPlayer].ignore = not TitanAllTradeCharacters[thisRealm][thisPlayer].ignore;

  TitanPanelButton_UpdateButton(TAT_UNIQUE_ID);
end


function sortByTATLevel(characters)
  local nameTable = {};
  local levelTable = {};
  table.foreach(characters, function(k)
                              if (k ~= 1) then
			        --get the level number
				local charLevel = characters[k].level;
				--where should this entry go?
				local tableIndex = getTATIndex(levelTable, charLevel);
				--insert data into our two tables.
				table.insert(nameTable, tableIndex, k);
				table.insert(levelTable, tableIndex, charLevel);
		              end
			    end);
  return nameTable;
end

function getTATIndex(pTable, pValue)
  --loop through the table and find out where this needs to go.
  local retIndex = 1;
  for k, v in pairs (pTable) do
    --we are in the table, loop until we find a bigger value
    if (pValue <= v) then
      retIndex = retIndex + 1;
    end
  end

  return retIndex
end

function sortTATLevelToggle()
  if (TitanAllTradeSettings["playerOptions"].listSort == TAT_ARG_LEVELSORT) then
    --if we turn off an option, we will automatically set the value to be current char
    TitanAllTradeSettings["playerOptions"].listSort = TAT_ARG_ALPHASORT;
  else
    --set it to be all servers
    TitanAllTradeSettings["playerOptions"].listSort = TAT_ARG_LEVELSORT;
  end
end

function getCurrentPlayerTradeStats()
  --Save (update) our entry for this player
  local thisRealm = GetRealmName();
  local thisPlayer = UnitName("player");
    
  if (TitanAllTradeCharacters[thisRealm] == nil) then
    TitanAllTradeCharacters[thisRealm] = {thisPlayer};
  end

  if (TitanAllTradeCharacters[thisRealm][thisPlayer] == nil) then
    TitanAllTradeCharacters[thisRealm][thisPlayer] = {realm, name, level, faction, firstTS, firstTSLevel, secondTS, secondTSLevel,
                                                      cookingLevel, firstAidLevel, fishingLevel, lastUpdate, ignore, showSecondary,
						      shortText, firstTSMax, secondTSMax, firstAidMax, fishingMax, cookingMax};

    TitanAllTradeCharacters[thisRealm][thisPlayer].ignore = false;
    TitanAllTradeCharacters[thisRealm][thisPlayer].showSecondary = false;
    TitanAllTradeCharacters[thisRealm][thisPlayer].shortText = true;
  end
    
  --set the data
  TitanAllTradeCharacters[thisRealm][thisPlayer].realm = thisRealm;
  TitanAllTradeCharacters[thisRealm][thisPlayer].name = thisPlayer;
  TitanAllTradeCharacters[thisRealm][thisPlayer].level = UnitLevel("player");
  TitanAllTradeCharacters[thisRealm][thisPlayer].faction = UnitFactionGroup("player");
  TitanAllTradeCharacters[thisRealm][thisPlayer].lastUpdate = time();
  
  --A player can dump his/her professions, so lets clear out our data so we don't report old info
  TitanAllTradeCharacters[thisRealm][thisPlayer].firstTS = nil;
  TitanAllTradeCharacters[thisRealm][thisPlayer].firstTSLevel = nil;
  TitanAllTradeCharacters[thisRealm][thisPlayer].firstTSMax = nil;
  TitanAllTradeCharacters[thisRealm][thisPlayer].secondTS = nil;
  TitanAllTradeCharacters[thisRealm][thisPlayer].secondTSLevel = nil;  
  TitanAllTradeCharacters[thisRealm][thisPlayer].secondTSMax = nil;

  local numSkills = GetNumSkillLines();
  
  for skillIndex = 1, numSkills do
    --return values from blizz call
    skillName, isHeader, isExpanded, skillRank, numTempPoints, skillModifier, skillMaxRank, isAbandonable, stepCost, rankCost, minLevel, skillCostType = GetSkillLineInfo(skillIndex);

    if (isHeader) then
      --only look for what we care about
      if (skillName == "Professions") then
	--open it
	if (not isExpanded) then
	  ExpandSkillHeader(skillIndex); 
        end
	
        --if we have this heading, we have at least one Profession (i hope)
        skillName1, isHeader1, isExpanded1, skillRank1, ntp1, sm1, skillMaxRank1 = GetSkillLineInfo(skillIndex + 1);
	--save this skill
	TitanAllTradeCharacters[thisRealm][thisPlayer].firstTS = skillName1;
	TitanAllTradeCharacters[thisRealm][thisPlayer].firstTSLevel = skillRank1;
	TitanAllTradeCharacters[thisRealm][thisPlayer].firstTSMax = skillMaxRank1;

        --we can potentially have another skill, two max
        skillName2, isHeader2, isExpanded2, skillRank2, ntp2, sm2, skillMaxRank2 = GetSkillLineInfo(skillIndex + 2);
	--if this is not a header, then we found another skill
	if (not isHeader2) then
	  --ok to save this info too
	  TitanAllTradeCharacters[thisRealm][thisPlayer].secondTS = skillName2;
          TitanAllTradeCharacters[thisRealm][thisPlayer].secondTSLevel = skillRank2;  
	  TitanAllTradeCharacters[thisRealm][thisPlayer].secondTSMax = skillMaxRank2;
        end

	--put this back
	if (not isExpanded) then
          CollapseSkillHeader(skillIndex); 
        end
      elseif (skillName == "Secondary Skills") then
        --open it
	if (not isExpanded) then
	  ExpandSkillHeader(skillIndex); 
        end

	--loop through the secondary skills
	--man, this is really really dumb, but I'm too lazy to figure out a better way
	--especially since I hate these for loops, right now we have a potential of 3 secondary skills,
	--first aid, fishing, cooking... they put the mount skills there too, but looks like they are always
	--at the bottom of the list.
        for secondaryIndex = 1, 3 do
	  --if we have this heading, we have at least one Secondary Skill (i hope)
          sSkillName, isSHeader, isSExpanded, sSkillRank, sntp, ssm, sSkillMaxRank = GetSkillLineInfo(skillIndex + secondaryIndex);

	  if (not isSHeader) then
	    --we found a secondary skill, but is it one we care about?
	    saveSecondarySkill(sSkillName, sSkillRank, thisRealm, thisPlayer, sSkillMaxRank);
	  else
	    --we found a header, stop loop in a really lame way (is there a "break" keyword in lua?)
	    secondaryIndex = 3;
	  end
	end
        
	--put this back
	if (not isExpanded) then
          CollapseSkillHeader(skillIndex); 
        end
      end
    end
  end
end

function saveSecondarySkill(secondaryName, secondaryRank, thisRealm, thisPlayer, secondaryMaxRank)
  if (secondaryName == TAT_SKILL_FIRSTAID) then
    --save first aid level
    TitanAllTradeCharacters[thisRealm][thisPlayer].firstAidLevel = secondaryRank;
    TitanAllTradeCharacters[thisRealm][thisPlayer].firstAidMax = secondaryMaxRank;
  elseif (secondaryName == TAT_SKILL_FISHING) then
    --save first aid level
    TitanAllTradeCharacters[thisRealm][thisPlayer].fishingLevel = secondaryRank;
    TitanAllTradeCharacters[thisRealm][thisPlayer].fishingMax = secondaryMaxRank;
  elseif (secondaryName == TAT_SKILL_COOKING) then
    --save first aid level
    TitanAllTradeCharacters[thisRealm][thisPlayer].cookingLevel = secondaryRank;
    TitanAllTradeCharacters[thisRealm][thisPlayer].cookingMax = secondaryMaxRank;
  end
end

function showTATSecondarySkillsToggle()
  TitanAllTradeSettings["playerOptions"].showSecondary = not TitanAllTradeSettings["playerOptions"].showSecondary;

  TitanPanelButton_UpdateButton(TAT_UNIQUE_ID);
end

function shortTextTATToggle()
  TitanAllTradeSettings["playerOptions"].shortText = not TitanAllTradeSettings["playerOptions"].shortText;

  TitanPanelButton_UpdateButton(TAT_UNIQUE_ID);
end

--decides how to display skills
function getTATDisplayStringForSkill(skillName)
  local retValue = skillName;
  --are we showing short names?
  if (TitanAllTradeSettings["playerOptions"].shortText) then
    --determine which skill we have and return my short name
    if (skillName == TAT_SKILL_ALCHEMY) then
      retValue = TAT_SKILL_ALCHEMY_ABB;
    elseif (skillName == TAT_SKILL_BLACKSMITH) then
      retValue = TAT_SKILL_BLACKSMITH_ABB;
    elseif (skillName == TAT_SKILL_COOKING) then
      retValue = TAT_SKILL_COOKING_ABB;
    elseif (skillName == TAT_SKILL_ENCHANT) then
      retValue = TAT_SKILL_ENCHANT_ABB;
    elseif (skillName == TAT_SKILL_ENGINEER) then
      retValue = TAT_SKILL_ENGINEER_ABB;
    elseif (skillName == TAT_SKILL_FIRSTAID) then
      retValue = TAT_SKILL_FIRSTAID_ABB;
    elseif (skillName == TAT_SKILL_FISHING) then
      retValue = TAT_SKILL_FISHING_ABB;
    elseif (skillName == TAT_SKILL_HERBAL) then
      retValue = TAT_SKILL_HERBAL_ABB;
    elseif (skillName == TAT_SKILL_LEATHER) then
      retValue = TAT_SKILL_LEATHER_ABB;
    elseif (skillName == TAT_SKILL_MINING) then
      retValue = TAT_SKILL_MINING_ABB;
    elseif (skillName == TAT_SKILL_SKINNING) then
      retValue = TAT_SKILL_SKINNING_ABB;
    elseif (skillName == TAT_SKILL_TAILOR) then
      retValue = TAT_SKILL_TAILOR_ABB;
    else
      retValue = skillName;
    end
  end

  return retValue;
end

function getTATSkillColor(skillValue, skillMax, fColor)
  local spacedValue = skillValue;
  --my genius tester wants this value spaced
  if (skillValue < 10) then
    spacedValue = "    " .. skillValue;
  elseif (skillValue < 100) then
    spacedValue = "  " .. skillValue;
  end
  --if we color the skills when they are maxed, give them green, otherwise, faction color.
  retValue = TitanUtils_GetColoredText(spacedValue, fColor);
  if (TitanAllTradeSettings["playerOptions"].showCompleteSkill) then
    if (skillValue == skillMax) then
      --we have maxed out this skill (for now :)
      retValue = TitanUtils_GetGreenText(spacedValue);
    end
  end

  return retValue;

end

function showTATCompleteSkillToggle()
  TitanAllTradeSettings["playerOptions"].showCompleteSkill = not TitanAllTradeSettings["playerOptions"].showCompleteSkill;

  TitanPanelButton_UpdateButton(TAT_UNIQUE_ID);
end