-- ## Tracks battleground queues and statistics.
-- ##
-- ## This mod was adapted from TitanBG 1600 developed by Sphiromi.
-- ##
-- ## Name    : TitanBG (Updated)
-- ## Author  : Nirme (Frostwolf), Sphiromi
-- ## Version : 1.3.2


-- #####################
-- ## EVENT FUNCTIONS ##
-- #####################


function TBG_OnEvent(event, arg1, arg2)
	if (event == "ADDON_LOADED" and not TBG.Var.Addon.ADDON_LOADED) then
		TBG.Var.Addon.ADDON_LOADED = true; TBG_Init_OnLoad();

	elseif (event == "VARIABLES_LOADED" and not TBG.Var.Addon.VARIABLES_LOADED) then
		TBG.Var.Addon.VARIABLES_LOADED = true; TBG_Init_OnLoad();

	-- Store player information variables.
	elseif (event == "PLAYER_ENTERING_WORLD") then
		if (TBG.Var.Addon.LOADED and not TBG.Var.Addon.PLAYER_ENTERING_WORLD) then
			TBG_Init_CreatePlayerInformation();
			TBG_ZonesChanged();
		end

		TBG.Var.Addon.PLAYER_ENTERING_WORLD = true;

	elseif (event == "ZONE_CHANGED_NEW_AREA") then
		if (TBG.Var.Addon.LOADED and TBG.Var.Addon.PLAYER_ENTERING_WORLD) then
			TBG_ZonesChanged();
		end

	else
		if (not TBG.Var.Addon.LOADED) then return nil; end
		if (not TBG_IsEnabled())      then return nil; end

		-- Do not update if zone variables have not been reset.
		if (TBG.Var.Player.zone ~= nil and TBG.Var.Player.zone == TBG.Var.Addon.ZONE_CHANGED_NEW_AREA) then

			if (event == "FRIENDLIST_UPDATE") then
				TBG.DB.friends = { };

				for i = 1, GetNumFriends() do
					local fname = GetFriendInfo(i);

					if (fname ~= nil) then
						TBG.DB.friends[fname] = i;
					end
				end

			-- ## Hide join popup and minimap button.
			elseif (event == "UPDATE_BATTLEFIELD_STATUS") then
				TBG_ToggleMiniMapIcon();

			elseif (event == "UPDATE_WORLD_STATES") then
				TBG_Update_Score();

			-- If the user opens the battlefield queue window.
			elseif (event == "BATTLEFIELDS_SHOW") then
				TBG_Battleground_StoreNumberOfOpenInstances();

			elseif (event == "RAID_ROSTER_UPDATE") then
				TBG.Var.BG.scan_raid = true;

			-- ## Alliance messages.
			elseif (event == "CHAT_MSG_BG_SYSTEM_ALLIANCE") then
				if (TBG.Var.Player.zone == TBG.Text.Ref.BG_WSG or TBG.Var.Player.zone == TBG.Text.Ref.BG_EOTS) then
					TBG.DB.active.stats.bg.stats.alliance.msg = arg1;
				end

				if (TBG_Saved("MGI_RedirectMessageAlliance")) then
					RaidWarningFrame:AddMessage(arg1, 0, 0.68, 0.94, 1, 0);
					TBG_Tip_Generate("BG_MESSAGE_REDIRECT");
				end

			-- ## Horde messages.
			elseif (event == "CHAT_MSG_BG_SYSTEM_HORDE") then
				if (TBG.Var.Player.zone == TBG.Text.Ref.BG_WSG or TBG.Var.Player.zone == TBG.Text.Ref.BG_EOTS) then
					TBG.DB.active.stats.bg.stats.horde.msg = arg1;
				end

				if (TBG_Saved("MGI_RedirectMessageHorde")) then
					RaidWarningFrame:AddMessage(arg1, 1.0, 0.1, 0.1, 1, 0);
					TBG_Tip_Generate("BG_MESSAGE_REDIRECT");
				end

			-- ## Neutral messages.
			elseif (event == "CHAT_MSG_BG_SYSTEM_NEUTRAL") then
				if (TBG_Saved("MGI_RedirectMessageNeutral")) then
					RaidWarningFrame:AddMessage(arg1, 1.0, 0.82, 0, 1, 0);
					TBG_Tip_Generate("BG_MESSAGE_REDIRECT");
				end

			elseif (event == "CHAT_MSG_BATTLEGROUND" or event == "CHAT_MSG_BATTLEGROUND_LEADER" or event == "CHAT_MSG_PARTY" or event == "CHAT_MSG_RAID" or event == "CHAT_MSG_RAID_LEADER" or event == "CHAT_MSG_RAID_WARNING" or event == "CHAT_MSG_WHISPER") then
				TBG_Battleground_InactivesSetChatAuthorActive(arg2);

			-- This event happens when the interface is loaded.
			elseif (event == "UPDATE_CHAT_WINDOWS") then

			-- Catch Ivus and Lokholar movement.
			elseif (event == "CHAT_MSG_MONSTER_YELL") then
				if (TBG.Var.Player.zone == TBG.Text.Ref.BG_AV) then

					-- Ivus has been summoned.
					if (string.find(arg1, TBG.Text.Ref.AV_IVUS_START)) then
						TBG.DB.active.stats.bg.stats.summons["IVUS"] = GetTime();

					-- Lokholar has been summoned.
					elseif (string.find(arg1, TBG.Text.Ref.AV_LOK_START)) then
						TBG.DB.active.stats.bg.stats.summons["LOK"] = GetTime();
					end
				end

			-- Scan battleground items.
			elseif (event == "BAG_UPDATE") then
				if (not arg1 or arg1 < 1 or arg1 > 4) then
				else
					local id                  = GetInventorySlotInfo("Bag" .. arg1 - 1 .. "Slot");
					local link                = GetInventoryItemLink("player", id);
					local _, _, _, _, _, type = GetItemInfo(link);

					if (type ~= "Container") then return nil; end
				end

				TBG.Var.BG.scan_inventory = true;

			elseif (event == "GOSSIP_SHOW") then
				TBG_Gossip_OnShow();
			end
		end
	end
end


-- ## Triggers a chat message event for the Blizzard chat frames and the addon.
function TBG_Event_TriggerChatMessage(event)
	ChatFrame_MessageEventHandler(event);
	TBG_OnEvent(event);
end


-- ## Checks that all files have been loaded correctly.
function TBG_AllFilesLoaded()
	if (not TBG_Files_Variables_IsLoaded     or
	    not TBG_Files_Utils_IsLoaded         or
	    not TBG_Files_Tooltip_IsLoaded       or
	    not TBG_Files_Settings_IsLoaded      or
	    not TBG_Files_Menu_IsLoaded          or
	    not TBG_Files_LeaderBoard_IsLoaded   or
	    not TBG_Files_Init_IsLoaded          or
	    not TBG_Files_Hooks_IsLoaded         or
	    not TBG_Files_Battlegrounds_IsLoaded or
	    not TBG_Files_Arenas_IsLoaded        or
	    not TBGX_XML_Tooltip_IsLoaded        or
	    not TBGX_XML_Templates_IsLoaded      or
	    not TBGX_XML_Menu_IsLoaded           or
	    not TBGX_XML_LeaderBoard_IsLoaded
	    ) then

	    return false;
	else
		TBG_Files_Loaded = true;
		return true;
	end
end


-- ## Periodic check performed by Titan Panel.
function TBG_CheckEvents() end


-- ## Performed when the user changes zones.
function TBG_ZonesChanged()
	if (not TBG_IsEnabled()) then return nil; end

	local zone = GetRealZoneText();

	if (zone ~= TBG.Var.Player.zone) then
		TBG.Var.Player.zone = zone;

		if (TBG.Var.Player.zone ~= nil) then
			TBG.Var.BG.stat_information_stored   = false;
			TBG.Var.BG.onzone_raidscan_performed = false;

			-- Reset battleground data.
			HideUIPanel(WorldStateScoreFrame);
			TBG_Init_CreateActiveBattlegroundInformation();
			TBG_Init_CreateBattlegroundInformation();
			TBG_Init_CreateWorldState();
			TBG_Init_CreatePlayerList();

			TBG.DB.inactives.names = { };
			TBG.DB.inactives.list  = { };

			-- Clear all panels.
			TBG_Init_ResetPanels();

			TBG_AutoReset(TBG.Var.Const.AUTO_JOIN);
			TBG_AutoReset(TBG.Var.Const.AUTO_LEAVE);

			-- Reset zone data.
			TBG.Var.External.blizzard_manualshowbg = nil;
			TBG.Var.Display.display_which_tt       = "1";
			TBG.Var.BG.AB.timeinc_check            = false;
			TBG.Var.BG.final_bg_run_time           = 0;

			-- Determine what battleground points were capped when zoning.
			TBG_Battleground_MapCheckExistingPoints();

			-- Update minimap to current zone.
			SetMapToCurrentZone();
			if (BattlefieldMinimap_Update) then BattlefieldMinimap_Update(); end

			TBG_Init_AddonSetup();            -- Setup attributes of addon XML.
			WorldStateFrame_ToggleMinimap();  -- Check if the battleground minimap needs to be shown.
			TBG_Battleground_ItemsScan();     -- Scan inventory for battleground items.

			if (TBG_IsSupported(TBG.Var.Player.zone)) then
				if (TBG.DB.active.stats.bg.run_estimate == nil) then
					TBG.DB.active.stats.bg.run_estimate = GetTime();
				end

				TBG_LeaderBoard_Resize(); -- Reset leader board for this area.
			else
				TBG.DB.active.stats.bg.run_estimate   = nil;
				TBG.DB.active.stats.bg.close_estimate = nil;
			end

			TBG.Var.BG.scan_raid                = true;
			TBG.Var.Addon.ZONE_CHANGED_NEW_AREA = zone;

			-- Fire specific events to make sure they get updated.
			TBG_OnEvent("BATTLEFIELDS_SHOW");
			TBG_OnEvent("UPDATE_BATTLEFIELD_STATUS");
			TBG_OnEvent("UPDATE_WORLD_STATES");
			TBG_OnEvent("FRIENDLIST_UPDATE");
			TBG_OnEvent("RAID_ROSTER_UPDATE");

			TBG.Var.BG.onzone_raidscan_performed = true;
		end
	end
end


-- ######################
-- ## UPDATE FUNCTIONS ##
-- ######################


-- ## Performs periodic updates.
function TBG_OnUpdate()
	if (not TBG.Var.Addon.LOADED)        then return nil; end
	if (not TBG.Var.Addon.TITAN.scanned) then TBG_Settings_ToggleTitan(); end
	if (not TBG_IsEnabled())             then return nil; end

	if (TBG.Var.Addon.LOADED and TBG.Var.Addon.PLAYER_ENTERING_WORLD) then
		if (TBG.Var.Player.zone == nil) then
			TBG_Init_CreatePlayerInformation();
		end
	end

	-- Do not update if zone variables have not been reset.
	if (TBG.Var.Player.zone ~= nil and TBG.Var.Player.zone == TBG.Var.Addon.ZONE_CHANGED_NEW_AREA) then
		TBG_Tooltip_Update(); -- Update the tooltip.

		-- Update minimap flag icons.
		-- TBG_Debug_Battleground_MinimapPlaceFlag();
		TBG_Battleground_MinimapPlaceFlag();

		-- ## Performs update of countdown variables.
		if (GetTime() - TBG.Var.Addon.update_countdown_last >= TBG.Var.Addon.update_countdown_freq) then
			TBG.Var.Addon.update_countdown_last = GetTime();

			TBG_Update_Auto(TBG.Var.Const.AUTO_JOIN);
			TBG_Update_Auto(TBG.Var.Const.AUTO_LEAVE);
		end

		-- ## Performs update of battleground player information.
		if (GetTime() - TBG.Var.BG.update_last >= TBG.Var.BG.update_freq) then
			TBG.Var.BG.update_last = GetTime();
			TBG_LeaderBoard_GetPlayerData();

			if (TBGX_LeaderBoard:IsVisible()) then
				TBG_LeaderBoard_UpdatePlayers();
			end
		end

		TBG_Assist_UpdateTargets();

		-- When this function is finally created by Blizzard, hook it.
		if (RaidGroupButton_OnEnter and Hook_Blizzard_RaidGroupButton_OnEnter == nil) then
			Hook_Blizzard_RaidGroupButton_OnEnter = RaidGroupButton_OnEnter;
			RaidGroupButton_OnEnter               = TBG_Hooks_Functions_RaidGroupButton_OnEnter;
		end

		-- ## Perform update on display variables.
		if (GetTime() - TBG.Var.Addon.update_last >= TBG.Var.Addon.update_freq) then
			TBG.Var.Addon.update_last = GetTime();

			-- Update the stored battleground information.
			TBG_Update_QueueArrayInformation();
			TBG_Update_StaticDialogs();

			-- Determine if the information for capture points and flags needs to be updated.
			if (TBG_IsSupported(TBG.Var.Player.zone)) then
				if (TBG.Var.BG.onload_capture_points_stored) then
					TBG_Battleground_MapCheckPoints(TBG.Var.Player.zone, false);
				end

				TBG_Battleground_MapCheckFlags(TBG.Var.Player.zone);
			end

			-- Update non titan display of captures and flags.
			TBG_Battleground_Other_UpdateCaptures();
			TBG_Battleground_Other_UpdateMovementTimers();
			TBG_Battleground_Other_UpdateFlags();

			TBG_Arena_GetRaidTargets();
			TBG_Arena_Other_UpdateHostilePlayers();

			-- If required, scan the inventory.
			if (TBG.Var.BG.scan_inventory) then
				TBG_Battleground_ItemsScan();
				TBG.Var.BG.scan_inventory = false;
			end

			-- Update battlefield run time.
			TBG.DB.active.stats.bg.run = GetBattlefieldInstanceRunTime();
		end

		-- ## Plays the battleground queue sound when the player is in a queue.
		if (TBG_Saved("MGI_RepeatQueuedSound") and  TBG_IsQueued()) then
			if (GetTime() - TBG.Var.BG.queue_sound_last >= TBG.Var.BG.queue_sound_delay) then
				TBG.Var.BG.queue_sound_last = GetTime();
				PlaySound("PVPENTERQUEUE");
			end
		end

		-- ## Reset timers and trackers when the winner is announced.
		if (GetBattlefieldWinner() ~= nil) then
			if (TBG.Var.Player.zone == TBG.Text.Ref.BG_EOTS or TBG.Var.Player.zone == TBG.Text.Ref.BG_WSG) then
				TBG.DB.active.stats.bg.stats.alliance.player = "";
				TBG.DB.active.stats.bg.stats.horde.player    = "";

				TBGX_Display_MinimapFlags_1:Hide();
				TBGX_Display_MinimapFlags_2:Hide();

			elseif (TBG.Var.Player.zone == TBG.Text.Ref.BG_AB or TBG.Var.Player.zone == TBG.Text.Ref.BG_AV) then
				TBG.DB.active.captures = { };
			end
		end

		TBG_Battleground_Resources_CheckTimeUntilNodeIncrease();   -- Check on Arathi Basin and Eye of the Storm nodes.
		TBG_Battleground_HistoryUpdate();                          -- Check if the battleground has been won and if so, get information on who and so forth.
		TBG_ToggleDetachScoreFrame();                              -- Check that the score frame is being correctly handled by the addon.
	end

	-- ## Perform periodic garbage collection.
	if (TBG_Saved("MG_AllowGarbageCollection")) then
		if (collectgarbage("step", 300)) then collectgarbage("restart"); end
	end
end


-- ## Updates auto join and auto leave information.
function TBG_Update_Auto(type)
	if (type.name ~= "") then
		local bg = TBG.DB.bgs.names[type.name];

		if (TBG_IsArena(type.name)) then
			bg = bg.arenas[type.size];
		end

		if (bg.auto >= 1 and not type.paused) then
			bg.auto = bg.auto - 1;
		end
	end
end


-- ## Stores information on a battleground and performs any actions based on this information that require periodic checks.
function TBG_Update_QueueArrayInformation()
	if (TBG.Var.BG.update_id and TBG.Var.BG.update_id > 1000) then TBG.Var.BG.update_id = 0; end

	TBG.DB.bgs.queue = 0;

	-- Perform battlefield actions.
	for i = 1, MAX_BATTLEFIELD_QUEUES do
		local status, name, instance, _, _, team_size = GetBattlefieldStatus(i);

		-- If this battleground is active, collect information on this battleground.
		if (status == TBG.Var.Const.BG_STATUS_ACTIVE) then

			-- Update friend information.
			if (not TBG.Var.BG.friends_asked) then
				ShowFriends();
				TBG.Var.BG.friends_asked = true;
			end

			if (name and GetBattlefieldWinner() ~= nil) then

				-- If a battleground is finished, get ready to leave it.
				if (TBG_Saved("MGB_AutoLeave") and not TBG_AutoStarted(TBG.Var.Const.AUTO_LEAVE)) then
					TBG_AutoBegin(TBG.Var.Const.AUTO_LEAVE, name, team_size);
				end

				-- Auto join a battleground if not already in a battleground.
				if (TBG_Saved("MGB_AutoLeave") and TBG_IsSupported(TBG.Var.Player.zone) and TBG_AutoFinished(TBG.Var.Const.AUTO_LEAVE)) then
					TBG_AcceptBattlefieldPort(i, nil, true);
				end
			end

		-- Otherwise, if this battleground is waiting for confirmation, perform auto join.
		elseif (status == TBG.Var.Const.BG_STATUS_CONFIRM) then

			-- If a battleground is ready for confirmation, start the auto join timer.
			if (not TBG_AutoStarted(TBG.Var.Const.AUTO_JOIN) and TBG_IsAutoJoinAllowed(name)) then
				TBG_AutoBegin(TBG.Var.Const.AUTO_JOIN, name, team_size);
			end

			-- Auto join a battleground if not already in a battleground.
			if (TBG_IsAuto(TBG.Var.Const.AUTO_JOIN, name) and TBG_AutoFinished(TBG.Var.Const.AUTO_JOIN) and not TBG.Var.Const.AUTO_JOIN.paused) then
				TBG_AutoJoinInstance(i);
			end
		end

		-- If this is a battleground or arena, store status infromation.
		if (TBG_IsSupported(name)) then
			local a = TBG.DB.bgs.names[name];

			if (a.type == TBG.Var.Const.ARENA) then
				a = a.arenas[team_size];
			end

			-- Play in queue sound on queue.
			if (status == TBG.Var.Const.BG_STATUS_QUEUED and a.sound_queue == false and TBG_Saved("MGI_PlaySoundWhenQueue")) then
				if (TBG_Saved("MGI_RepeatQueuedSound") and GetTime() - TBG.Var.BG.queue_sound_last >= TBG.Var.BG.queue_sound_delay) then
				else
					TBG.Var.BG.queue_sound_last = GetTime();
					PlaySound("PVPENTERQUEUE");
				end

				a.sound_queue = true;
			end

			if (status == TBG.Var.Const.BG_STATUS_QUEUED or status == TBG.Var.Const.BG_STATUS_CONFIRM) then
				TBG.DB.bgs.queue = TBG.DB.bgs.queue + 1;
			end

			TBG_Update_SoundInstanceReady(a, status);

			if (TBG_Saved("MD_TT_WhenNoQueueHide") and TBG.Var.Display.tt_mouse_over_last > 0 and GetTime() - TBG.Var.Display.tt_mouse_over_last >= TBG_Saved("MD_TT_HideDelay")) then
				TBG_Tooltip_Load();
				TBG_Tooltip_AlwaysShow();
			end

			if (status ~= TBG.Var.Const.BG_STATUS_NONE) then
				a.index    = i;
				a.status   = status;
				a.instance = instance;
				a.update   = TBG.Var.BG.update_id;

				if (status == TBG.Var.Const.BG_STATUS_CONFIRM) then
					a.expires = GetBattlefieldPortExpiration(i) / 1000;
				else
					a.expires = 0;
				end

				TBG.DB.bgs.blizzard[i] = name;
			end

		-- Otherwise, this slot is not being used by blizzard.
		else
			if (TBG.DB.bgs.blizzard[i]) then
				TBG.DB.bgs.blizzard[i] = nil;
			end
		end
	end

	function TBG_ResetBattlegroundInformation(n, a)
		if (a.update ~= TBG.Var.BG.update_id) then
			a.index       = 0;
			a.status      = TBG.Var.Const.BG_STATUS_NONE;
			a.instance    = 0;

			a.update      = nil;
			a.expires     = 0;

			a.sound       = false;
			a.sound_timer = 0;
			a.sound_queue = false;
			a.auto        = 0;

			if (TBG_IsAuto(TBG.Var.Const.AUTO_JOIN, a.name, a.team_size)) then
				TBG_AutoReset(TBG.Var.Const.AUTO_JOIN);
			end
		end
	end


	-- Reset all the battlegrounds that were not updated.
	for i = 1, table.getn(TBG.Var.Const.BATTLEGROUNDS) do
		local n = TBG.Var.Const.BATTLEGROUNDS[i];

		TBG_ResetBattlegroundInformation(n, TBG.DB.bgs.names[n]);
	end

	-- Reset all of the arenas that were not updated.
	for i = 1, table.getn(TBG.Var.Const.ARENAS) do
		local n = TBG.Var.Const.ARENAS[i];

		for s = 1, table.getn(TBG.Var.Const.ARENA_SIZES) do
			local a = TBG.Var.Const.ARENA_SIZES[s];

			TBG_ResetBattlegroundInformation(n, TBG.DB.bgs.names[n].arenas[a]);
		end

		TBG.DB.bgs.names[n].queue = TBG_IsArenaActive(n);
	end

	if (TBG_Saved("MD_TT_WhenNoQueueHide") and TBG.Var.Display.tt_mouse_over_last > 0 and GetTime() - TBG.Var.Display.tt_mouse_over_last >= TBG_Saved("MD_TT_HideDelay")) then
		TBG_Tooltip_Load();
		TBG_Tooltip_AlwaysShow();
	end

	TBG.Var.BG.update_id = TBG.Var.BG.update_id + 1;
end


-- ## Plays sound when a battleground is ready to be entered.
function TBG_Update_SoundInstanceReady(a, status)

	-- Check if a new sound needs to be played.
	if (status == TBG.Var.Const.BG_STATUS_CONFIRM and a.status ~= status) then
		a.sound       = true;
		a.sound_timer = 0;

	elseif (status ~= TBG.Var.Const.BG_STATUS_CONFIRM) then
		a.sound       = false;
		a.sound_timer = 0;
	end

	-- Play a new sound for this battleground or arena.
	if (a.sound and TBG_Saved("MGI_PlayWhatSoundOnConfirm") > 0) then
		local PlayReadySound = function()
			if (TBG_Saved("MGI_PlayWhatSoundOnConfirm") == 1) then
				PlaySound("PVPTHROUGHQUEUE", true);
			else
				PlaySoundFile(TBG.Var.Const.SOUND_BGWARNING[TBG_Saved("MGI_PlayWhatSoundOnConfirm")].file);
			end
		end

		-- If repeat the sound periodically.
		if (TBG_Saved("MGI_RepeatConfirmSound")) then
			if (GetTime() - a.sound_timer >= TBG.Var.BG.bgready_sound_delay) then
				a.sound_timer = GetTime();

				PlayReadySound();
			end
		else
			PlayReadySound();
			a.sound = false;
		end
	end
end


-- ## Updates the display on static dialogs.
function TBG_Update_StaticDialogs()
	if (not TBG_Saved("MGI_HideJoinPopup")) then
		local i = 1;

		while (i <= table.getn(TBG.DB.popups)) do
			local bg = TBG.DB.bgs.names[TBG.DB.popups[i].text_arg1];

			if (TBG_Saved("MDW_Other_ModifyPopupQueueExpire") and TBG.DB.popups[i].which == "CONFIRM_BATTLEFIELD_ENTRY" and TBG.DB.popups[i].dialog:IsVisible() and bg) then
				if (bg.type == TBG.Var.Const.ARENA) then
					bg = bg.arenas[TBG.DB.popups[i].text_arg5];
				end

				if (bg.status == TBG.Var.Const.BG_STATUS_CONFIRM) then
					local expire = bg.expires;
					local name   = TBG.Var.Const.COLOR_WHITE .. TBG_GetInstanceName(bg) .. " " .. bg.instance .. TBG.Var.Const.COLOR_GREEN;

					-- Check if the queue expire time is to be shown.
					if (expire == 0) then
						expire = TBG.Text.Label.Tooltip.CLOSING;
					else
						expire = TBG_SecondsToTime(expire);
					end

					expire = " (" .. TBG.Var.Const.COLOR_RED .. expire .. TBG.Var.Const.COLOR_GREEN .. ")";

					-- If the user is auto joining.
					if (TBG_Saved("MGB_AutoJoin") and TBG_IsAuto(TBG.Var.Const.AUTO_JOIN, bg.name) and not TBG_AutoFinished(TBG.Var.Const.AUTO_JOIN)) then
						local autojoin = TBG_SecondsToTime(bg.auto);

						if (TBG.Var.Const.AUTO_JOIN.paused) then
							autojoin = autojoin .. TBG.Var.Const.COLOR_WHITE .. " (" .. TBG.Text.Label.Tooltip.PAUSED .. ")";
							getglobal(TBG.DB.popups[i].dialog:GetName().."Button2"):SetText(TBG.Text.Label.Tooltip.UNPAUSE);
						else
							getglobal(TBG.DB.popups[i].dialog:GetName().."Button2"):SetText(KEY_PAUSE);
						end

						TBG.DB.popups[i].text:SetText(TBG.Var.Const.COLOR_GREEN .. string.gsub(string.gsub(TBG.Text.Label.Tooltip.AUTOJOIN, "{bg}", name, 1), "{time}", TBG.Var.Const.COLOR_YELLOW .. autojoin .. TBG.Var.Const.COLOR_GREEN .. expire, 1));

					-- If the user is in a battleground or is not autojoining.
					else
						TBG.DB.popups[i].text:SetText(TBG.Var.Const.COLOR_GREEN .. string.gsub(string.gsub(TBG.Text.Label.Tooltip.CONFIRM_JOIN, "{bg}", name, 1), "{time}", TBG.Var.Const.COLOR_GREEN .. expire, 1));
					end
				end
			else
				TBG.DB.popups[i].text:SetText(TBG.Var.Const.COLOR_WHITE .. TBG.DB.popups[i].old_text);
				getglobal(TBG.DB.popups[i].dialog:GetName().."Button2"):SetText(TEXT(HIDE));

				TBG.DB.popups[i].dialog.added = false;

				table.remove(TBG.DB.popups, i);
				i = i - 1;
			end

			i = i + 1;
		end
	end
end

-- ## Collects world score frame information.
function TBG_Update_Score()
	local inInstance, instanceType = IsInInstance();
	local frames                   = 0;
	local score                    = { [1] = { }, [2] = { } };

	local texts                    = { [1] = { text = "", icon = "" }, [2] = { text = "", icon = "" }, };

	if ((WORLD_PVP_OBJECTIVES_DISPLAY == "1") or (WORLD_PVP_OBJECTIVES_DISPLAY == "2" and IsSubZonePVPPOI()) or (instanceType == "pvp")) then
		score.count = 0;

		for i = 1, GetNumWorldStateUI() do
			state, text, icon, dynamicIcon, tooltip, dynamicTooltip, extendedUI, extendedUIState1, extendedUIState2, extendedUIState3 = GetWorldStateUIInfo(i);

			if (state > 0) then
				if (extendedUI ~= "") then
					if (extendedUIState1 ~= TBG.Var.BG.point_score_current) then

						TBG.Var.BG.point_score_neutral = extendedUIState2;

						TBG.Var.BG.point_score_last         = TBG.Var.BG.point_score_current;
						TBG.Var.BG.point_score_last_time    = TBG.Var.BG.point_score_current_time;
						TBG.Var.BG.point_score_current      = extendedUIState1;
						TBG.Var.BG.point_score_current_time = time();

						TBG.Var.BG.point_score_updated = true;
					end

				else
					score.count               = score.count + 1;
					score[score.count].active = true;

					if (icon == nil) then
						score[score.count].faction = FACTION_OTHER;
					elseif (string.find(icon, "Alliance")) then
						score[score.count].faction = FACTION_ALLIANCE;
					elseif (string.find(icon, "Horde")) then
						score[score.count].faction = FACTION_HORDE;
					else
						score[score.count].faction = FACTION_OTHER;
					end

					if (string.find(icon, "Tower")) then
						score[score.count].type = TBG.Text.Label.Tooltip.SCORES_TOWERS;
					elseif (string.find(icon, "PVP") and TBG.Var.Player.zone ~= TBG.Text.Ref.BG_WSG) then
						score[score.count].type = TBG.Text.Label.Tooltip.SCORES_POINTS;
					elseif (string.find(icon, "Flag") or TBG.Var.Player.zone == TBG.Text.Ref.BG_WSG) then
						score[score.count].type = TBG.Text.Label.Tooltip.SCORES_FLAGS;
					end

					if (TBG_IsArena(TBG.Var.Player.zone)) then
						for digits in string.gmatch(text, "%d+") do
							score[score.count].number = digits;
							break;
						end

						score[score.count].text = string.trim(string.gsub(string.sub(text, string.find(text, ":") + 2, string.len(text)), "%d+", ""));
					elseif (TBG.Var.Player.zone == TBG.Text.Ref.BG_WSG) then
						local array = TBG_Split(text, "/");

						if (table.getn(array) >= 2) then
							text =         TBG.Var.Const.COLOR_WHITE .. array[1];
							text = text .. TBG.Var.Const.COLOR_YELLOW .. "/";
							text = text .. TBG.Var.Const.COLOR_YELLOW .. array[2];

							score[score.count].flags  = array[1];
							score[score.count].number = text;
						end

					elseif (TBG.Var.Player.zone == TBG.Text.Ref.BG_AB or TBG.Var.Player.zone == TBG.Text.Ref.BG_EOTS) then
						local array = TBG_Split(text, "%d+", false, true);

						if (table.getn(array) >= 6) then
							local bases     = TBG.Var.Const.COLOR_YELLOW .. array[1] .. TBG.Var.Const.COLOR_WHITE .. array[2];
							local resources = TBG.Var.Const.COLOR_YELLOW .. array[3] .. TBG.Var.Const.COLOR_WHITE .. array[4] .. array[5] .. array[6];

							score[score.count].bases     = array[2];
							score[score.count].resources = array[4];

							score[score.count].number = bases .. " " .. resources;
						end
					else
						if (string.find(text, "-")) then
							score[score.count].text   = string.sub(text, 1, string.find(text, "-") - 2);
							score[score.count].number = string.sub(text, string.find(text, "-") + 2, string.len(text));
						elseif (string.find(text, ":")) then
							score[score.count].text   = string.sub(text, 1, string.find(text, ":") - 1);
							score[score.count].number = string.sub(text, string.find(text, ":") + 2, string.len(text));
						else
							score[score.count].text = text;
						end
					end
				end
			end
		end
	end

	TBG.DB.world_state = score;
end


-- ######################
-- ## ASSIST FUNCTIONS ##
-- ######################


-- ## Loop through and draw the targets.
function TBG_Assist_UpdateTargets()
	if (not UnitInRaid("player")) then return nil; end

	local i = 1;

	for assist, value in pairs(TBG.DB.players.assists) do
		local frame = getglobal("TBGX_Target_" .. i);

		-- Create the frame if it doesn't exist.
		if (not frame) then
			frame = TBG_Assist_CreateFrame(i);
		end

		-- If this unit is no longer the assigned unit, then update the keys.
		if (UnitName(value.unit) ~= assist) then
			if (TBG.DB.players.names[assist .. "|" .. TBG.Var.Player.faction] and TBG.DB.players.names[assist .. "|" .. TBG.Var.Player.faction].raid > 0) then
				local raid = TBG.DB.players.names[assist .. "|" .. TBG.Var.Player.faction].raid;

				TBG.DB.players.assists[assist].unit   = "raid" .. raid;
				TBG.DB.players.assists[assist].target = "raid" .. raid .. "target";
				TBG.DB.players.assists[assist].show   = true;

			-- Unit no longer exists or cannot be found, do not display.
			else
				TBG.DB.players.assists[assist].show = false;
			end
		end

		-- Display only if it still exists and can be shown.
		if (TBG.DB.players.assists[assist] and TBG.DB.players.assists[assist].show ~= false and UnitClass(value.unit)) then
			local color = TBG.Var.Const.COLOR_YELLOW;

			if (TBG.Var.Const.NAME_CLASS[UnitClass(value.unit)]) then
				color = TBG.Var.Const.COLOR_CLASS[TBG.Var.Const.NAME_CLASS[UnitClass(value.unit)]].color;
			end

			getglobal("TBGX_Assist_" .. i .. "_ID"):SetText(color .. assist .. "'s");
			getglobal("TBGX_Assist_" .. i .. "_Text"):SetText(TBG.Var.Const.COLOR_GREY .. "Target");

			getglobal("TBGX_Assist_" .. i).name    = assist;
			getglobal("TBGX_Assist_" .. i).unit    = value.unit;
			getglobal("TBGX_Assist_" .. i).tooltip = color .. assist .. TBG.Var.Const.COLOR_YELLOW .. " (" .. TBG.Var.Const.COLOR_WHITE .. UnitClass(value.unit) .. TBG.Var.Const.COLOR_YELLOW .. ")";

			-- Draw this frame.
			TBG_Assist_DrawAssist(value.target, frame);
			getglobal("TBGX_Assist_" .. i):Show();

			i = i + 1;
		end
	end

	-- If there are any unused assists, hide them.
	while (true) do
		if (getglobal("TBGX_Assist_" .. i)) then
			getglobal("TBGX_Assist_" .. i):Hide();
		else
			break;
		end

		i = i + 1;
	end
end


-- ## Creates an assist target frame.
function TBG_Assist_CreateFrame(id)
	local assist = CreateFrame("Frame", "TBGX_Assist_" .. id, TBGX_Assist_Drag, "TBGX_Template_Frame_Assist");
	local frame  = CreateFrame("Frame", "TBGX_Target_" .. id, assist, "TBGX_Template_Frame_Target");

	frame:SetID(id);
	getglobal(frame:GetName() .. "_HPBG"):SetAlpha(0.22);
	getglobal(frame:GetName() .. "_MPBG"):SetAlpha(0.22);

	frame:SetBackdropColor(0, 0, 0, 1);
	frame:SetBackdropBorderColor(0.341, 0.341, 0.341, 1);

	assist:SetBackdropColor(0, 0, 0, 1);
	assist:SetBackdropBorderColor(0.341, 0.341, 0.341, 1);

	getglobal(assist:GetName() .. "_ID"):SetText(id);
	getglobal(frame:GetName() .. "_Click"):SetAttribute("type", "target");

	-- Connect this frame.
	if (id == 1) then
		assist:SetPoint("CENTER", TBGX_Assist_Drag, "CENTER", 0, 0);
	else
		assist:SetPoint("TOPRIGHT", getglobal("TBGX_Assist_" .. (id - 1)), "BOTTOMRIGHT", 0, 2);
	end

	frame:SetPoint("LEFT", assist, "RIGHT", 0, 0);

	return frame;
end


-- ## Draws a particular target in a particular group.
function TBG_Assist_DrawAssist(id, frame)
	if (UnitName(id) == nil or UnitName(id) == UNKNOWN) then
		frame:Hide();
	else
		local color        = "";
		local power        = UnitPowerType(id);
		local power_colors = {
			[0] = { 0, 0, 1, 1 }, -- Mana
			[1] = { 1, 0, 0, 1 }, -- Rage
			[2] = { 1, 1, 0, 1 }, -- Focus
			[3] = { 1, 1, 0, 1 }, -- Energy
			[4] = { 0, 1, 0, 1 }, -- Happiness
		};

		if (TBG.Var.Const.NAME_CLASS[UnitClass(id)]) then
			color = TBG.Var.Const.COLOR_CLASS[TBG.Var.Const.NAME_CLASS[UnitClass(id)]].color;
		end

		getglobal(frame:GetName() .. "_Click").unit = id;
		getglobal(frame:GetName() .. "_Click"):SetAttribute("unit", id);
		getglobal(frame:GetName() .. "_Name"):SetText(color .. UnitName(id));

		if (UnitLevel(id) > 0) then
			getglobal(frame:GetName() .. "_Level"):SetText(UnitLevel(id));
		else
			getglobal(frame:GetName() .. "_Level"):SetText("");
		end

		getglobal(frame:GetName() .. "_HPBar"):SetMinMaxValues(0, UnitHealthMax(id));
		getglobal(frame:GetName() .. "_MPBar"):SetMinMaxValues(0, UnitManaMax(id));

		getglobal(frame:GetName() .. "_HPBar"):SetValue(UnitHealth(id));
		getglobal(frame:GetName() .. "_MPBar"):SetValue(UnitMana(id));

		if (UnitIsDeadOrGhost(id)) then
			getglobal(frame:GetName() .. "_HPBar_Text"):SetText("Dead");
		else
			getglobal(frame:GetName() .. "_HPBar_Text"):SetText(UnitHealth(id));
		end

		getglobal(frame:GetName() .. "_MPBar"):SetStatusBarColor(power_colors[power][1], power_colors[power][2], power_colors[power][3], power_colors[power][4]);

		-- If the unit is not a player, mark as beast or pet.
		if (not UnitIsPlayer(id)) then
			getglobal(frame:GetName() .. "_Pet"):Show();
		else
			getglobal(frame:GetName() .. "_Pet"):Hide();
		end

		-- Determine if the unit is in combat.
		if (UnitAffectingCombat(id)) then
			getglobal(frame:GetName() .. "_Combat"):Show();
		else
			getglobal(frame:GetName() .. "_Combat"):Hide();
		end

		frame:Show();
	end
end


-- #####################
-- ## SCORE FUNCTIONS ##
-- #####################


-- ## Returns the score frame text for a specific battleground.
function TBG_Score_GetText()
	local score  = "";

	if (TBG.DB.world_state and TBG.DB.world_state[1] and TBG.DB.world_state[1].active) then
		for i = 1, TBG.DB.world_state.count do
			local color1  = TBG.Var.Const.COLOR_WHITE;
			local faction = TBG.DB.world_state[i].faction;

			-- If this is an areana.
			if (TBG_IsArena(TBG.Var.Player.zone)) then
				if (i == 1) then
					color1  = TBG.Var.Const.COLOR_YELLOW;
					faction = TBG.Text.Label.Tooltip.TEAM_GOLD;
				else
					color1  = TBG.Var.Const.COLOR_GREEN;
					faction = TBG.Text.Label.Tooltip.TEAM_GREEN;
				end

			-- Otherwise, set the colors for the factions.
			elseif (faction == FACTION_ALLIANCE) then
				color1 = TBG.Var.Const.COLOR_BLUE;
			elseif (faction == FACTION_HORDE) then
				color1 = TBG.Var.Const.COLOR_RED;
			end

			if (score ~= "") then
				score = score .. "\n";
			end

			if (TBG.DB.world_state[i].number) then
				score = score .. color1 .. faction;

				if (TBG.DB.world_state[i].type) then
					score = score .. TBG.Var.Const.COLOR_YELLOW .. " (";
					score = score .. TBG.Var.Const.COLOR_WHITE .. TBG.DB.world_state[i].type;
					score = score .. TBG.Var.Const.COLOR_YELLOW .. ") ";
				end

				score = score .. color1 .. ":\t";

				if (TBG.DB.world_state[i].text) then
					score = score .. TBG.Var.Const.COLOR_YELLOW .. TBG.DB.world_state[i].text .. " - ";
				end

				score = score .. TBG.Var.Const.COLOR_WHITE  .. TBG.DB.world_state[i].number;
			else
				score = score .. TBG.Var.Const.COLOR_YELLOW .. TBG.DB.world_state[i].text .. "\t";
				score = score .. color1 .. faction;
			end
		end

		return score;
	end

	return nil;
end


-- ## Returns the short score frame text for a specific battleground.
function TBG_Score_GetTextShort()
	local score  = "";

	if (TBG.DB.world_state and TBG.DB.world_state[1] and TBG.DB.world_state[1].active) then
		for i = 1, TBG.DB.world_state.count do
			local color1 = TBG.Var.Const.COLOR_WHITE;
			local faction = TBG.DB.world_state[i].faction;

			-- If this is an areana.
			if (TBG_IsArena(TBG.Var.Player.zone)) then
				if (i == 1) then
					color1  = TBG.Var.Const.COLOR_YELLOW;
					faction = TBG.Text.Label.Button.TEAM_GOLD;
				else
					color1  = TBG.Var.Const.COLOR_GREEN;
					faction = TBG.Text.Label.Button.TEAM_GREEN;
				end

			-- Otherwise, set the colors for the factions.
			elseif (faction == FACTION_ALLIANCE) then
				color1 = TBG.Var.Const.COLOR_BLUE;
			elseif (faction == FACTION_HORDE) then
				color1 = TBG.Var.Const.COLOR_RED;
			end

			if (score ~= "") then
				score = score .. TBG.Var.Const.COLOR_YELLOW .. ":";
			end

			if (TBG.Var.Player.zone == TBG.Text.Ref.BG_WSG) then
				score = score .. color1 .. TBG.DB.world_state[i].flags;
			elseif (TBG.Var.Player.zone == TBG.Text.Ref.BG_AB or TBG.Var.Player.zone == TBG.Text.Ref.BG_EOTS) then
				score = score .. color1 .. TBG.DB.world_state[i].bases .. "/" .. TBG.DB.world_state[1].resources;
			elseif (TBG.DB.world_state[i].number) then
				score = score .. color1 .. TBG.DB.world_state[i].number;
			else
				score = score .. color1 .. faction;
			end
		end

		return score;
	end

	return nil;
end


-- ################################
-- ## GOSSIP AND QUEST FUNCTIONS ##
-- ################################


-- ## Actions occuring when the gossip frame is opened.
function TBG_Gossip_OnShow()
	local i      = 1;
	local npc    = GossipFrameNpcNameText:GetText();
	local quests = false;

	while (true) do
		local button = getglobal("GossipTitleButton" .. i);

		if (button and button:IsVisible()) then
			local texture = getglobal("GossipTitleButton" .. i .. "GossipIcon"):GetTexture();
			local gossip  = false;

			-- If this is a battlemaster.
			if (string.find(texture, "battlemaster") and TBG_Saved("MGB_SkipBattlemasterGossip") and not TBG_IsModifierKeyDown()) then
				gossip = true;

			-- If this is a flight path.
			elseif (string.find(texture, "taxi") and TBG_Saved("MGB_SkipTaxiGossip") and not TBG_IsModifierKeyDown()) then
				gossip = true;

			-- If this is a vendor.
			elseif (string.find(texture, "vendor") and TBG_Saved("MGB_SkipVendorGossip") and not TBG_IsModifierKeyDown()) then
				gossip = true;

			-- If this is a bank.
			elseif (string.find(texture, "banker") and TBG_Saved("MGB_SkipBankGossip") and not TBG_IsModifierKeyDown()) then
				gossip = true;
			end

			-- If skipping this gossip.
			if (gossip) then
				SelectGossipOption(button:GetID());
				TBG_Tip_Generate("GOSSIP");
			end


			-- If this is a quest, check for auto turnins.
			if (string.find(texture, "QuestIcon") and TBG_Saved("MGB_AutoTurnInQuests")) then
				local quest   = button:GetText();
				local OnClick = button:GetScript("OnClick");

				quests = true;

				if (TBG_Saved("AutoTurnInQuests")[npc] ~= nil) then
					TBG_Saved("AutoTurnInQuests")[npc].gossip = true;
				end

				if (TBG_Saved("AutoTurnInQuests")[npc] ~= nil and TBG_Saved("AutoTurnInQuests")[npc].index ~= nil and TBG_Saved("AutoTurnInQuests")[npc].names ~= nil and TBG_Saved("AutoTurnInQuests")[npc].names[quest] ~= nil) then
					local id = TBG_Saved("AutoTurnInQuests")[npc].names[quest];

					if (TBG_Saved("AutoTurnInQuests")[npc].index[id] ~= nil) then
						TBG_Saved("AutoTurnInQuests")[npc].index[id].id = i;
					end
				end

				button:SetScript("OnClick", function()
					if (IsControlKeyDown() and TBG_Saved("MGB_AutoTurnInQuests")) then
						TBG_AutoComplete_Add(npc, quest);

					elseif (IsAltKeyDown() and TBG_Saved("MGB_AutoTurnInQuests")) then
						TBG_AutoComplete_Remove(npc, quest);

					else
						OnClick();
					end
				end);
			end
		else
			break;
		end

		i = i + 1;
	end

	if (quests) then
		TBG_Tip_Generate("QUEST_AUTO");
	end

	-- Check for auto turnins for this npc.
	if (not TBG_IsModifierKeyDown() and TBG_Saved("AutoTurnInQuests")[npc] ~= nil and TBG_Saved("AutoTurnInQuests")[npc].index ~= nil) then
		for i = 1, table.getn(TBG_Saved("AutoTurnInQuests")[npc].index) do
			local quest     = TBG_Saved("AutoTurnInQuests")[npc].index[i];
			local available = true;
			local skip      = false;

			if (TBG_Saved("AutoTurnInQuests")[npc].index[i].no_items == true and TBG_Saved("AutoTurnInQuests")[npc].no_items_last_quest ~= nil) then
				if (TBG_Saved("AutoTurnInQuests")[npc].no_items_last_quest <= i) then
					skip = true;
				end

				if (TBG_Saved("AutoTurnInQuests")[npc].no_items_last_quest == table.getn(TBG_Saved("AutoTurnInQuests")[npc].index)) then
					available = false;
				end
			end

			if (not skip) then
				if (TBG_Saved("AutoTurnInQuests")[npc].index[i].items ~= nil) then
					if (TBG_Saved("AutoTurnInQuests")[npc].items_last_quest ~= nil and TBG_Saved("AutoTurnInQuests")[npc].items_last_quest <= i) then
						available = false;
					end

					if (available) then

						-- Loop through the items and make sure the player has all of them. If not, skip.
						for x = 1, table.getn(TBG_Saved("AutoTurnInQuests")[npc].index[i].items) do
							local item = TBG_Saved("AutoTurnInQuests")[npc].index[i].items[x];

							if (TBG.DB.inventory.all[item.name] == nil or TBG.DB.inventory.all[item.name] < item.count) then
								available = false;
							end
						end
					end
				end

				if (available) then
					SelectGossipAvailableQuest(quest.id);
					SelectGossipActiveQuest(quest.id);
				end
			end

			if (not available and i == table.getn(TBG_Saved("AutoTurnInQuests")[npc].index)) then
				TBG_Quest_AnnounceInvalidComplete();
			end
		end

		TBG_Saved("AutoTurnInQuests")[npc].no_items_last_quest = nil;
		TBG_Saved("AutoTurnInQuests")[npc].items_last_quest    = nil;
	end
end


-- ## Shows a message indicating that the quest can no longer be automatically completed.
function TBG_Quest_AnnounceInvalidComplete()
	if (TBG_Saved("MGB_AutoTurnInQuests_Warning")) then
		RaidWarningFrame:AddMessage(TBG.Text.Message.DIALOG_AUTO_TURNIN_COMPLETE, 1, 0, 0, 1, 0);
		PlaySound("igQuestFailed");
	end
end


-- ## Has a chance to generate a random tip and place it in the chat log, based on a specific user action.
function TBG_Tip_Generate(action)
	if (not TBG_Saved("MG_ShowTips")) then return nil; end

	local disable = math.random(100);

	-- Chance to display how to disable tips.
	if (disable <= 10) then
		TBG_Print_Tip(TBG.Text.Message.TIP_DISABLE_TIPS);

	else
		if (action == "GOSSIP") then
			local gossip = math.random(100);

			if (gossip <= 50) then
				TBG_Print_Tip(TBG.Text.Message.TIP_GOSSIP_SKIP_BYPASS);
			else
				TBG_Print_Tip(TBG.Text.Message.TIP_GOSSIP_SETTINGS);
			end

		elseif (action == "QUEUE_AUTO") then
			TBG_Print_Tip(TBG.Text.Message.TIP_QUEUE_AUTO);

		elseif (action == "QUEST_AUTO") then
			local quest_auto = math.random(100);

			if (quest_auto <= 25) then
				TBG_Print_Tip(TBG.Text.Message.TIP_QUEST_AUTO_ADD);
			elseif (quest_auto <= 50) then
				TBG_Print_Tip(TBG.Text.Message.TIP_QUEST_AUTO_REMOVE);
			elseif (quest_auto <= 75) then
				TBG_Print_Tip(TBG.Text.Message.TIP_QUEST_AUTO_ORDER);
			else
				TBG_Print_Tip(TBG.Text.Message.TIP_QUEST_AUTO_SKIP);
			end

		elseif (action == "BG_MESSAGE_REDIRECT") then
			local bg_message_redirect = math.random(100);

			if (bg_message_redirect <= 50) then
				TBG_Print_Tip(TBG.Text.Message.TIP_BG_MESSAGES_REDIRECT);
			end
		elseif (action == "ASSIST") then
			TBG_Print_Tip(TBG.Text.Message.TIP_RAID_ASSISTS);
		end
	end
end


-- ######################
-- ## ACTION FUNCTIONS ##
-- ######################


-- ## Events that occur when the user clicks on the addon button in the Tital Panel.
function TBG_OnClick(button)
	if (not TBG.Var.Addon.LOADED) then return nil; end
	if (not TBG_IsEnabled())      then return nil; end

	if (button == "LeftButton") then
		CloseDropDownMenus();

		-- Check on auto join pauses.
		-- If the user want to pause the auto join timer.
		if (TBG_Saved("MGB_AutoJoin") and not TBG.Var.Const.AUTO_JOIN.paused) then
			TBG.Var.Const.AUTO_JOIN.paused = true;

		-- If the user wants to unpause the auto join timer.
		elseif (TBG_Saved("MGB_AutoJoin") and TBG.Var.Const.AUTO_JOIN.paused) then
			TBG.Var.Const.AUTO_JOIN.paused = false;
		end

		-- Check on auto leave pauses.
		-- If the user want to pause the auto leave timer.
		if (TBG_Saved("MGB_AutoLeave") and not TBG.Var.Const.AUTO_LEAVE.paused) then
			TBG.Var.Const.AUTO_LEAVE.paused = true;

		-- If the user wants to unpause the auto leave timer.
		elseif (TBG_Saved("MGB_AutoLeave") and TBG.Var.Const.AUTO_LEAVE.paused) then
			TBG.Var.Const.AUTO_LEAVE.paused = false;
		end

		-- Show the battlemap.
		if (IsShiftKeyDown() and not IsControlKeyDown() and not IsAltKeyDown()) then
			if ((TBG_Saved("MGI_AutoShowMinimap") and BattlefieldMinimap:IsShown()) or (not TBG_IsSupported(TBG.Var.Player.zone) and BattlefieldMinimap:IsShown())) then
				TBG.Var.External.blizzard_manualshowbg = "HIDE";
			elseif ((not TBG_Saved("MGI_AutoShowMinimap") and not BattlefieldMinimap:IsShown()) or (not TBG_IsSupported(TBG.Var.Player.zone) and not BattlefieldMinimap:IsShown())) then
				TBG.Var.External.blizzard_manualshowbg = "SHOW";
			else
				TBG.Var.External.blizzard_manualshowbg = nil;
			end

			WorldStateFrame_ToggleMinimap();
		end

		-- Switch tooltip views if necessary.
		if (not IsShiftKeyDown() and not IsAltKeyDown() and IsControlKeyDown()) then
			if (TBG.Var.Display.display_which_tt ~= "1") then
				TBG.Var.Display.display_which_tt = "1";
			else
				TBG.Var.Display.display_which_tt = "2";
			end

			TBG_Display_SetText(TBG.Var.Const.ADDON_ID);
		end

		-- Show battleground history if necessary.
		if (not IsShiftKeyDown() and not IsControlKeyDown() and IsAltKeyDown()) then
			if (TBG.Var.Display.display_which_tt ~= "HISTORY") then
				TBG.Var.Display.display_which_tt = "HISTORY";
			else
				TBG.Var.Display.display_which_tt = "1";
			end

			TBG_Display_SetText(TBG.Var.Const.ADDON_ID);
		end

		-- If the user is in a battleground.
		if (TBG_IsSupported(TBG.Var.Player.zone) and not TBG_AutoStarted(TBG.Var.Const.AUTO_JOIN) and not TBG_AutoStarted(TBG.Var.Const.AUTO_LEAVE) and (GetBattlefieldWinner() == nil or not WorldStateScoreFrame:IsVisible()) and not IsControlKeyDown() and not IsShiftKeyDown() and not IsAltKeyDown()) then
			ToggleWorldStateScoreFrame();

			TBG_Tooltip_Hide();
		end

		TBG.Var.External.titan_titanbg_leftclick = true;

	-- Call the right click menu.
	elseif (button == "RightButton") then

		-- If this is the draggable button.
		if (this:GetID() == 1) then
			ToggleDropDownMenu(1, nil, TBGX_RightClickMenu, this:GetName(), 0, 2);

		-- If this is the Titan Panel button.
		elseif (this:GetID() == 2) then
			ToggleDropDownMenu(1, nil, TBGX_RightClickMenu, this:GetName(), 0, -2);
		end

		GameTooltip:Hide();
	end
end


-- ## Determines if the join or leave is already in progress.
function TBG_AutoStarted(type)
	if (type.name == "") then
		return false;
	else
		return true;
	end
end


-- ## Determines if the timer has finished counting.
function TBG_AutoFinished(type)
	if (type.name ~= "") then
		local bg = TBG.DB.bgs.names[type.name];

		if (TBG_IsArena(type.name)) then
			bg = bg.arenas[type.size];
		end

		if (bg.auto == 0) then
			return true;
		else
			return false;
		end
	end
end


-- ## Begins join or leave on a specific instance.
function TBG_AutoBegin(type, name, size)
	local bg = TBG.DB.bgs.names[name];

	if (TBG_IsArena(name)) then
		bg = bg.arenas[size];
	end

	type.name   = name;
	type.size   = size;
	type.paused = false;

	bg.auto = TBG_Saved(type.var);
end


-- ## Resets join or leave.
function TBG_AutoReset(type)
	type.name   = "";
	type.paused = false;
end


-- ## Automatically joins a battleground.
function TBG_AutoJoinInstance(i)
	if (TBG_Saved("MGB_AutoJoin")) then
		TBG_AutoReset(TBG.Var.Const.AUTO_JOIN);
		TBG_AcceptBattlefieldPort(i, 1, true);
	end
end


-- ## Ports a user into the battleground, or leaves the queue.
function TBG_AcceptBattlefieldPort(index, accept, titanbg)

	-- Check if called by a blizzard drop down menu functions.
	if (this.arg1) then
		index   = this.arg1;
		accept  = this.arg2;
		titanbg = true;
	end

	-- If called by a Blizzard or a TitanBG function, then perform this action.
	if (TBG.Var.External.blizzard_interactbg or titanbg) then
		TitanPanelRightClickMenu_Close();
		AcceptBattlefieldPort(index, accept);
	end
end


-- ## Hides the battleground ready popup.
function TBG_HideBattlegroundReadyPopup()
	if (TBG_Saved("MGI_HideJoinPopup") and TBG_IsEnabled()) then
		StaticPopup_Hide("CONFIRM_BATTLEFIELD_ENTRY");
	end
end


-- ## Adds a quest to automatic completion.
function TBG_AutoComplete_Add(npc, quest)
	if (TBG_Saved("AutoTurnInQuests")[npc] == nil)       then TBG_Saved("AutoTurnInQuests")[npc]       = { }; end
	if (TBG_Saved("AutoTurnInQuests")[npc].index == nil) then TBG_Saved("AutoTurnInQuests")[npc].index = { }; end
	if (TBG_Saved("AutoTurnInQuests")[npc].names == nil) then TBG_Saved("AutoTurnInQuests")[npc].names = { }; end

	if (TBG_Saved("AutoTurnInQuests")[npc].names[quest] == nil) then
		local array         = { };
		      array.name    = quest;
		      array.scanned = false;

		if (this:GetID() ~= nil) then
			array.id = this:GetID();
		end

		table.insert(TBG_Saved("AutoTurnInQuests")[npc].index, array);
		TBG_Saved("AutoTurnInQuests")[npc].names[quest] = table.getn(TBG_Saved("AutoTurnInQuests")[npc].index);
	end
end


-- ## Removes a quest to automatic completion.
function TBG_AutoComplete_Remove(npc, quest)
	if (TBG_Saved("AutoTurnInQuests")[npc] ~= nil and TBG_Saved("AutoTurnInQuests")[npc].names ~= nil and TBG_Saved("AutoTurnInQuests")[npc].index ~= nil and TBG_Saved("AutoTurnInQuests")[npc].names[quest] ~= nil) then
		TBG_Saved("AutoTurnInQuests")[npc].names[quest] = nil;

		for i = 1, table.getn(TBG_Saved("AutoTurnInQuests")[npc].index) do
			if (TBG_Saved("AutoTurnInQuests")[npc].index[i].name == quest) then
				table.remove(TBG_Saved("AutoTurnInQuests")[npc].index, i);
				break;
			end
		end
	end
end


-- ##################################
-- ## TOOLTIP AND BUTTON FUNCTIONS ##
-- ##################################


-- ## Creates the text located on the Titan Panel bar.
function TBG_Button_SetText(id)
	TBG_CheckEvents();

	local text = ""; -- The tooltip text.

	local active = ""; -- Active battleground data.
	local ready  = ""; -- Ready battleground data.
	local queue  = ""; -- Queue battleground data.
	local score = "";  -- World score frame text.

	-- Find world score frame text.
	if (TBG_Saved("MDW_Button_DisplayScore")) then
		local s = TBG_Score_GetTextShort();

		if (s) then
			score = TBG.Var.Const.COLOR_YELLOW .. "[";
			score = score .. s;
			score = score .. TBG.Var.Const.COLOR_YELLOW .. ")" .. TBG.Var.Const.COLOR_WHITE;
		end
	end

	local spacer = TBG.Var.Const.COLOR_BLUE .. TBG.Var.Const.ICON_TITANBUTTON_SPACER;

	-- If the user wants to display the label text, display it.
	if (TBG_Saved("M_ShowButtonText")) then

		function TBG_Button_GetBattlegroundInfo(info)
			if (info.status ~= TBG.Var.Const.BG_STATUS_NONE) then

				-- Get active battleground text.
				if (info.status == TBG.Var.Const.BG_STATUS_ACTIVE) then
					active = active .. TBG_Button_GetText(info);

				-- Ready battleground data.
				elseif (info.status == TBG.Var.Const.BG_STATUS_CONFIRM) then
					if (ready ~= "") then ready = ready .. "   "; end

					ready = ready .. TBG_Button_GetText(info);

				-- Queue data.
				else
					if (queue ~= "") then queue = queue .. "   "; end

					queue = queue .. TBG_Button_GetText(info);
				end
			end

		end

		-- Loop through and process all the battlegrounds.
		for i = 1, table.getn(TBG.Var.Const.BATTLEGROUNDS) do
			TBG_Button_GetBattlegroundInfo(TBG.DB.bgs.names[TBG.Var.Const.BATTLEGROUNDS[i]]);
		end

		for i = 1, table.getn(TBG.Var.Const.ARENAS) do
			local n = TBG.DB.bgs.names[TBG.Var.Const.ARENAS[i]];

			for s = 1, table.getn(TBG.Var.Const.ARENA_SIZES) do
				local a = TBG.Var.Const.ARENA_SIZES[s];

				TBG_Button_GetBattlegroundInfo(TBG.DB.bgs.names[TBG.Var.Const.ARENAS[i]].arenas[a]);
			end
		end

		-- If this is a battleground, don't display generic queue information.
		if (TBG_IsSupported(TBG.Var.Player.zone)) then
			queue = "";

			-- If a queue is ready and the user wants the active information hidden.
			if (TBG_Saved("MDB_Button_ReplaceActiveWhenConfirm") and ready ~= "") then
				active = "";
			end
		end

		if (active ~= "" and ready ~= "")                                 then ready = "   " .. ready; end
		if ((active ~= "" or ready ~= "") and queue ~= "")                then queue = "   " .. queue; end
		if ((active ~= "" or ready ~= "" or queue ~= "") and score ~= "") then score = score .. " - "; end

		if (TBG_Saved("MD_ShowSpacers")) then
			text = (spacer .. " " .. score .. active .. ready .. queue .. " " .. spacer);
		else
			text = (score .. active .. ready .. queue);
		end

		-- If nothing is going on, show place holder.
		if (score == "" and active == "" and ready == "" and queue == "") then
			text = TBG.Var.Const.COLOR_YELLOW .. TBG.Text.Menu.HEADER;
		end
	end

	-- Return the button text.
	return text;
end


-- ## Forms the text printed on the Titan Panel button.
function TBG_Button_GetText(bg)
	local text      = "";
	local short     = TBG.Var.Const.COLOR_GREEN .. TBG_GetInstanceName(bg, true);

	-- If the user is queued for a BG.
	if (bg.status == TBG.Var.Const.BG_STATUS_QUEUED) then

		-- If the user wants to go into the first BG available.
		if (bg.instance == 0) then
			text = short .. ": " .. TBG.Var.Const.COLOR_WHITE .. TBG.Text.Label.Button.FIRST_AVAILABLE;

		-- If the user wants to go into a specific bg.
		else
			text = short .. ": " .. TBG.Var.Const.COLOR_WHITE .. bg.instance;
		end

		-- If the user wants to display the time in the queue and wait time, show it.
		if (TBG_Saved("MDW_Button_TimeInQueue")) then

			local wait_time = GetBattlefieldEstimatedWaitTime(bg.index);
			local in_queue  = TBG_SecondsToTimeAbbrev(GetBattlefieldTimeWaited(bg.index) / 1000);

			-- Unavailable.
			if (wait_time == 0) then
				wait_time = TBG.Var.Const.COLOR_RED .. "?";

			-- Time.
			else
				wait_time = TBG_SecondsToTimeAbbrev(wait_time / 1000, 1);
			end

			text = text .. TBG.Var.Const.COLOR_YELLOW .. " [" .. TBG.Var.Const.COLOR_WHITE .. in_queue .. TBG.Var.Const.COLOR_YELLOW .. "/" .. wait_time .. TBG.Var.Const.COLOR_YELLOW ..")";
		end

	-- If the user can join a battleground on confirmation.
	elseif (bg.status == TBG.Var.Const.BG_STATUS_CONFIRM) then

		-- If auto joining and not currently in a battleground.
		if (TBG_Saved("MGB_AutoJoin") and TBG_IsAuto(TBG.Var.Const.AUTO_JOIN, bg.name) and not TBG_AutoFinished(TBG.Var.Const.AUTO_JOIN)) then
			text = short .. " " .. bg.instance ..": " .. TBG.Var.Const.COLOR_GREEN .. TBG_SecondsToTime(bg.auto);

			if (TBG.Var.Const.AUTO_JOIN.paused) then
				text = text .. TBG.Var.Const.COLOR_WHITE .. " (" .. TBG.Var.Const.COLOR_GREEN .. TBG.Text.Label.Button.PAUSED .. TBG.Var.Const.COLOR_WHITE .. ")";
			end

		-- If currently in a battleground or not auto joining.
		else
			text = short .. " " .. bg.instance ..": " .. TBG.Var.Const.COLOR_GREEN .. TBG.Text.Label.Button.READY;
		end

		if (TBG_Saved("MDW_Button_TimeBeforeQueueExpires")) then
			local bg_expires = "";

			if (bg.expires == 0) then
				bg_expires = TBG.Text.Label.Tooltip.CLOSING;
			else
				bg_expires = TBG_SecondsToTime(bg.expires);
			end

			text = text .. TBG.Var.Const.COLOR_YELLOW .." (" .. TBG.Var.Const.COLOR_RED .. bg_expires .. TBG.Var.Const.COLOR_YELLOW ..")";
		end

	-- If the user is currently in a battleground.
	elseif (bg.status == TBG.Var.Const.BG_STATUS_ACTIVE) then
		local id    = TBG.Var.Player.name .. "|" .. TBG.Var.Player.faction;
		local total = TBG.DB.players.alliance + TBG.DB.players.horde;
		      text  = short .. " " .. bg.instance;

		if (TBG_IsBattleground(TBG.Var.Player.zone)) then

			-- Print the number of players in the battleground.
			if (TBG_Saved("MDB_Button_NumberOfPlayers") and total > 0) then
				text = text .. TBG.Var.Const.COLOR_WHITE .. "  (".. TBG.Var.Const.COLOR_BLUE .. TBG.DB.players.alliance .. TBG.Var.Const.COLOR_WHITE .. ":" .. TBG.Var.Const.COLOR_RED .. TBG.DB.players.horde .. TBG.Var.Const.COLOR_WHITE .. ")";
			end

			-- Print top server percentage.
			if (TBG_Saved("MDB_Button_TopServerPercentage") and (not TBG_Saved("MD_DisablePercentInAV") or (TBG_Saved("MD_DisablePercentInAV") and TBG.Var.Player.zone ~= TBG.Text.Ref.BG_AV))) then
				local enemy   = TBG.DB.active.stats.bg.servers[TBG.Var.Player.enemy_faction];
				local percent = TBG.Var.Const.COLOR_FACTION[TBG.Var.Player.enemy_faction];

				if (enemy and enemy.index and enemy.index[1]) then
					local total = TBG.DB.players.alliance;

					if (TBG.Var.Player.enemy_faction == FACTION_HORDE) then
						total = TBG.DB.players.horde;
					end

					local p, n = TBG_MathPercent(enemy.reference[enemy.index[1]], total);

					if (TBG_Saved("MDB_Button_ServerPercentageColor")) then
						if (n >= 76) then
							percent = TBG.Var.Const.COLOR_RED;
						elseif (n >= 51) then
							percent = TBG.Var.Const.COLOR_ORANGE;
						elseif (n >= 26) then
							percent = TBG.Var.Const.COLOR_YELLOW;
						else
							percent = TBG.Var.Const.COLOR_GREEN;
						end
					end

					percent = percent .. p;
					text    = text .. TBG.Var.Const.COLOR_YELLOW .. " - " .. TBG.Var.Const.COLOR_WHITE .. percent;
				end
			end
		end

		text = text .. TBG.Var.Const.COLOR_YELLOW .. ":   ";

		-- If the user wants to display the amount of time before the battle begins.
		if (TBG.DB.active.stats.bg.run ~= nil and TBG.DB.active.stats.bg.run ~= "" and TBG.DB.active.stats.bg.run > 0 and (TBG.DB.active.stats.bg.run / 1000) < bg.start_delay and TBG_Saved("MDB_Button_TimeToStart")) then
			text = text .. string.gsub(TBG.Text.Label.Button.START, "{time}", TBG.Var.Const.COLOR_WHITE .. TBG_SecondsToTime(bg.start_delay - (TBG.DB.active.stats.bg.run / 1000)) .. TBG.Var.Const.COLOR_YELLOW, 1);
		else
			if (TBG.DB.players.names[id]) then

				-- If this is an arena.
				if (TBG_IsArena(bg.name)) then
					if (TBG.DB.world_state[1] and TBG.DB.world_state[2] and TBG.DB.world_state[1].number and TBG.DB.world_state[2].number) then
						text = text .. TBG.Var.Const.COLOR_GREEN  .. TBG.Text.Label.Button.TEAM_GREEN .. TBG.Var.Const.COLOR_WHITE .. TBG.DB.world_state[1].number .. "   ";
						text = text .. TBG.Var.Const.COLOR_YELLOW .. TBG.Text.Label.Button.TEAM_GOLD  .. TBG.Var.Const.COLOR_WHITE .. TBG.DB.world_state[2].number;
					end
				end

				-- If this is a battleground.
				if (TBG_IsBattleground(bg.name)) then
					if (TBG_Saved("MDB_Button_Standing") and TBG.DB.players.names[id].standing ~= nil) then
						text = text .. TBG.Var.Const.COLOR_YELLOW .. TBG.Text.Label.Button.STANDING .. TBG.Var.Const.COLOR_WHITE .. TBG.DB.players.names[id].standing .. " ";
					end

					if (TBG_Saved("MDB_Button_KillingBlows") and TBG.DB.players.names[id].killing_blows ~= nil) then
						text = text .. TBG.Var.Const.COLOR_YELLOW .. TBG.Text.Label.Button.KILLING_BLOWS .. TBG.Var.Const.COLOR_GREEN .. TBG.DB.players.names[id].killing_blows .. " ";
					end

					if (TBG_Saved("MDB_Button_Kills") and TBG.DB.players.names[id].kills ~= nil) then
						text = text .. TBG.Var.Const.COLOR_YELLOW .. TBG.Text.Label.Button.KILLS .. TBG.Var.Const.COLOR_GREEN .. TBG.DB.players.names[id].kills .. " ";
					end

					if (TBG_Saved("MDB_Button_Deaths") and TBG.DB.players.names[id].deaths ~= nil) then
						text = text .. TBG.Var.Const.COLOR_YELLOW .. TBG.Text.Label.Button.DEATHS .. TBG.Var.Const.COLOR_RED .. TBG.DB.players.names[id].deaths .. " ";
					end

					if (TBG_Saved("MDB_Button_DamageDone") and TBG.DB.players.names[id].damage ~= nil) then
						text = text .. TBG.Var.Const.COLOR_YELLOW .. TBG.Text.Label.Button.DAMAGE_DONE .. TBG.Var.Const.COLOR_RED .. TBG.DB.players.names[id].damage .. " ";
					end

					if (TBG_Saved("MDB_Button_HealingDone") and TBG.DB.players.names[id].healing ~= nil) then
						text = text .. TBG.Var.Const.COLOR_YELLOW .. TBG.Text.Label.Button.HEALING_DONE .. TBG.Var.Const.COLOR_GREEN .. TBG.DB.players.names[id].healing .. " ";
					end

					if (TBG_Saved("MDB_Button_Honor") and TBG.DB.players.names[id].honor ~= nil) then
						text = text .. TBG.Var.Const.COLOR_YELLOW .. TBG.Text.Label.Button.HONOR .. TBG.Var.Const.COLOR_WHITE .. TBG.DB.players.names[id].honor .. " ";
					end
				end
			end

			if (TBG_IsArena(TBG.Var.Player.zone)) then
				text = text .. TBG.Var.Const.COLOR_WHITE .. "(" .. TBG_Score_GetTextShort() .. TBG.Var.Const.COLOR_WHITE .. ")";
			end

			-- If the player is auto leaving the battleground.
			if (TBG_Saved("MGB_AutoLeave") and TBG_IsAuto(TBG.Var.Const.AUTO_LEAVE, bg.name) and not TBG_AutoFinished(TBG.Var.Const.AUTO_LEAVE)) then
				text = short .. " " .. bg.instance ..": " .. TBG.Var.Const.COLOR_RED .. TBG_SecondsToTime(bg.auto);

				if (TBG.Var.Const.AUTO_LEAVE.paused) then
					text = text .. TBG.Var.Const.COLOR_WHITE .. " (" .. TBG.Var.Const.COLOR_RED .. TBG.Text.Label.Button.PAUSED .. TBG.Var.Const.COLOR_WHITE .. ")";
				end
			end
		end
	end

	return text;
end


-- ## Creates the text located on the Titan Panel bar.
function TBG_Display_SetText(id)
	local text     = ""; -- The tooltip text.
	local gathered = "";

	local active  = "";                     -- Active battleground data.
	local ready   = "";                     -- Ready battleground data.
	local queue   = "";                     -- Queue battleground data.
	local empty   = "";                     -- Irrelevant battleground data.
	local hint    = TBG_Display_GetHints(); -- Any hints to add to the end.
	local score   = "";                     -- World score frame text.

	-- Show primary tooltips.
	if (TBG.Var.Display.display_which_tt ~= "HISTORY") then

		function TBG_Display_GetBattlegroundInfo(info)

			-- No battleground interaction.
			if (info.status == TBG.Var.Const.BG_STATUS_NONE) then
				if (TBG_Saved("MDW_TT_DisplayNotInQueueText")) then
					if (info.type == TBG.Var.Const.ARENA and info.reference == 1) then
					elseif (info.type == TBG.Var.Const.ARENA and TBG.DB.bgs.names[TBG.Var.Const.ARENAS[1]].queue) then
					elseif (info.type == TBG.Var.Const.ARENA and TBG.DB.bgs.names[info.name].queue) then
					else
						if (empty ~= "") then empty = empty .. "\n"; end

						empty = empty .. TBG_Display_GetText(info);
					end
				end

			-- Active battleground data.
			elseif (info.status == TBG.Var.Const.BG_STATUS_ACTIVE) then
				if (active ~= "") then active = active .. "\n\n"; end

				active = active .. TBG_Display_GetText(info);

			-- Ready battleground data.
			elseif (info.status == TBG.Var.Const.BG_STATUS_CONFIRM) then
				if (ready ~= "") then ready = ready .. "\n"; end

				ready = ready .. TBG_Display_GetText(info);

			-- Queue data.
			else
				if (queue ~= "") then queue = queue .. "\n\n"; end

				queue = queue .. TBG_Display_GetText(info);
			end
		end

		-- Loop through and process all the battlegrounds.
		for i = 1, table.getn(TBG.Var.Const.BATTLEGROUNDS) do
			TBG_Display_GetBattlegroundInfo(TBG.DB.bgs.names[TBG.Var.Const.BATTLEGROUNDS[i]])
		end

		-- Loop through and process all of the arenas.
		for i = 1, table.getn(TBG.Var.Const.ARENAS) do
			local n = TBG.DB.bgs.names[TBG.Var.Const.ARENAS[i]];

			-- Display no queue message for only one size for the battleground arena.
			if (not n.queue) then
				TBG_Display_GetBattlegroundInfo(n.arenas[TBG.Var.Const.ARENA_SIZES[1]]);
			else
				for s = 1, table.getn(TBG.Var.Const.ARENA_SIZES) do
					local a = TBG.Var.Const.ARENA_SIZES[s];

					TBG_Display_GetBattlegroundInfo(n.arenas[a]);
				end
			end
		end

		-- Find world score frame text.
		if (TBG_Saved("MDW_TT_DisplayScore")) then
			local score = "";
			local s     = TBG_Score_GetText();

			if (s) then
				if (text ~= "") then score = "\n"; end
				score = score .. s;
			end

			if (score ~= "") then
				text = score .. text;
			end
		end

		-- If the player wants to show it and it is known, show what battleground weekend it will be.
		if (TBG_Saved("MDB_TT_BattlegroundWeekend") and TBG.Var.BG.battleground_weekend ~= UNKNOWN) then
			local str  = TBG.Var.Const.COLOR_WHITE .. string.gsub(TBG.Text.Label.Tooltip.BATTLEGROUND_WEEKEND, "{bg}", TBG.Var.Const.COLOR_GREEN .. TBG.Var.BG.battleground_weekend .. TBG.Var.Const.COLOR_WHITE, 1);

			if (text ~= "") then text = text .. "\n\n"; end
			text = text .. str;
		end

		-- If the user wants two tooltips.
		if (TBG_Saved("MDB_TT_SeperateActiveAndQueue") and TBG_IsSupported(TBG.Var.Player.zone)) then
			if (TBG.Var.Display.display_which_tt == "2") then
				if (ready ~= "" and queue ~= "")                  then queue = "\n\n"  .. queue; end
				if ((ready ~= "" or queue ~= "") and empty ~= "") then empty = "\n\n"  .. empty; end

				gathered = ready .. queue .. empty;
			else
				gathered = active;
			end

		-- If the user wants to display everything in one tooltip.
		else
			if (ready ~= "" and active ~= "")                                 then active = "\n\n" .. active; end
			if ((ready ~= "" or active ~= "") and queue ~= "")                then queue = "\n\n"  .. queue;  end
			if ((ready ~= "" or active ~= "" or queue ~= "") and empty ~= "") then empty = "\n\n"  .. empty;  end

			gathered = ready .. active .. queue .. empty;
		end

		if (gathered ~= "") then
			if (text ~= "") then text = text .. "\n\n"; end
			text = text .. gathered;
		end

		if (text ~= "") then
			text = text .. hint;
		else
			text = TBG.Var.Const.COLOR_RED .. TBG.Text.Label.Tooltip.WAITING;
		end

	-- Show the history, if the user wants it shown.
	elseif (TBG.Var.Display.display_which_tt == "HISTORY") then
		text = TBG_Display_GetHistoryText();
		text = text .. hint;
	end

	if (TBG.Var.Display.display_which_tt ~= "HISTORY") then TBG.Var.General.history_displayed = false; end

	TBG_Tooltip_SetText("\n" .. text);
	return text;
end


-- ## Forms the text printed on the Titan Panel tooltip.
function TBG_Display_GetText(bg)
	local text  = "";
	local name  = TBG_GetInstanceName(bg);

	-- If the user is not queued for the battleground.
	if (bg.status == TBG.Var.Const.BG_STATUS_NONE) then
		text = text .. TBG.Var.Const.COLOR_GREY .. string.gsub(TBG.Text.Label.Tooltip.NOT_IN_QUEUE, "{bg}", bg.name, 1);

	-- If the user is queued for the battleground.
	elseif (bg.status == TBG.Var.Const.BG_STATUS_QUEUED) then
		local wait_time = GetBattlefieldEstimatedWaitTime(bg.index);
		local in_queue  = GetBattlefieldTimeWaited(bg.index);

		text = text .. TBG.Var.Const.COLOR_GREEN .. string.upper(name) .. "\n";

		if (bg.instance ~= 0) then
			text = text .. TBG.Var.Const.COLOR_GREY .. string.gsub(TBG.Text.Label.Tooltip.QUEUED_PREF, "{bg}", name .. " " .. bg.instance, 1);
		else
			text = text .. TBG.Var.Const.COLOR_GREY .. string.gsub(TBG.Text.Label.Tooltip.QUEUED_NO_PREF, "{bg}", name, 1);
		end

		-- If the user knows how many active battlegrounds there were previously.
		if (TBG_IsBattleground(bg.name) and bg.open_age > 0 and TBG_Saved("MDW_TT_RememberOpenBGs")) then
			local age = "";

			-- If older than 10 minutes, display age.
			if (GetTime() - bg.open_age >= 300) then
				age = TBG.Var.Const.COLOR_YELLOW .. " [" .. TBG.Var.Const.COLOR_WHITE .. TBG_SecondsToTimeAbbrev(GetTime() - bg.open_age) .. TBG.Var.Const.COLOR_YELLOW .. "]";
			end

			text = text .. "\n" .. TBG.Var.Const.COLOR_YELLOW .. TBG.Text.Label.Tooltip.ACTIVE_INSTANCES .. "\t" .. TBG.Var.Const.COLOR_WHITE .. bg.open_instances .. age;
		end

		if (TBG_Saved("MDW_TT_TimeBeforeQueueExpires")) then
			text = text .. "\n\n" .. TBG.Var.Const.COLOR_YELLOW .. TBG.Text.Label.Tooltip.ESTIMATED_WAIT;

			if (wait_time == 0) then
				text = text .. "\t" .. TBG.Var.Const.COLOR_RED .. QUEUE_TIME_UNAVAILABLE .. ".";
			elseif (wait_time < 60000) then
				text = text .. "\t" .. TBG.Var.Const.COLOR_GREEN .. TBG.Text.Label.Tooltip.LESS_THAN_ONE_MIN;
			else
				text = text .. "\t" .. TBG.Var.Const.COLOR_WHITE .. TBG_SecondsToTime(wait_time / 1000, 1);
			end

			text = text .. "\n" .. TBG.Var.Const.COLOR_YELLOW .. TBG.Text.Label.Tooltip.TIME_IN_QUEUE .. "\t" .. TBG.Var.Const.COLOR_WHITE .. TBG_SecondsToTime(in_queue / 1000);

			-- Estimated time remaining.
			if (wait_time > in_queue) then
				text = text .. "\n" .. TBG.Var.Const.COLOR_YELLOW .. TBG.Text.Label.Tooltip.ESTIMATED_REMAINING .. "\t" .. TBG.Var.Const.COLOR_WHITE .. TBG_SecondsToTime((wait_time - in_queue) / 1000);
			end
		end

	-- If the user can now accept to go into the battleground.
	elseif (bg.status == TBG.Var.Const.BG_STATUS_CONFIRM) then
		local expire = "";

		-- Check if the queue expire time is to be shown.
		if (TBG_Saved("MDW_TT_TimeBeforeQueueExpires")) then
			local bg_expires = "";

			if (bg.expires == 0) then
				bg_expires = TBG.Text.Label.Tooltip.CLOSING;
			else
				bg_expires = TBG_SecondsToTime(bg.expires);
			end

			expire = " (" .. TBG.Var.Const.COLOR_RED .. bg_expires .. TBG.Var.Const.COLOR_GREEN .. ")";
		end

		local name = TBG.Var.Const.COLOR_WHITE .. name .. " " .. bg.instance .. TBG.Var.Const.COLOR_GREEN;

		-- If the user is not already in a battleground and is autojoining.
		if (TBG_Saved("MGB_AutoJoin") and TBG_IsAuto(TBG.Var.Const.AUTO_JOIN, bg.name) and  not TBG_AutoFinished(TBG.Var.Const.AUTO_JOIN)) then
			local autojoin = TBG_SecondsToTime(bg.auto);

			if (TBG.Var.Const.AUTO_JOIN.paused) then
				autojoin = autojoin .. TBG.Var.Const.COLOR_WHITE .. " (" .. TBG.Text.Label.Tooltip.PAUSED .. ")";
			end

			text = text .. TBG.Var.Const.COLOR_GREEN .. string.gsub(string.gsub(TBG.Text.Label.Tooltip.AUTOJOIN, "{bg}", name, 1), "{time}", TBG.Var.Const.COLOR_YELLOW .. autojoin .. TBG.Var.Const.COLOR_GREEN .. expire, 1);

		-- If the user is in a battleground or is not autojoining.
		else
			text = text .. TBG.Var.Const.COLOR_GREEN .. string.gsub(string.gsub(TBG.Text.Label.Tooltip.CONFIRM_JOIN, "{bg}", name, 1), "{time}", TBG.Var.Const.COLOR_GREEN .. expire, 1);
		end

	-- If the battleground is currently active.
	elseif (bg.status == TBG.Var.Const.BG_STATUS_ACTIVE) then
		local id    = TBG.Var.Player.name .. "|" .. TBG.Var.Player.faction;
		local total = TBG.DB.players.alliance + TBG.DB.players.horde;
		local arena = "";
		      text  = TBG.Var.Const.COLOR_GREEN  .. string.upper(name) .. "\n" .. text;

		-- If the player is auto leaving the battleground.
		if (TBG_Saved("MGB_AutoLeave") and TBG_IsAuto(TBG.Var.Const.AUTO_LEAVE, bg.name) and not TBG_AutoFinished(TBG.Var.Const.AUTO_LEAVE)) then
			local autoleave = TBG_SecondsToTime(bg.auto);

			if (TBG.Var.Const.AUTO_LEAVE.paused) then
				autoleave = autoleave .. TBG.Var.Const.COLOR_WHITE .. " (" .. TBG.Text.Label.Tooltip.PAUSED .. ")";
			end

			text = text .. TBG.Var.Const.COLOR_RED .. string.gsub(TBG.Text.Label.Tooltip.AUTO_LEAVE, "{time}", autoleave .. TBG.Var.Const.COLOR_RED, 1) .. "\n\n";
		end

		-- If the user wants to display the amount of time before the battle begins.
		if (TBG.DB.active.stats.bg.run ~= nil and TBG.DB.active.stats.bg.run ~= "" and TBG.DB.active.stats.bg.run > 0 and (TBG.DB.active.stats.bg.run / 1000) < bg.start_delay and TBG_Saved("MDB_TT_TimeToStart")) then
			text = text .. string.gsub(string.gsub(TBG.Text.Label.Tooltip.START_BG, "{time}", TBG.Var.Const.COLOR_WHITE .. TBG_SecondsToTime(bg.start_delay - (TBG.DB.active.stats.bg.run / 1000)) .. TBG.Var.Const.COLOR_YELLOW, 1), "{bg}", TBG.Var.Const.COLOR_WHITE .. bg.name .. TBG.Var.Const.COLOR_YELLOW, 1) .. "\n\n";
		end

		text = text .. TBG.Var.Const.COLOR_YELLOW .. bg.name .. " "  .. bg.instance .. ":";

		-- Print the instance and the run time.
		if (TBG.DB.active.stats.bg.run ~= nil and TBG.DB.active.stats.bg.run ~= "" and TBG.DB.active.stats.bg.run > 0) then
			text = text .. "\t" .. TBG.Var.Const.COLOR_WHITE .. TBG_SecondsToTime(TBG.DB.active.stats.bg.run / 1000, 1);
		else
			if (TBG.DB.active.stats.bg.run_estimate ~= nil and TBG.DB.active.stats.bg.run_estimate ~= "" and TBG.DB.active.stats.bg.run_estimate > 0) then
				text = text .. "\t" .. TBG.Var.Const.COLOR_WHITE .. " ~ " .. TBG_SecondsToTime(GetTime() - TBG.DB.active.stats.bg.run_estimate, 1);
			else
				text = text .. "\t" .. TBG.Var.Const.COLOR_RED   .. TBG.Text.Label.Tooltip.INSTANCE_RUN_TIME_NA;
			end
		end

		if (TBG_IsBattleground(bg.name)) then

			-- Print the number of players.
			if (total > 0) then
				text = text .. "\n" .. TBG.Var.Const.COLOR_YELLOW .. TBG.Text.Label.Tooltip.PLAYERS .. "\t" .. TBG.Var.Const.COLOR_WHITE .. total;

				text = text .. TBG.Var.Const.COLOR_YELLOW .. " [" .. TBG.Var.Const.COLOR_BLUE .. TBG.DB.players.alliance .. TBG.Var.Const.COLOR_YELLOW .. "]";
				text = text .. TBG.Var.Const.COLOR_YELLOW .. " [" .. TBG.Var.Const.COLOR_RED  .. TBG.DB.players.horde    .. TBG.Var.Const.COLOR_YELLOW .. "]";
			end

			-- If this is the Eye of the Storm, print the amount of flags captures by each team.
			if (TBG.Var.Player.zone == TBG.Text.Ref.BG_EOTS) then
				text = text .. "\n\n" .. TBG.Var.Const.COLOR_YELLOW .. string.gsub(SCORE_FLAGS_CAPTURED, "\n", " ") .. ":" .. "\t";

				text = text .. TBG.Var.Const.COLOR_YELLOW .. " [" .. TBG.Var.Const.COLOR_BLUE .. TBG.DB.active.stats.bg.stats.flags_captured.alliance .. TBG.Var.Const.COLOR_YELLOW .. "]";
				text = text .. TBG.Var.Const.COLOR_YELLOW .. " [" .. TBG.Var.Const.COLOR_RED  .. TBG.DB.active.stats.bg.stats.flags_captured.horde    .. TBG.Var.Const.COLOR_YELLOW .. "]";
			end
		end

		if (TBG_IsArena(bg.name)) then
			text = text .. "\n\n" .. TBG_Score_GetText();
		end

		-- Strings associated with specific headers.
		local str = {
			["PLAYER_STATISTICS"]    = "",
			["AB_WIN_ESTIMATES"]     = "",
			["MOVEMENT_TIMERS"]      = "",
			["PROGRESS_ESTIMATES"]   = "",
			["CAPTURE_TIMERS"]       = "",
			["WSG_FLAG_TRACKER"]     = "",
			["OBJECTIVE_STATISTICS"] = "",
			["FRIENDS"]              = "",
			["SERVER_STATISTICS"]    = "",
			["CLASS_STATISTICS"]     = "",
			["INACTIVE_PLAYERS"]     = "",
			["BATTLEGROUND_ITEMS"]   = "",
		};

		-- Only process the following for battlegrounds.
		if (TBG_IsBattleground(bg.name)) then

			-- Print movement timers.
			if (TBG.Var.Player.zone == TBG.Text.Ref.BG_AV and TBG_Saved("MDB_TT_MovementTimers") and (TBG.DB.active.stats.bg.stats.summons["IVUS"] ~= 0 or TBG.DB.active.stats.bg.stats.summons["LOK"] ~= 0)) then
				str["MOVEMENT_TIMERS"] = str["MOVEMENT_TIMERS"] .. "\n";

				if (TBG_Saved("MDW_TT_DisplayHeaders")) then
					str["MOVEMENT_TIMERS"] = str["MOVEMENT_TIMERS"] .. "\n" .. TBG.Var.Const.COLOR_WHITE .. TBG.Text.Label.Tooltip.MOVEMENT_TIMERS;
				end

				if (TBG.DB.active.stats.bg.stats.summons["IVUS"] ~= 0) then
					local mtime = TBG.Text.Label.Button.READY;
					local color = TBG.Var.Const.COLOR_BLUE;

					if (TBG.Var.BG.AV.time_to_movement - (GetTime() - TBG.DB.active.stats.bg.stats.summons["IVUS"]) > 0) then
						mtime = TBG_SecondsToTime(TBG.Var.BG.AV.time_to_movement - (GetTime() - TBG.DB.active.stats.bg.stats.summons["IVUS"]));

						local header = TBG.Var.Const.REF_MOVEMENT_TIMER .. "|name=" .. TBG.Text.Label.Other.IVUS  .. "|eta=" .. mtime .."}}";

						str["MOVEMENT_TIMERS"] = str["MOVEMENT_TIMERS"] .. "\n" .. header .. color .. TBG.Text.Label.Other.IVUS .. "\t" .. TBG.Var.Const.COLOR_WHITE .. mtime;
					else
						TBG.DB.active.stats.bg.stats.summons["IVUS"] = 0;
					end
				end

				if (TBG.DB.active.stats.bg.stats.summons["LOK"] ~= 0) then
					local mtime  = TBG.Text.Label.Button.READY;
					local color  = TBG.Var.Const.COLOR_RED;

					if (TBG.Var.BG.AV.time_to_movement - (GetTime() - TBG.DB.active.stats.bg.stats.summons["LOK"]) > 0) then
						mtime = TBG_SecondsToTime(TBG.Var.BG.AV.time_to_movement - (GetTime() - TBG.DB.active.stats.bg.stats.summons["LOK"]));

						local header = TBG.Var.Const.REF_MOVEMENT_TIMER .. "|name=" .. TBG.Text.Label.Other.LOKHOLAR  .. "|eta=" .. mtime .."}}";

						str["MOVEMENT_TIMERS"] = str["MOVEMENT_TIMERS"] .. "\n" .. header .. color .. TBG.Text.Label.Other.LOKHOLAR .. "\t" .. TBG.Var.Const.COLOR_WHITE .. mtime;
					else
						TBG.DB.active.stats.bg.stats.summons["LOK"] = 0;
					end
				end
			end


			-- Print server information.
			if ((TBG_Saved("MDB_TT_ServerStats_A") or TBG_Saved("MDB_TT_ServerStats_H")) and (not TBG_Saved("MD_DisablePercentInAV") or (TBG_Saved("MD_DisablePercentInAV") and TBG.Var.Player.zone ~= TBG.Text.Ref.BG_AV))) then
				str["SERVER_STATISTICS"] = str["SERVER_STATISTICS"] .. "\n";

				if (TBG_Saved("MDW_TT_DisplayHeaders")) then
					str["SERVER_STATISTICS"] = str["SERVER_STATISTICS"] .. "\n" .. TBG.Var.Const.COLOR_WHITE  .. TBG.Text.Label.Tooltip.SERVER_STATS;
				end

				local f         = 0;
				local reference = { [FACTION_ALLIANCE] = "MDB_TT_ServerStats_A", [FACTION_HORDE] = "MDB_TT_ServerStats_H", };

				for faction, value in pairs(TBG.DB.active.stats.bg.servers) do
					if (TBG_Saved(reference[faction])) then
						if (f > 0) then str["SERVER_STATISTICS"] = str["SERVER_STATISTICS"] .. "\n"; end

						for i = 1, table.getn(value.index) do
							if (TBG_Saved("MDB_TT_ServerStats_Top") and i > 3) then
								break;
							else

								str["SERVER_STATISTICS"] = str["SERVER_STATISTICS"] .. "\n" .. TBG.Var.Const.COLOR_YELLOW .. value.index[i] .. "\t" .. TBG.Var.Const.COLOR_FACTION[faction] .. value.reference[value.index[i]];

								if (TBG_Saved("MDB_TT_ServerStats_Percent")) then
									local percent = 0;

									if (faction == FACTION_ALLIANCE) then
										percent = TBG_MathPercent(value.reference[value.index[i]], TBG.DB.players.alliance);
									else
										percent = TBG_MathPercent(value.reference[value.index[i]], TBG.DB.players.horde);
									end

									str["SERVER_STATISTICS"] = str["SERVER_STATISTICS"] .. TBG.Var.Const.COLOR_YELLOW .. " (" .. TBG.Var.Const.COLOR_WHITE .. percent .. TBG.Var.Const.COLOR_YELLOW .. ")";
								end
							end
						end

						f = f + 1;
					end
				end
			end

			-- Print class information.
			if (TBG_Saved("MDB_TT_ClassStats_A") or TBG_Saved("MDB_TT_ClassStats_H")) then
				str["CLASS_STATISTICS"] = str["CLASS_STATISTICS"] .. "\n\n";

				if (TBG_Saved("MDW_TT_DisplayHeaders")) then
					str["CLASS_STATISTICS"] = str["CLASS_STATISTICS"] .. TBG.Var.Const.COLOR_WHITE  .. TBG.Text.Label.Tooltip.CLASS_STATS .. "\n";
				end

				local GetClasses = function(faction)
					local class = "";
					local cstr  = "";

					if (TBG_Saved("MDB_TT_ClassStats_Name")) then
						cstr = cstr .. "\n";
					end

					if (faction == 1) then
						cstr = cstr .. TBG.Var.Const.COLOR_BLUE .. FACTION_ALLIANCE;
					else
						cstr = cstr .. TBG.Var.Const.COLOR_RED .. FACTION_HORDE;
					end

					class = "DRUID";
					if (TBG_Saved("MDB_TT_ClassStats_Name")) then
						cstr  = cstr .. "\n" .. TBG.Var.Const.COLOR_CLASS[class].color .. TBG.Text.Ref.CLASS_NAME[class] .. TBG.Var.Const.COLOR_YELLOW .. " (" .. TBG.Var.Const.COLOR_WHITE .. TBG.DB.players.classes[faction][class] .. TBG.Var.Const.COLOR_YELLOW .. ")";
					else
						cstr  = cstr .. "\t" .. TBG.Var.Const.COLOR_CLASS[class].color .. TBG.DB.players.classes[faction][class] .. TBG.Var.Const.COLOR_YELLOW;
					end

					class = "HUNTER";
					if (TBG_Saved("MDB_TT_ClassStats_Name")) then
						cstr  = cstr .. ", " .. TBG.Var.Const.COLOR_CLASS[class].color .. TBG.Text.Ref.CLASS_NAME[class] .. TBG.Var.Const.COLOR_YELLOW .. " (" .. TBG.Var.Const.COLOR_WHITE .. TBG.DB.players.classes[faction][class] .. TBG.Var.Const.COLOR_YELLOW .. ")";
					else
						cstr  = cstr .. ", " .. TBG.Var.Const.COLOR_CLASS[class].color .. TBG.DB.players.classes[faction][class] .. TBG.Var.Const.COLOR_YELLOW;
					end

					class = "MAGE";
					if (TBG_Saved("MDB_TT_ClassStats_Name")) then
						cstr  = cstr .. ", " .. TBG.Var.Const.COLOR_CLASS[class].color .. TBG.Text.Ref.CLASS_NAME[class] .. TBG.Var.Const.COLOR_YELLOW .. " (" .. TBG.Var.Const.COLOR_WHITE .. TBG.DB.players.classes[faction][class] .. TBG.Var.Const.COLOR_YELLOW .. ")";
					else
						cstr  = cstr .. ", " .. TBG.Var.Const.COLOR_CLASS[class].color .. TBG.DB.players.classes[faction][class] .. TBG.Var.Const.COLOR_YELLOW;
					end

					class = "PALADIN";
					if (TBG_Saved("MDB_TT_ClassStats_Name")) then
						cstr  = cstr .. ", " .. TBG.Var.Const.COLOR_CLASS[class].color .. TBG.Text.Ref.CLASS_NAME[class] .. TBG.Var.Const.COLOR_YELLOW .. " (" .. TBG.Var.Const.COLOR_WHITE .. TBG.DB.players.classes[faction][class] .. TBG.Var.Const.COLOR_YELLOW .. ")";
					else
						cstr  = cstr .. ", " .. TBG.Var.Const.COLOR_CLASS[class].color .. TBG.DB.players.classes[faction][class] .. TBG.Var.Const.COLOR_YELLOW;
					end

					class = "PRIEST";
					if (TBG_Saved("MDB_TT_ClassStats_Name")) then
						cstr  = cstr .. ", " .. TBG.Var.Const.COLOR_CLASS[class].color .. TBG.Text.Ref.CLASS_NAME[class] .. TBG.Var.Const.COLOR_YELLOW .. " (" .. TBG.Var.Const.COLOR_WHITE .. TBG.DB.players.classes[faction][class] .. TBG.Var.Const.COLOR_YELLOW .. ")";
					else
						cstr  = cstr .. ", " .. TBG.Var.Const.COLOR_CLASS[class].color .. TBG.DB.players.classes[faction][class] .. TBG.Var.Const.COLOR_YELLOW;
					end

					class = "ROGUE";
					if (TBG_Saved("MDB_TT_ClassStats_Name")) then
						cstr  = cstr .. ", " .. TBG.Var.Const.COLOR_CLASS[class].color .. TBG.Text.Ref.CLASS_NAME[class] .. TBG.Var.Const.COLOR_YELLOW .. " (" .. TBG.Var.Const.COLOR_WHITE .. TBG.DB.players.classes[faction][class] .. TBG.Var.Const.COLOR_YELLOW .. ")";
					else
						cstr  = cstr .. ", " .. TBG.Var.Const.COLOR_CLASS[class].color .. TBG.DB.players.classes[faction][class] .. TBG.Var.Const.COLOR_YELLOW;
					end

					class = "SHAMAN";
					if (TBG_Saved("MDB_TT_ClassStats_Name")) then
						cstr  = cstr .. ", " .. TBG.Var.Const.COLOR_CLASS[class].color .. TBG.Text.Ref.CLASS_NAME[class] .. TBG.Var.Const.COLOR_YELLOW .. " (" .. TBG.Var.Const.COLOR_WHITE .. TBG.DB.players.classes[faction][class] .. TBG.Var.Const.COLOR_YELLOW .. ")";
					else
						cstr  = cstr .. ", " .. TBG.Var.Const.COLOR_CLASS[class].color .. TBG.DB.players.classes[faction][class] .. TBG.Var.Const.COLOR_YELLOW;
					end

					class = "WARLOCK";
					if (TBG_Saved("MDB_TT_ClassStats_Name")) then
						cstr  = cstr .. ", " .. TBG.Var.Const.COLOR_CLASS[class].color .. TBG.Text.Ref.CLASS_NAME[class] .. TBG.Var.Const.COLOR_YELLOW .. " (" .. TBG.Var.Const.COLOR_WHITE .. TBG.DB.players.classes[faction][class] .. TBG.Var.Const.COLOR_YELLOW .. ")";
					else
						cstr  = cstr .. ", " .. TBG.Var.Const.COLOR_CLASS[class].color .. TBG.DB.players.classes[faction][class] .. TBG.Var.Const.COLOR_YELLOW;
					end

					class = "WARRIOR";
					if (TBG_Saved("MDB_TT_ClassStats_Name")) then
						cstr  = cstr .. ", " .. TBG.Var.Const.COLOR_CLASS[class].color .. TBG.Text.Ref.CLASS_NAME[class] .. TBG.Var.Const.COLOR_YELLOW .. " (" .. TBG.Var.Const.COLOR_WHITE .. TBG.DB.players.classes[faction][class] .. TBG.Var.Const.COLOR_YELLOW .. ")";
					else
						cstr  = cstr .. ", " .. TBG.Var.Const.COLOR_CLASS[class].color .. TBG.DB.players.classes[faction][class] .. TBG.Var.Const.COLOR_YELLOW;
					end

					return cstr;
				end

				-- Retrieve alliance information.
				if (TBG_Saved("MDB_TT_ClassStats_A")) then
					str["CLASS_STATISTICS"] = str["CLASS_STATISTICS"] .. GetClasses(1);


					if (TBG_Saved("MDB_TT_ClassStats_H")) then
						str["CLASS_STATISTICS"] = str["CLASS_STATISTICS"] .. "\n";
					end
				end

				-- Retrieve horde information.
				if (TBG_Saved("MDB_TT_ClassStats_H")) then
					str["CLASS_STATISTICS"] = str["CLASS_STATISTICS"] .. GetClasses(0);
				end
			end

			-- If there are inactive players, print them if the player wants to.
			if (table.getn(TBG.DB.inactives.list) > 0 and TBG_Saved("MDB_TT_InactivePlayers")) then
				local inactives_text = "";
				local i              = 1;

				TBG.DB.inactives.list = TBG_Battleground_InactivesSort(TBG.DB.inactives.list);

				while (i <= table.getn(TBG.DB.inactives.list)) do
					if (not TBG.DB.players.names[TBG.DB.inactives.list[i]] or not TBG.DB.players.names[TBG.DB.inactives.list[i]].inactive) then
						table.remove(TBG.DB.inactives.list, i);

					else
						local inactive = TBG_SecondsToTime(GetTime() - TBG.DB.players.names[TBG.DB.inactives.list[i]].last_movement);

						inactives_text = inactives_text .. "\n" .. TBG.Var.Const.COLOR_YELLOW .. TBG.DB.players.names[TBG.DB.inactives.list[i]].name .. "\t" .. TBG.Var.Const.COLOR_RED .. inactive;

						i = i + 1;
					end
				end

				if (inactives_text ~= "") then
					str["INACTIVE_PLAYERS"] = str["INACTIVE_PLAYERS"] .. "\n";

					if (TBG_Saved("MDW_TT_DisplayHeaders")) then
						str["INACTIVE_PLAYERS"] = str["INACTIVE_PLAYERS"] .. "\n" .. TBG.Var.Const.COLOR_WHITE .. TBG.Text.Label.Tooltip.INACTIVES;
					end

					str["INACTIVE_PLAYERS"] = str["INACTIVE_PLAYERS"] .. inactives_text;
				end
			end

			-- If Alterac Valley, track the progress of objectives. Killing Captains, Generals, beacons planted etc.
			if (TBG_Saved("MDB_TT_ProgressEstimates") and bg.name == TBG.Text.Ref.BG_AV) then
				str["PROGRESS_ESTIMATES"] = str["PROGRESS_ESTIMATES"] .. "\n";

				if (TBG_Saved("MDW_TT_DisplayHeaders")) then
					str["PROGRESS_ESTIMATES"] = str["PROGRESS_ESTIMATES"] .. "\n" .. TBG.Var.Const.COLOR_WHITE .. TBG.Text.Label.Tooltip.AV_PROGRESS_ESTIMATE;
				end

				for key, value in pairs(TBG.DB.active.stats.bg.stats.generals) do
					str["PROGRESS_ESTIMATES"] = str["PROGRESS_ESTIMATES"] .. "\n" .. TBG.Var.Const.COLOR_YELLOW .. "(" .. value.color .. "General" .. TBG.Var.Const.COLOR_YELLOW .. ")";
					str["PROGRESS_ESTIMATES"] = str["PROGRESS_ESTIMATES"] .. " " .. value.color .. value.npc .. "\t";

					if (value.timer == 0) then
						str["PROGRESS_ESTIMATES"] = str["PROGRESS_ESTIMATES"] .. TBG.Var.Const.COLOR_GREEN .. "Inactive";
					else
						str["PROGRESS_ESTIMATES"] = str["PROGRESS_ESTIMATES"] .. TBG.Var.Const.COLOR_RED .. "Engaged" .. TBG.Var.Const.COLOR_YELLOW .. " (" .. TBG.Var.Const.COLOR_WHITE .. TBG_SecondsToTimeAbbrev(GetTime() - value.timer, 1) .. TBG.Var.Const.COLOR_YELLOW .. ")";
					end
				end
			end

			-- If this is Arathi Basin or Eye of the Storm, print who is likely to win this battleground.
			if ((bg.name == TBG.Text.Ref.BG_AB or bg.name == TBG.Text.Ref.BG_EOTS) and TBG_Saved("MDB_TT_ABWinEstimates") and TBG.DB.world_state[1] and TBG.DB.world_state[1].active and GetBattlefieldWinner() == nil) then
				local winner, win_time, alliance_score, horde_score, nodes = TBG_Battleground_Resources_GetWinner(TBG.DB.world_state[1].bases, TBG.DB.world_state[2].bases);

				str["AB_WIN_ESTIMATES"] = str["AB_WIN_ESTIMATES"] .. "\n";

				if (TBG_Saved("MDW_TT_DisplayHeaders")) then
					str["AB_WIN_ESTIMATES"] = str["AB_WIN_ESTIMATES"] .. "\n" .. TBG.Var.Const.COLOR_WHITE .. TBG.Text.Label.Tooltip.AB_WIN_ESTIMATE;
				end

				-- If unknown.
				if (winner == nil) then
					str["AB_WIN_ESTIMATES"] = str["AB_WIN_ESTIMATES"] .. "\n" .. TBG.Var.Const.COLOR_YELLOW .. TBG.Text.Label.Tooltip.AB_WINNER .. "\t" .. TBG.Var.Const.COLOR_WHITE .. TBG.Text.Label.Tooltip.AB_WINNER_UNKNOWN;

				-- If Horde.
				elseif (winner == "Horde") then
					str["AB_WIN_ESTIMATES"] = str["AB_WIN_ESTIMATES"] .. "\n" .. TBG.Var.Const.COLOR_YELLOW .. TBG.Text.Label.Tooltip.AB_WINNER .. "\t" .. TBG.Var.Const.COLOR_RED .. FACTION_HORDE;

				-- If Alliance.
				elseif (winner == "Alliance") then
					str["AB_WIN_ESTIMATES"] = str["AB_WIN_ESTIMATES"] .. "\n" .. TBG.Var.Const.COLOR_YELLOW .. TBG.Text.Label.Tooltip.AB_WINNER .. "\t" .. TBG.Var.Const.COLOR_BLUE .. FACTION_ALLIANCE;
				end

				if (winner ~= nil) then
					str["AB_WIN_ESTIMATES"] = str["AB_WIN_ESTIMATES"] .. "\n" .. TBG.Var.Const.COLOR_YELLOW .. TBG.Text.Label.Tooltip.AB_FINAL_SCORE .. "\t" .. TBG.Var.Const.COLOR_BLUE  .. alliance_score .. TBG.Var.Const.COLOR_WHITE .. " : " .. TBG.Var.Const.COLOR_RED .. horde_score;
					str["AB_WIN_ESTIMATES"] = str["AB_WIN_ESTIMATES"] .. "\n" .. TBG.Var.Const.COLOR_YELLOW .. TBG.Text.Label.Tooltip.AB_TIME_LEFT   .. "\t" .. TBG.Var.Const.COLOR_WHITE .. TBG_SecondsToTimeAbbrev(win_time);

					-- Determine number of bases the player needs to win.

					-- If the player's faction isn't going to win.
					if (winner ~= UnitFactionGroup("player")) then
						local bases                   = TBG_Battleground_Resources_NodesRequiredToWin(TBG.Var.Player.faction);
						      str["AB_WIN_ESTIMATES"] = str["AB_WIN_ESTIMATES"] .. "\n\n" .. TBG.Var.Const.COLOR_YELLOW .. TBG.Text.Label.Tooltip.AB_TO_WIN  .. "\t" .. TBG.Var.Const.COLOR_WHITE .. bases;

					-- Otherwise, if their faction is going to win.
					else
						local bases                   = TBG_Battleground_Resources_NodesRequiredToWin(TBG.Var.Player.enemy_faction);
							  str["AB_WIN_ESTIMATES"] = str["AB_WIN_ESTIMATES"] .. "\n\n" .. TBG.Var.Const.COLOR_YELLOW .. TBG.Text.Label.Tooltip.AB_TO_WIN_ENEMY .. "\t" .. TBG.Var.Const.COLOR_WHITE .. bases;
					end

					-- Time left until the number of nodes needed increases.
					if (TBG.Var.BG.AB.time_to_node_increase > 0) then
						str["AB_WIN_ESTIMATES"] = str["AB_WIN_ESTIMATES"] .. TBG.Var.Const.COLOR_YELLOW .. " (" .. TBG.Var.Const.COLOR_RED .. TBG_SecondsToTimeAbbrev(TBG.Var.BG.AB.time_to_node_increase) .. TBG.Var.Const.COLOR_YELLOW .. ")";
					end
				end
			end

			if (TBG.DB.players.names and TBG.DB.players.names[id]) then

				-- Print information on the user.
				if (TBG_Saved("MDB_TT_PlayerStats")) then
					str["PLAYER_STATISTICS"] = str["PLAYER_STATISTICS"] .. "\n";

					if (TBG_Saved("MDW_TT_DisplayHeaders")) then
						str["PLAYER_STATISTICS"] = str["PLAYER_STATISTICS"] .. "\n" .. TBG.Var.Const.COLOR_WHITE  .. TBG.Text.Label.Tooltip.PLAYER_STATS;
					end

					str["PLAYER_STATISTICS"] = str["PLAYER_STATISTICS"] .. "\n"   .. TBG.Var.Const.COLOR_YELLOW .. TBG.Text.Label.Tooltip.PLAYER_STANDING        .. "\t" .. TBG.Var.Const.COLOR_WHITE .. TBG.DB.players.names[id].standing;
					str["PLAYER_STATISTICS"] = str["PLAYER_STATISTICS"] .. "\n"   .. TBG.Var.Const.COLOR_YELLOW .. string.gsub(SCORE_KILLING_BLOWS, "\n", " ")   .. ":\t" .. TBG.Var.Const.COLOR_GREEN .. TBG.DB.players.names[id].killing_blows;
					str["PLAYER_STATISTICS"] = str["PLAYER_STATISTICS"] .. "\n"   .. TBG.Var.Const.COLOR_YELLOW .. string.gsub(SCORE_HONORABLE_KILLS, "\n", " ") .. ":\t" .. TBG.Var.Const.COLOR_GREEN .. TBG.DB.players.names[id].kills;
					str["PLAYER_STATISTICS"] = str["PLAYER_STATISTICS"] .. "\n"   .. TBG.Var.Const.COLOR_YELLOW .. string.gsub(DEATHS, "\n", " ")                .. ":\t" .. TBG.Var.Const.COLOR_RED   .. TBG.DB.players.names[id].deaths;
					str["PLAYER_STATISTICS"] = str["PLAYER_STATISTICS"] .. "\n"   .. TBG.Var.Const.COLOR_YELLOW .. string.gsub(SCORE_DAMAGE_DONE, "\n", " ")     .. ":\t" .. TBG.Var.Const.COLOR_RED   .. TBG.DB.players.names[id].damage;
					str["PLAYER_STATISTICS"] = str["PLAYER_STATISTICS"] .. "\n"   .. TBG.Var.Const.COLOR_YELLOW .. string.gsub(SCORE_HEALING_DONE, "\n", " ")    .. ":\t" .. TBG.Var.Const.COLOR_GREEN .. TBG.DB.players.names[id].healing;
					str["PLAYER_STATISTICS"] = str["PLAYER_STATISTICS"] .. "\n"   .. TBG.Var.Const.COLOR_YELLOW .. string.gsub(SCORE_HONOR_GAINED, "\n", " ")    .. ":\t" .. TBG.Var.Const.COLOR_WHITE .. TBG.DB.players.names[id].honor;
				end

				if (TBG.DB.players.names[id].data) then

					if (TBG_Saved("MDB_TT_ObjectiveStats") and (bg.name == TBG.Text.Ref.BG_AV or bg.name == TBG.Text.Ref.BG_WSG or bg.name == TBG.Text.Ref.BG_AB)) then
						str["OBJECTIVE_STATISTICS"] = str["OBJECTIVE_STATISTICS"] .. "\n";

						if (TBG_Saved("MDW_TT_DisplayHeaders")) then
							str["OBJECTIVE_STATISTICS"] = str["OBJECTIVE_STATISTICS"] .. "\n" .. TBG.Var.Const.COLOR_WHITE  .. TBG.Text.Label.Tooltip.OBJECTIVE_STATS;
						end
					end

					-- Print information on the locations in Alterac Valley.
					if (TBG_Saved("MDB_TT_ObjectiveStats") and bg.name == TBG.Text.Ref.BG_AV) then
						if (TBG.DB.players.names[id].data[1]) then str["OBJECTIVE_STATISTICS"] = str["OBJECTIVE_STATISTICS"] .. "\n"   .. TBG.Var.Const.COLOR_YELLOW .. TBG.Text.Label.Tooltip.OBJECTIVE_GY_ASSAULTED .. "\t" .. TBG.Var.Const.COLOR_WHITE .. TBG.DB.players.names[id].data[1].count; end
						if (TBG.DB.players.names[id].data[2]) then str["OBJECTIVE_STATISTICS"] = str["OBJECTIVE_STATISTICS"] .. "\n"   .. TBG.Var.Const.COLOR_YELLOW .. TBG.Text.Label.Tooltip.OBJECTIVE_GY_DEFENDED  .. "\t" .. TBG.Var.Const.COLOR_WHITE .. TBG.DB.players.names[id].data[2].count; end
						if (TBG.DB.players.names[id].data[3]) then str["OBJECTIVE_STATISTICS"] = str["OBJECTIVE_STATISTICS"] .. "\n"   .. TBG.Var.Const.COLOR_YELLOW .. TBG.Text.Label.Tooltip.OBJECTIVE_TOWERS_ASSAULTED  .. "\t" .. TBG.Var.Const.COLOR_WHITE .. TBG.DB.players.names[id].data[3].count; end
						if (TBG.DB.players.names[id].data[4]) then str["OBJECTIVE_STATISTICS"] = str["OBJECTIVE_STATISTICS"] .. "\n"   .. TBG.Var.Const.COLOR_YELLOW .. TBG.Text.Label.Tooltip.OBJECTIVE_TOWERS_DEFENDED   .. "\t" .. TBG.Var.Const.COLOR_WHITE .. TBG.DB.players.names[id].data[4].count; end
						if (TBG.DB.players.names[id].data[5]) then str["OBJECTIVE_STATISTICS"] = str["OBJECTIVE_STATISTICS"] .. "\n"   .. TBG.Var.Const.COLOR_YELLOW .. TBG.Text.Label.Tooltip.OBJECTIVE_SECONDARY_OBJECTIVES      .. "\t" .. TBG.Var.Const.COLOR_WHITE .. TBG.DB.players.names[id].data[5].count; end

					-- Print information on the locations in Warsong Gulch.
					elseif (TBG_Saved("MDB_TT_ObjectiveStats") and bg.name == TBG.Text.Ref.BG_WSG) then
						if (TBG.DB.players.names[id].data[1]) then str["OBJECTIVE_STATISTICS"] = str["OBJECTIVE_STATISTICS"] .. "\n"   .. TBG.Var.Const.COLOR_YELLOW .. TBG.Text.Label.Tooltip.OBJECTIVE_FLAGS_CAPTURED.. "\t" .. TBG.Var.Const.COLOR_WHITE .. TBG.DB.players.names[id].data[1].count; end
						if (TBG.DB.players.names[id].data[2]) then str["OBJECTIVE_STATISTICS"] = str["OBJECTIVE_STATISTICS"] .. "\n"   .. TBG.Var.Const.COLOR_YELLOW .. TBG.Text.Label.Tooltip.OBJECTIVE_FLAGS_RETURNED.. "\t" .. TBG.Var.Const.COLOR_WHITE .. TBG.DB.players.names[id].data[2].count; end

					-- Print information on the locations in Arathi Basin.
					elseif (TBG_Saved("MDB_TT_ObjectiveStats") and bg.name == TBG.Text.Ref.BG_AB) then
						if (TBG.DB.players.names[id].data[1]) then str["OBJECTIVE_STATISTICS"] = str["OBJECTIVE_STATISTICS"] .. "\n"   .. TBG.Var.Const.COLOR_YELLOW .. TBG.Text.Label.Tooltip.OBJECTIVE_BASES_ASSULTED.. "\t" .. TBG.Var.Const.COLOR_WHITE .. TBG.DB.players.names[id].data[1].count; end
						if (TBG.DB.players.names[id].data[2]) then str["OBJECTIVE_STATISTICS"] = str["OBJECTIVE_STATISTICS"] .. "\n"   .. TBG.Var.Const.COLOR_YELLOW .. TBG.Text.Label.Tooltip.OBJECTIVE_BASES_DEFENDED.. "\t" .. TBG.Var.Const.COLOR_WHITE .. TBG.DB.players.names[id].data[2].count; end
					end
				end
			end

			-- Print information on friends.
			if ( TBG_Saved("MDB_TT_Friends") and TBG.Var.BG.friends_str ~= "" ) then
				str["FRIENDS"] = str["FRIENDS"] .. "\n";

				if (TBG_Saved("MDW_TT_DisplayHeaders")) then
					str["FRIENDS"] = str["FRIENDS"] .. "\n" .. TBG.Var.Const.COLOR_WHITE .. TBG.Text.Label.Tooltip.FRIENDS;
				end

				str["FRIENDS"] = str["FRIENDS"] .. "\n" .. TBG.Var.BG.friends_str;
			end

			-- Print information on incoming captures.
			if (TBG_Saved("MDB_TT_CaptureTimers") and TBG.Var.Const.BATTLEGROUNDS_INFORMATION[bg.name].has_points and TBG.DB.active.captures ~= nil and table.getn(TBG.DB.active.captures) > 0 and GetBattlefieldWinner() == nil) then
				local capInfo = "";

				for i = 1, table.getn(TBG.DB.active.captures) do
					local eta = "";

					-- If the capture was done while the player was not there, show unknown.
					if (not TBG_Saved("MD_AllowInaccurateTimers") and TBG.DB.active.captures[i].enter) then
						eta = TBG.Text.Label.Tooltip.CAPTURE_TIMERS_UNKNOWN;

					-- Otherwise, show the timer.
					else
						eta = TBG_SecondsToTime(TBG.DB.active.captures[i].cap - GetTime());

						-- If there is a capture imminent, indicate.
						if (eta == "") then
							eta = TBG.Text.Label.Tooltip.CAPTURE_TIMERS_IMINENT;
						end
					end

					-- If not unknown, display timers.
					if (eta ~= TBG.Text.Label.Tooltip.CAPTURE_TIMERS_UNKNOWN) then
						local header = TBG.Var.Const.REF_CAPTURE_TIMER .. "who=" .. TBG.DB.active.captures[i].who .. "|name=" .. TBG.DB.active.captures[i].name .. "|eta=" ..eta .."}}";

						if (TBG.DB.active.captures[i].who == "Horde") then
							capInfo = capInfo .. "\n" .. header .. TBG.Var.Const.COLOR_YELLOW .. TBG.DB.active.captures[i].name .. "\t" .. TBG.Var.Const.COLOR_RED  .. eta;
						elseif (TBG.DB.active.captures[i].who == "Alliance") then
							capInfo = capInfo .. "\n" .. header .. TBG.Var.Const.COLOR_YELLOW .. TBG.DB.active.captures[i].name .. "\t" .. TBG.Var.Const.COLOR_BLUE .. eta;
						end
					end
				end

				-- If any valid captures, print them.
				if (capInfo ~= "") then
					str["CAPTURE_TIMERS"] = str["CAPTURE_TIMERS"] .. "\n";

					if (TBG_Saved("MDW_TT_DisplayHeaders")) then
						str["CAPTURE_TIMERS"] = str["CAPTURE_TIMERS"] .. "\n" .. TBG.Var.Const.COLOR_WHITE .. TBG.Var.Const.REF_CAPTURE_TIMERS .. TBG.Text.Label.Tooltip.CAPTURE_TIMERS;
					end

					str["CAPTURE_TIMERS"] = str["CAPTURE_TIMERS"] .. capInfo;
				end
			end

			-- Print information on flag caps.
			if (TBG_Saved("MDB_TT_FlagTracker") and TBG.Var.Const.BATTLEGROUNDS_INFORMATION[bg.name].has_flag and GetBattlefieldWinner() == nil) then
				if ((TBG.DB.active.stats.bg.stats.horde.player ~= "" and TBG.DB.active.stats.bg.stats.horde.player ~= nil) or (TBG.DB.active.stats.bg.stats.alliance.player ~= "" and TBG.DB.active.stats.bg.stats.alliance.player ~= nil)) then
					str["WSG_FLAG_TRACKER"] = str["WSG_FLAG_TRACKER"] .. "\n";

					if (TBG_Saved("MDW_TT_DisplayHeaders")) then
						str["WSG_FLAG_TRACKER"] = str["WSG_FLAG_TRACKER"] .. "\n" .. TBG.Var.Const.COLOR_WHITE .. TBG.Text.Label.Tooltip.WSG_FLAG_TRACKER;
					end

					if (TBG.DB.active.stats.bg.stats.alliance.player ~= "" and TBG.DB.active.stats.bg.stats.alliance.player ~= nil) then
						local player  = TBG.DB.players.names[TBG.DB.active.stats.bg.stats.alliance.player .. "|Alliance"];
						local faction = FACTION_ALLIANCE;
						local flag    = TBG.Var.Const.COLOR_RED;

						if (TBG.Var.Player.zone == TBG.Text.Ref.BG_EOTS) then
							flag = TBG.Var.Const.COLOR_BLUE;
						end

						if (player) then
							local header                  = TBG.Var.Const.REF_FLAG .. "faction=" .. TBG.Var.Const.FACTION_NAME.enemy[faction] .. "|player=" .. TBG.DB.active.stats.bg.stats.alliance.player .. "|class=" .. player.class .. "}}";
							      str["WSG_FLAG_TRACKER"] = str["WSG_FLAG_TRACKER"] .. "\n" .. header .. TBG.Var.Const.COLOR_YELLOW .. "(" ..  flag .. TBG.Text.Label.Tooltip.WSG_FLAG .. TBG.Var.Const.COLOR_YELLOW .. ") " .. TBG.Var.Const.COLOR_BLUE .. TBG.DB.active.stats.bg.stats.alliance.player .. "\t" .. TBG.Var.Const.COLOR_WHITE .. player.class;
						end
					end

					if (TBG.DB.active.stats.bg.stats.horde.player ~= "" and TBG.DB.active.stats.bg.stats.horde.player ~= nil) then
						local player  = TBG.DB.players.names[TBG.DB.active.stats.bg.stats.horde.player .. "|Horde"];
						local faction = FACTION_HORDE;
						local flag    = TBG.Var.Const.COLOR_BLUE;

						if (TBG.Var.Player.zone == TBG.Text.Ref.BG_EOTS) then
							flag = TBG.Var.Const.COLOR_RED;
						end

						if (player) then
							local header                  = TBG.Var.Const.REF_FLAG .. "faction=" .. TBG.Var.Const.FACTION_NAME.enemy[faction] .. "|player=" .. TBG.DB.active.stats.bg.stats.horde.player .. "|class=" .. player.class .. "}}";
							      str["WSG_FLAG_TRACKER"] = str["WSG_FLAG_TRACKER"] .. "\n" .. header .. TBG.Var.Const.COLOR_YELLOW .. "(" ..  flag .. TBG.Text.Label.Tooltip.WSG_FLAG .. TBG.Var.Const.COLOR_YELLOW .. ") " .. TBG.Var.Const.COLOR_RED .. TBG.DB.active.stats.bg.stats.horde.player .. "\t" .. TBG.Var.Const.COLOR_WHITE .. player.class;
						end
					end
				end
			end

			-- Print information on battleground items held.
			if (TBG_Saved("MDB_TT_BattlegroundItems") and bg.name == TBG.Text.Ref.BG_AV and table.getn(TBG.DB.inventory.battleground.index) > 0) then
				str["BATTLEGROUND_ITEMS"] = str["BATTLEGROUND_ITEMS"] .. "\n";

				if (TBG_Saved("MDW_TT_DisplayHeaders")) then
					str["BATTLEGROUND_ITEMS"] = str["BATTLEGROUND_ITEMS"] .. "\n" .. TBG.Var.Const.COLOR_WHITE .. TBG.Text.Label.Tooltip.BGITEMS;
				end

				for i = 1, table.getn(TBG.DB.inventory.battleground.index) do
					local item = TBG.DB.inventory.battleground.index[i];

					str["BATTLEGROUND_ITEMS"] = str["BATTLEGROUND_ITEMS"] .. "\n" .. TBG.Var.Const.COLOR_YELLOW .. item.name .. "\t" .. TBG.Var.Const.COLOR_WHITE .. item.count;
				end
			end
		end

		-- Form the text in the order the user desires.
		for i = 1, table.getn(TBG_Saved("MD_TT_DisplayOrder").list) do
			if (str[TBG_Saved("MD_TT_DisplayOrder").list[i]]) then
				text = text .. str[TBG_Saved("MD_TT_DisplayOrder").list[i]];
			end
		end
	end

	return text;
end


-- ## Returns text for the battleground history.
function TBG_Display_GetHistoryText()
	local text = TBG.Var.Const.COLOR_WHITE .. TBG.Text.Label.Tooltip.HISTORY .. "\n\n";

	-- If this tooltip has only just been shown, then validate the history.
	if (not TBG.Var.General.history_displayed) then
		TBG_Battleground_HistoryValidate();
		TBG.Var.General.history_displayed = true;
	end

	local Won = function(t)
		local str   = "";
		local total = t.total.pug[1] + t.total.team[1];

		-- Played.
		str = str .. TBG.Var.Const.COLOR_YELLOW .. TBG.Text.Label.Tooltip.HISTORY_PLAYED .. "\t" .. TBG.Var.Const.COLOR_WHITE .. total .. " ";
		str = str .. TBG.Var.Const.COLOR_YELLOW .. "[" .. TBG.Var.Const.COLOR_BLUE .. t.total.pug[1] .. TBG.Var.Const.COLOR_YELLOW .. ":" .. TBG.Var.Const.COLOR_BLUE .. t.total.team[1] .. TBG.Var.Const.COLOR_YELLOW .. "] ";
		str = str .. TBG.Var.Const.COLOR_YELLOW .. "[" .. TBG.Var.Const.COLOR_RED .. t.total.pug[0] .. TBG.Var.Const.COLOR_YELLOW .. ":" .. TBG.Var.Const.COLOR_RED .. t.total.team[0] .. TBG.Var.Const.COLOR_YELLOW .. "]\n";

		-- Won.
		str = str .. TBG.Var.Const.COLOR_YELLOW .. TBG.Text.Label.Tooltip.HISTORY_WON .. "\t";
		str = str .. TBG.Var.Const.COLOR_YELLOW .. "[" .. TBG.Var.Const.COLOR_BLUE .. t.won.pug[1] .. TBG.Var.Const.COLOR_YELLOW .. ":" .. TBG.Var.Const.COLOR_BLUE .. t.won.team[1] .. TBG.Var.Const.COLOR_YELLOW .. "] ";
		str = str .. TBG.Var.Const.COLOR_YELLOW .. "[" .. TBG.Var.Const.COLOR_RED .. t.won.pug[0] .. TBG.Var.Const.COLOR_YELLOW .. ":" .. TBG.Var.Const.COLOR_RED .. t.won.team[0] .. TBG.Var.Const.COLOR_YELLOW .. "]";

		return str;
	end

	local Length = function(t, title)
		local str     = "";
		local faction = {
			pug  = t.pug.faction,
			team = t.team.faction,
		};

		if (faction.pug == 1) then
			faction.pug = TBG.Var.Const.COLOR_BLUE;
		elseif (faction.pug == 0) then
			faction.pug = TBG.Var.Const.COLOR_RED;
		end

		if (faction.team == 1) then
			faction.team = TBG.Var.Const.COLOR_BLUE;
		elseif (faction.team == 0) then
			faction.team = TBG.Var.Const.COLOR_RED;
		end

		if (faction.pug == nil) then
			faction.pug = TBG.Var.Const.COLOR_WHITE .. TBG.Text.Label.Tooltip.HISTORY_NOT_APPLICABLE;
		else
			faction.pug = faction.pug .. TBG_SecondsToTimeAbbrev(t.pug.length / 1000);
		end

		if (faction.team == nil) then
			faction.team = TBG.Var.Const.COLOR_WHITE .. TBG.Text.Label.Tooltip.HISTORY_NOT_APPLICABLE;
		else
			faction.team = faction.team .. TBG_SecondsToTimeAbbrev(t.team.length / 1000);
		end

		str = str .. TBG.Var.Const.COLOR_YELLOW .. title .. "\t";
		str = str .. TBG.Var.Const.COLOR_YELLOW .. "[" .. faction.pug .. TBG.Var.Const.COLOR_YELLOW .. ":" .. faction.team .. TBG.Var.Const.COLOR_YELLOW .. "]";

		return str;
	end

	local Score = function(t, bg)
		local str = "";
		local alliance = t.total_score[1][1];
		local horde    = t.total_score[0][1];
		local total    = t.total.team[1] + t.total.pug[1];

		if (alliance == nil) then alliance = 0; end
		if (horde == nil)    then horde    = 0; end

		if (total == 0) then
			alliance = TBG.Var.Const.COLOR_WHITE .. TBG.Text.Label.Tooltip.HISTORY_NOT_APPLICABLE;
			horde    = alliance;
		else
			alliance = TBG.Var.Const.COLOR_BLUE .. TBG_MathRound(alliance / total, 0);
			horde    = TBG.Var.Const.COLOR_RED .. TBG_MathRound(horde / total, 0);
		end

		str = str .. TBG.Var.Const.COLOR_YELLOW .. TBG.Text.Label.Tooltip.HISTORY_SCORE .. " (" .. TBG.Var.Const.COLOR_WHITE .. TBG.Text.Label.Tooltip.HISTORY_AVERAGE .. TBG.Var.Const.COLOR_YELLOW ..")\t";
		str = str .. TBG.Var.Const.COLOR_YELLOW .. "[" .. alliance .. TBG.Var.Const.COLOR_YELLOW .. "] ";
		str = str .. TBG.Var.Const.COLOR_YELLOW .. "[" .. horde .. TBG.Var.Const.COLOR_YELLOW .. "]";

		return str;
	end

	text = text .. TBG.Var.Const.COLOR_GREY .. TBG.Text.Label.Tooltip.HISTORY_HELP_RATIOS .. " " .. TBG.Var.Const.COLOR_YELLOW .. "[" .. TBG.Var.Const.COLOR_BLUE .. FACTION_ALLIANCE .. TBG.Var.Const.COLOR_YELLOW .. "] [" .. TBG.Var.Const.COLOR_RED .. FACTION_HORDE .. TBG.Var.Const.COLOR_YELLOW .. "]" .. TBG.Var.Const.COLOR_GREY .. ".\n";
	text = text .. TBG.Var.Const.COLOR_GREY .. TBG.Text.Label.Tooltip.HISTORY_HELP_RATIOS_INNER .. " " .. TBG.Var.Const.COLOR_YELLOW .. "[" .. TBG.Var.Const.COLOR_WHITE .. TBG.Text.Label.Tooltip.HISTORY_PICK_UP_GROUPS .. TBG.Var.Const.COLOR_YELLOW .. ":" .. TBG.Var.Const.COLOR_WHITE .. TBG.Text.Label.Tooltip.HISTORY_TEAM .. TBG.Var.Const.COLOR_YELLOW .. "]" .. TBG.Var.Const.COLOR_GREY .. ".\n\n";

	-- Total.
	text = text .. Won(TBG_Saved("BattlegroundHistory")) .. "\n";

	for i = 1, table.getn(TBG.Var.Const.BATTLEGROUNDS) do
		local bg = TBG_Saved("BattlegroundHistory")[TBG.Var.Const.BATTLEGROUNDS[i]];

		text = text .. "\n" .. TBG.Var.Const.COLOR_GREEN .. string.upper(bg.name) .. "\n";
		text = text .. Won(bg) .. "\n";

		if (TBG.Var.Const.BATTLEGROUNDS_INFORMATION[bg.name].has_score) then
			text = text .. Score(bg, "AB") .. "\n";
		end

		text = text .. "\n";
		text = text .. Length(bg.shortest_game, TBG.Text.Label.Tooltip.HISTORY_SHORTEST) .. "\n";

		if (TBG.Var.Const.BATTLEGROUNDS_INFORMATION[bg.name].time_is_limited) then
			text = text .. Length(bg.longest_game, TBG.Text.Label.Tooltip.HISTORY_LONGEST) .. "\n";
		end
	end

	return text;
end


-- Returns hints applicable to the moment.
function TBG_Display_GetHints()
	if (not TBG_Saved("MDB_TT_NoHints")) then return ""; end
	local hint = "";

	-- Show hint on starting and pausing the auto join or leave timer.
	if (TBG_AutoStarted(TBG.Var.Const.AUTO_JOIN) or TBG_AutoStarted(TBG.Var.Const.AUTO_LEAVE)) then
		hint = hint .. "\n" .. TBG.Var.Const.COLOR_RED .. TBG.Text.Label.Tooltip.AUTOJOIN_HINT .. "\n";
	end

	-- Show hint on hiding and displaying the score frame.
	if (TBG_IsBattleground(TBG.Var.Player.zone)) then
		hint = hint .. "\n"  .. TBG.Var.Const.COLOR_RED .. TBG.Text.Label.Tooltip.HINT_SCORE_FRAME;
	end

	-- Show hint on hiding and displaying the minimap.
	if (not TBG_IsArena(TBG.Var.Player.zone)) then
		hint = hint .. "\n"  .. TBG.Var.Const.COLOR_RED .. TBG.Text.Label.Tooltip.HINT_MINIMAP .. "\n";
	end

	-- If the user wants two tooltips.
	if (TBG_Saved("MDB_TT_SeperateActiveAndQueue") and TBG_IsSupported(TBG.Var.Player.zone)) then
		hint = hint .. "\n"  .. TBG.Var.Const.COLOR_RED .. TBG.Text.Label.Tooltip.HINT_TOOLTIP;
	end

	-- Display hint on battleground history.
	hint = hint .. "\n"  .. TBG.Var.Const.COLOR_RED .. TBG.Text.Label.Tooltip.HINT_HISTORY;

	return "\n" .. hint;
end


-- ####################
-- ## SLASH COMMANDS ##
-- ####################


-- ## Prints help on slash commands.
function TBG_PrintSlashCommandHelp(t)
	local message = string.gsub(TBG.Text.Message.SLASH_HEADER, "{addon}", TBG.Var.Const.COLOR_BLUE .. TBG.Text.ADDON_NAME, 1);
		  message = string.gsub(message, "{version}", TBG.Var.Const.COLOR_WHITE .. "v" .. TBG.Text.ADDON_VERSION .. TBG.Var.Const.COLOR_YELLOW, 1);
		  message = string.gsub(message, "'n'", "'" .. TBG.Var.Const.COLOR_GREY .. "n" .. TBG.Var.Const.COLOR_YELLOW .. "'", 1);

	DEFAULT_CHAT_FRAME:AddMessage("-");
	DEFAULT_CHAT_FRAME:AddMessage(message);
	DEFAULT_CHAT_FRAME:AddMessage(" ");

	TBG_PrintSlashCommand(t);

	DEFAULT_CHAT_FRAME:AddMessage("-");
end


-- ## Print a slash command.
function TBG_PrintSlashCommand(t)
	local message = t.expr;

	if (message and t.details) then
		message = message .. TBG.Var.Const.COLOR_YELLOW .. " - " .. t.details;
		message = string.gsub(message, "'n'", "'" .. TBG.Var.Const.COLOR_GREY .. "n" .. TBG.Var.Const.COLOR_YELLOW .. "'");
	end

	if (message) then
		DEFAULT_CHAT_FRAME:AddMessage(message);
	end

	local i = 1;

	while (true) do
		if (t[i]) then
			TBG_PrintSlashCommand(t[i]);

			i = i + 1;
		else
			break;
		end
	end
end


-- ## Processes slash commands.
function TBG_SlashProcess(command)
	if (not TBG.Var.Addon.LOADED)                                                                 then return nil; end
	if (not TBG_IsEnabled())                                                                      then return nil; end
	if (TBG.Var.Player.zone == nil or TBG.Var.Player.zone ~= TBG.Var.Addon.ZONE_CHANGED_NEW_AREA) then return nil; end

	if (not command or command == "") then
		TBG_Menu_Toggle();

	else
		local arg                            = { };
		      arg[1], arg[2], arg[3], arg[4] = string.split(" ", command);

		-- Resetting commands.
		if (string.lower(arg[1]) == TBG.Var.Const.Slash[1].command) then
			TBG_PrintSlashCommandHelp(TBG.Var.Const.Slash);

		elseif (string.lower(arg[1]) == TBG.Var.Const.Slash[3].command) then

			-- Reset position of draggable frames.
			if (arg[2] ~= nil and string.lower(arg[2]) == TBG.Var.Const.Slash[3][1].command) then
				TBG_ResetOnScreenDragFramePositions();

			-- Reset battleground history.
			elseif (arg[2] ~= nil and string.lower(arg[2]) == TBG.Var.Const.Slash[3][2].command) then
				TBG_Init_CreateBattlegroundHistory();

			-- Reset saved variables.
			elseif (arg[2] ~= nil and string.lower(arg[2]) == TBG.Var.Const.Slash[3][3].command) then
				TBG_ResetSavedVariables();
				ReloadUI();
			else
				TBG_PrintSlashCommandHelp(TBG.Var.Const.Slash[3]);
			end

		-- Printing commands.
		elseif (string.lower(arg[1]) == TBG.Var.Const.Slash[2].command) then

			-- Print flag carriers.
			if (arg[2] ~= nil and string.lower(arg[2]) == TBG.Var.Const.Slash[2][1].command) then
				if (arg[3] ~= nil and string.lower(arg[3]) == TBG.Var.Const.Slash[2][1][1].command) then
					if (TBG.DB.players.names[TBG.DB.active.stats.bg.stats.alliance.player .. "|Alliance"] and TBG.Var.Player.zone == TBG.Text.Ref.BG_WSG) then
						TBG_Battleground_PrintFlagCarrier(FACTION_HORDE, TBG.DB.active.stats.bg.stats.alliance.player, TBG.DB.players.names[TBG.DB.active.stats.bg.stats.alliance.player .. "|Alliance"].class);
					end
				elseif (arg[3] ~= nil and string.lower(arg[3]) == TBG.Var.Const.Slash[2][1][2].command) then
					if (TBG.DB.players.names[TBG.DB.active.stats.bg.stats.horde.player .. "|Horde"] and TBG.Var.Player.zone == TBG.Text.Ref.BG_WSG) then
						TBG_Battleground_PrintFlagCarrier(FACTION_ALLIANCE, TBG.DB.active.stats.bg.stats.horde.player, TBG.DB.players.names[TBG.DB.active.stats.bg.stats.horde.player .. "|Horde"].class);
					end
				else
					TBG_PrintSlashCommandHelp(TBG.Var.Const.Slash[2][1]);
				end

			-- Print capture timers.
			elseif (arg[2] ~= nil and string.lower(arg[2]) == TBG.Var.Const.Slash[2][2].command) then

				-- Print all or the top 'n' timers.
				if (arg[3] ~= nil and string.lower(arg[3]) == TBG.Var.Const.Slash[2][2][1].command) then
					if (TBG.Var.Player.zone == TBG.Text.Ref.BG_AB or TBG.Var.Player.zone == TBG.Text.Ref.BG_AV) then
						TBG_Battleground_PrintCaptureTimers();
					end

				elseif (arg[3] ~= nil and string.lower(arg[3]) == TBG.Var.Const.Slash[2][2][2].command) then
					if (arg[4] ~= nil and tonumber(arg[4])) then
						if (TBG.Var.Player.zone == TBG.Text.Ref.BG_AB or TBG.Var.Player.zone == TBG.Text.Ref.BG_AV) then
							TBG_Battleground_PrintCaptureTimers(arg[4]);
						end
					else
						TBG_PrintSlashCommandHelp(TBG.Var.Const.Slash[2][2][2]);
					end

				-- Print a specific timer.
				elseif (arg[3] ~= nil and tonumber(arg[3])) then
					if (TBG.Var.Player.zone == TBG.Text.Ref.BG_AB or TBG.Var.Player.zone == TBG.Text.Ref.BG_AV) then
						local valid = 0;
						local found = 0;

						for i = 1, table.getn(TBG.DB.active.captures) do

							-- If the capture was done while the player was not there, show unknown.
							if (TBG.DB.active.captures[i].cap ~= 0) then
								valid = valid + 1;

								if (tonumber(valid) == tonumber(arg[3])) then
									found = i;
									break;
								end
							end
						end

						if (found > 0) then
							local eta = TBG_SecondsToTime(TBG.DB.active.captures[found].cap - GetTime());

							-- If there is a capture imminent, indicate.
							if (eta == "") then
								eta = TBG.Text.Label.Tooltip.CAPTURE_TIMERS_IMINENT;
							end

							TBG_Battleground_PrintCaptureTimer(TBG.DB.active.captures[found].who, TBG.DB.active.captures[found].name, eta);
						end
					end
				else
					TBG_PrintSlashCommandHelp(TBG.Var.Const.Slash[2][2]);
				end

			-- Print movement timers.
			elseif (arg[2] ~= nil and string.lower(arg[2]) == TBG.Var.Const.Slash[2][3].command) then

				-- Ivus the Forest Lord
				if (arg[3] ~= nil and string.lower(arg[3]) == TBG.Var.Const.Slash[2][3][1].command) then
					if (TBG.Var.Player.zone == TBG.Text.Ref.BG_AV and TBG.DB.active.stats.bg.stats.summons["IVUS"] ~= 0) then
						local mtime = TBG.Text.Label.Button.READY;

						if (TBG.Var.BG.AV.time_to_movement - (GetTime() - TBG.DB.active.stats.bg.stats.summons["IVUS"]) > 0) then
							mtime = TBG_SecondsToTime(TBG.Var.BG.AV.time_to_movement - (GetTime() - TBG.DB.active.stats.bg.stats.summons["IVUS"]));
						end

						TBG_Battleground_PrintMovementTimer(TBG.Text.Label.Other.IVUS, mtime);
					end

				-- Lokholar the Ice Lord
				elseif (arg[3] ~= nil and string.lower(arg[3]) == TBG.Var.Const.Slash[2][3][2].command) then
					if (TBG.Var.Player.zone == TBG.Text.Ref.BG_AV and TBG.DB.active.stats.bg.stats.summons["LOK"] ~= 0) then
						local mtime = TBG.Text.Label.Button.READY;

						if (TBG.Var.BG.AV.time_to_movement - (GetTime() - TBG.DB.active.stats.bg.stats.summons["LOK"]) > 0) then
							mtime = TBG_SecondsToTime(TBG.Var.BG.AV.time_to_movement - (GetTime() - TBG.DB.active.stats.bg.stats.summons["LOK"]));
						end

						TBG_Battleground_PrintMovementTimer(TBG.Text.Label.Other.LOKHOLAR, mtime);
					end
				else
					TBG_PrintSlashCommandHelp(TBG.Var.Const.Slash[2][3]);
				end
			else
				TBG_PrintSlashCommandHelp(TBG.Var.Const.Slash[2]);
			end
		else
			TBG_PrintSlashCommandHelp(TBG.Var.Const.Slash);
		end
	end
end