TBG_Files_Battlegrounds_IsLoaded = true; -- Indicates this file has been loaded into memory.

-- #############
-- ## GENERAL ##
-- #############


	-- ## Stores information about the instances that were active when a user looked at the queue window.
	function TBG_Battleground_StoreNumberOfOpenInstances()
		if (not TBG_IsBattleground(TBG.Var.Player.zone)) then return nil; end

		local name, _, _, _, _, _, _, _, _ = GetBattlefieldInfo();
		local instances                    = GetNumBattlefields();
		local bg                           = TBG.DB.bgs.names[name];

		if (bg ~= nil) then
			bg.open_instances = instances;
			bg.open_age       = GetTime();
		end
	end


-- ###############
-- ## INACTIVES ##
-- ###############


	-- ## If a player talks in a battleground, mark as if they have moved.
	function TBG_Battleground_InactivesSetChatAuthorActive(player_name)
		if (not TBG_IsBattleground(TBG.Var.Player.zone)) then return nil; end

		if (player_name ~= nil) then
			local player_id = "";
			local t         = GetTime();

			if (string.find(player_name, "-")) then
				player_name = string.sub(player_name, 1, string.find(player_name, "-") - 1);
			end

			if (TBG.Var.Player.faction == TBG.Var.Const.HORDE) then
				player_id = player_name .. "|Horde";
			else
				player_id = player_name .. "|Alliance";
			end

			if (TBG.DB.players.names[player_id]) then
				TBG.DB.players.names[player_id].last_movement = t;

				-- Check if the player is inactive.
				if ((t - TBG.DB.players.names[player_id].last_movement) >= TBG.Var.BG.inactive_delay) then
					if (not TBG.DB.players.names[player_id].inactive and TBG.DB.inactives.names[player_id] == nil) then
						table.insert(TBG.DB.inactives.list, player_id);
						TBG.DB.inactives.names[player_id] = table.getn(TBG.DB.inactives.list);
					end

					TBG.DB.players.names[player_id].inactive = true;
				else
					TBG.DB.players.names[player_id].inactive = false;
				end
			end
		end
	end


	-- ## Sorts the list of inactives alphabetically.
	function TBG_Battleground_InactivesSort(list)
		if (not TBG_IsBattleground(TBG.Var.Player.zone)) then return list; end

		table.sort(list, function (v1, v2)
			return v1 < v2;
		end);

		return list;
	end


-- ##########
-- ## RAID ##
-- ##########


	-- ## Loops through the raid and stores the raid location of each member in the player list.
	function TBG_Battleground_RaidScan()
		if (not UnitInRaid("player")) then return nil; end

		TBG.Var.BG.scanning_raid = true;

		for i = 1, GetNumRaidMembers() do
			local name = GetRaidRosterInfo(i);

			if (name) then
				local player_id = name .. "|" .. TBG.Var.Player.faction;

				if (TBG.DB.players.names[player_id]) then
					TBG.DB.players.names[player_id].raid = i;
				end
			end
		end
	end


	-- ## Finds the location of a specific member in a raid.
	function TBG_Battleground_RaidGetPlayerLocation(name)
		if (not TBG_IsBattleground(TBG.Var.Player.zone)) then return -1; end

		for i = 1, GetNumRaidMembers() do
			local n = GetRaidRosterInfo(i);

			if (n == name) then
				return i;
			end
		end

		return -1;
	end


-- #############
-- ## SERVERS ##
-- #############


	-- ## Sorts the server list from the most populated, to the least.
	function TBG_Battleground_ServersSortByCount(list)
		if (not TBG_IsBattleground(TBG.Var.Player.zone)) then return list; end

		table.sort(list.index, function (v1, v2)
			if (list.reference[v1] == list.reference[v2]) then
				return v1 < v2;
			else
				return list.reference[v1] > list.reference[v2];
			end
       	end);

		return list;
	end


-- #########
-- ## MAP ##
-- #########


	-- ## Stores existing capture points.
	function TBG_Battleground_MapCheckExistingPoints()
		if (not TBG_IsBattleground(TBG.Var.Player.zone))                                 then return nil; end
		if (not TBG.Var.Const.BATTLEGROUNDS_INFORMATION[TBG.Var.Player.zone].has_points) then return nil; end

		TBG.Var.BG.onload_capture_points_stored = false;
		TBG_Battleground_MapCheckPoints(TBG.Var.Player.zone, true);
		TBG.Var.BG.onload_capture_points_stored = true;
	end


	-- ## Checks on the location in the battleground and their capture status.
	function TBG_Battleground_MapCheckPoints(bg_name, just_entered)
		if (not TBG_IsBattleground(TBG.Var.Player.zone))                                 then return nil; end
		if (not TBG.Var.Const.BATTLEGROUNDS_INFORMATION[TBG.Var.Player.zone].has_points) then return nil; end
		if (GetBattlefieldWinner() ~= nil)                                               then return nil; end

		local marks = GetNumMapLandmarks();
		local delay = TBG.Var.Const.BATTLEGROUNDS_INFORMATION[bg_name].point_capture_delay;

		-- Reset base owned tracking variables.
		TBG.DB.active.stats.bg.stats.bases_alliance = 0;
		TBG.DB.active.stats.bg.stats.bases_horde    = 0;

		-- If this is a cap on enter, reset capture information.
		if (just_entered) then
			TBG.DB.active.captures = { };
		end

		for i = 1, marks, 1 do
			local name, _, txture, x, y = GetMapLandmarkInfo(i);
			local owned                 = TBG_Battleground_MapPointOwnedBy(txture);

			if (name == "Friedhof des Sturmlanzen") then
				name = "Friedhof der Sturmlanzen";
			end

			-- Determine who this item is owned by and if we are in ab, increase the count for that faction.
			if (bg_name == TBG.Text.Ref.BG_AB) then
				if (owned == "Alliance") then
					TBG.DB.active.stats.bg.stats.bases_alliance = TBG.DB.active.stats.bg.stats.bases_alliance + 1;
				elseif (owned == "Horde") then
					TBG.DB.active.stats.bg.stats.bases_horde = TBG.DB.active.stats.bg.stats.bases_horde + 1;
				end
			end

			-- Set up the local capture information.
			local c = {
				name         = "",    -- What is the name of this item.
				cap          = 0,     -- Time when the point will be capped.
				who          = "",    -- Who the point will be capped by.
				t            = 15,    -- The texture of the cap.
				enter        = false, -- If the point was capped before entering the battleground.
			};

			-- Store data.
			c.name         = name;
			c.who, c.text  = TBG_Battleground_MapPointCappedBy(txture);
			c.t            = txture;

			-- Location of the capture in the capture's list.
			local loc = -1;

			-- Loop through and find the capture if it is in the list.
			if (table.getn(TBG.DB.active.captures) > 0) then
				for j = 1, table.getn(TBG.DB.active.captures) do

					if (TBG.DB.active.captures[j].name == c.name) then
						loc = j;
						break;
					end
				end
			end

			-- If this is a new item being contested, store data.
			if (loc == -1 and c.who ~= "") then
				c.cap = GetTime() + delay;

				-- If the time capped is known, store the time to cap.
				if (just_entered) then
					c.enter = true;
				end

				table.insert(TBG.DB.active.captures, c);

			-- Otherwise, remove any item not contested.
			elseif (loc > -1 and (c.who == "" or (c.who ~= "" and TBG.DB.active.captures[loc].who ~= c.who))) then
				table.remove(TBG.DB.active.captures, loc);
			end
		end
	end


	function TBG_Battleground_MapCheckFlags()
		if (not TBG_IsBattleground(TBG.Var.Player.zone))                               then return nil; end
		if (not TBG.Var.Const.BATTLEGROUNDS_INFORMATION[TBG.Var.Player.zone].has_flag) then return nil; end

		-- Collect flag information.
		TBG.DB.active.stats.bg.stats.alliance.flag = GetWorldStateUIInfo(1);
		TBG.DB.active.stats.bg.stats.horde.flag    = GetWorldStateUIInfo(2);

		local flag_picked_up = TBG.Text.Ref.WSG_FLAG_PICKED_UP;
		local flag_dropped   = TBG.Text.Ref.WSG_FLAG_DROPPED;
		local flag_captured  = TBG.Text.Ref.WSG_FLAG_CAPTURED;

		if (TBG.Var.Player.zone == TBG.Text.Ref.BG_EOTS) then
			flag_picked_up = TBG.Text.Ref.EOTS_FLAG_PICKED_UP;
			flag_dropped   = TBG.Text.Ref.EOTS_FLAG_DROPPED;
			flag_captured  = TBG.Text.Ref.EOTS_FLAG_CAPTURED;
		end

		-- Collect alliance flag information.
		if (TBG.DB.active.stats.bg.stats.alliance.msg ~= "") then

			-- Flag picked up. The faction the message is for is the same as the faction of the player carrying the flag.
			-- "The Horde flag was picked up by Maina!".
			if (string.find(TBG.DB.active.stats.bg.stats.alliance.msg, flag_picked_up)) then
				string.gsub(TBG.DB.active.stats.bg.stats.alliance.msg, flag_picked_up, function(a, b)

					if (GetLocale() == "deDE" or GetLocale() == "frFR" or TBG.Var.Player.zone == TBG.Text.Ref.BG_EOTS) then
						TBG.DB.active.stats.bg.stats.alliance.player = string.trim(a);

						if (TBG.Var.Player.zone == TBG.Text.Ref.BG_EOTS) then
							TBG.DB.active.stats.bg.stats.horde.player = "";
						end

					elseif (GetLocale() == "enUS") then
						TBG.DB.active.stats.bg.stats.alliance.player = string.trim(b);
					end
				end);

			-- Flag dropped. The faction the message is for is the faction that owns the flag. The player dropping the flag is the enemy.
			-- "The Alliance flag was dropped by Maina!".
			elseif (string.find(TBG.DB.active.stats.bg.stats.alliance.msg, flag_dropped)) then
				TBG.DB.active.stats.bg.stats.horde.player = "";

			-- Flag captured. The faction the message is for is the same as the faction of the player carrying the flag.
			-- "Maina captured the Horde flag!".
			elseif (string.find(TBG.DB.active.stats.bg.stats.alliance.msg, flag_captured)) then
				TBG.DB.active.stats.bg.stats.alliance.player = "";

				if (TBG.Var.Player.zone == TBG.Text.Ref.BG_EOTS) then
					TBG.DB.active.stats.bg.stats.flags_captured.alliance = TBG.DB.active.stats.bg.stats.flags_captured.alliance + 1;
				end
			end
		end

		-- Collect horde flag information.
		if (TBG.DB.active.stats.bg.stats.horde.msg ~= "") then

			-- Flag picked up. The faction the message is for is the same as the faction of the player carrying the flag.
			-- "The Alliance flag was picked up by Maina!".
			if (string.find(TBG.DB.active.stats.bg.stats.horde.msg, flag_picked_up)) then
				string.gsub(TBG.DB.active.stats.bg.stats.horde.msg, flag_picked_up, function(a, b)

					if (GetLocale() == "deDE" or GetLocale() == "frFR" or TBG.Var.Player.zone == TBG.Text.Ref.BG_EOTS) then
						TBG.DB.active.stats.bg.stats.horde.player = string.trim(a);

						if (TBG.Var.Player.zone == TBG.Text.Ref.BG_EOTS) then
							TBG.DB.active.stats.bg.stats.alliance.player = "";
						end

					elseif (GetLocale() == "enUS") then
						TBG.DB.active.stats.bg.stats.horde.player = string.trim(b);
					end
				end);

			-- Flag dropped. The faction the message is for is the faction that owns the flag. The player dropping the flag is the enemy.
			-- "The Horde flag was dropped by Maina!".
			elseif (string.find(TBG.DB.active.stats.bg.stats.horde.msg, flag_dropped)) then
				TBG.DB.active.stats.bg.stats.alliance.player = "";

			-- Flag captured. The faction the message is for is the same as the faction of the player carrying the flag.
			-- "Maina captured the Alliance flag!".
			elseif (string.find(TBG.DB.active.stats.bg.stats.horde.msg, flag_captured)) then
				TBG.DB.active.stats.bg.stats.horde.player = "";

				if (TBG.Var.Player.zone == TBG.Text.Ref.BG_EOTS) then
					TBG.DB.active.stats.bg.stats.flags_captured.horde = TBG.DB.active.stats.bg.stats.flags_captured.horde + 1;
				end
			end
		end

		if (TBG.DB.active.stats.bg.stats.alliance.msg ~= "" or TBG.DB.active.stats.bg.stats.horde.msg ~= "") then
			TBG.DB.active.stats.bg.stats.horde.msg    = "";
			TBG.DB.active.stats.bg.stats.alliance.msg = "";
		end
	end


	-- ## Used to check positioning for the minimap flags. Debug function.
	function TBG_Debug_Battleground_MinimapPlaceFlag()
		local x, y = GetPlayerMapPosition("player");

		TBG_Print("x = " .. x);
		TBG_Print("y = " .. y);

		TBGX_Stuff:SetPoint("CENTER", Minimap, "CENTER", 0, 0);
	end


	-- ## If a flag is currently being carried by a player, pleace this flag on the minimap.
	function TBG_Battleground_MinimapPlaceFlag()
		if (not TBG_IsBattleground(TBG.Var.Player.zone))                               then return nil; end
		if (not TBG.Var.Const.BATTLEGROUNDS_INFORMATION[TBG.Var.Player.zone].has_flag) then return nil; end
		if (GetBattlefieldWinner() ~= nil)                                             then return nil; end

		local initial    = TBG.Var.Const.BATTLEGROUNDS_INFORMATION[TBG.Var.Player.zone].stats.minimap.initial;
		local difference = TBG.Var.Const.BATTLEGROUNDS_INFORMATION[TBG.Var.Player.zone].stats.minimap.difference;
		local faction    = { };

		local distance = {
			[0] = { x = initial.x + (difference.x * 6), y = initial.y + (difference.y * 6), },
			[1] = { x = initial.x + (difference.x * 5), y = initial.y + (difference.y * 5), },
			[2] = { x = initial.x + (difference.x * 4), y = initial.y + (difference.y * 4), },
			[3] = { x = initial.x + (difference.x * 3), y = initial.y + (difference.y * 3), },
			[4] = { x = initial.x + (difference.x * 2), y = initial.y + (difference.y * 2), },
			[5] = { x = initial.x + (difference.x),     y = initial.y + (difference.y), },
		};

		local x, y = GetPlayerMapPosition("player");

		local icon = 0;
		local last = 0;

		faction["AllianceFlag"] = { name = TBG.DB.active.stats.bg.stats.alliance.player, faction = FACTION_ALLIANCE, };
		faction["HordeFlag"]    = { name = TBG.DB.active.stats.bg.stats.horde.player,    faction = FACTION_HORDE,    };

		-- Flip flags if Warsong Gulch.
		if (TBG.Var.Player.zone == TBG.Text.Ref.BG_WSG) then
			local old = faction["AllianceFlag"];

			faction["AllianceFlag"] = faction["HordeFlag"];
			faction["HordeFlag"]    = old;
		end

		for i = 1, GetNumBattlefieldFlagPositions() do
			local ox, oy, token = GetBattlefieldFlagPosition(i);
			local f             = token;
			local display       = true;

			-- Check that this flag type can be displayed.
			if (string.find(token, "Horde") and not TBG_Saved("MDB_Other_MinimapFlag_Horde"))       then display = false; end
			if (string.find(token, "Alliance") and not TBG_Saved("MDB_Other_MinimapFlag_Alliance")) then display = false; end

			if (display) then
				local player  = TBG.DB.players.names[faction[f].name .. "|" .. TBG.Var.Const.FACTION_NAME.from[faction[f].faction]];
				local texture = "Interface\\WorldStateFrame\\" .. token;
				local zoom    = Minimap:GetZoom();

				-- If the player carrying the flag isn't the current player, or the player is unknown.
				if (not player or (player and player.name ~= TBG.Var.Player.name)) then
					icon = icon + 1;

					-- Get the x an y coordinates of the object from the player on the minimap in pixels.
					local xCoord = (ox - x) / distance[zoom].x;
					local yCoord = (oy - y) / distance[zoom].y;

					-- Calculate the distance in pixels of the coordinate from the player.
					local a = xCoord;
					local b = yCoord;

					if (a < 0) then a = a * -1; end
					if (b < 0) then b = b * -1; end

					local c        = (a * a) + (b * b);
					local distance = math.sqrt(c);
					local radius   = 58;

					-- Get the button.
					local button = getglobal("TBGX_Display_MinimapFlags_" .. icon);
					local edge   = false;

					-- If out of bounds of the minimap, position on the edge.
					if (distance >= radius) then
						local factor = radius / distance;

						xCoord = xCoord * factor;
						yCoord = yCoord * factor;
						edge   = true;
					end

					if (edge and TBG_Saved("MDB_Other_MinimapFlag_OutOfBoundsFade")) then
						getglobal(button:GetName() .. "_Icon"):SetAlpha(0.7);
					else
						getglobal(button:GetName() .. "_Icon"):SetAlpha(1);
					end

					if (edge and not TBG_Saved("MDB_Other_MinimapFlag_AllowOutOfBounds")) then
						button:Hide();
					else
						button:Show();
						getglobal(button:GetName() .. "_Icon"):SetTexture(texture);

						if (TBG_Saved("MDB_Other_MinimapFlag_AllowMouse")) then
							if (player) then
								local color = TBG.Var.Const.COLOR_RED;
								local class = TBG.Var.Const.COLOR_WHITE .. " - " .. TBG.Var.Const.COLOR_CLASS[TBG.Var.Const.NAME_CLASS[player.class]].color .. player.class;

								if (player.faction == FACTION_ALLIANCE) then
									color = TBG.Var.Const.COLOR_BLUE;
								end

								-- Set on click script.
								button:SetAttribute("type1", "macro");
								button:SetAttribute("macrotext", "/target " .. player.name);

								button.tooltip = color .. player.name .. class;
								button:EnableMouse(true);

							else
								button.tooltip = nil;

								button:SetAttribute("type1", "macro");
								button:SetAttribute("macrotext", "");
								button:EnableMouse(false);
							end
						end

						button:ClearAllPoints();
						button:SetPoint("CENTER", Minimap, "CENTER", xCoord, -yCoord);

						last = icon;
					end
				end
			end
		end

		last = last + 1;

		-- Hide unused flag tokens.
		while (true) do
			if (getglobal("TBGX_Display_MinimapFlags_" .. last)) then
				getglobal("TBGX_Display_MinimapFlags_" .. last):Hide();

				last = last + 1;
			else
				break;
			end
		end

		return true;
	end


	-- TEXTURE KEY (Alterac Valley)
	--
	-- Mine
	--     0: None
	--     1: Horde
	--     2: Alliance
	--
	-- Graveyard
	--      7: Free
	--     12: Horde
	--      3: Contested (Horde, Assulted by the Alliance)
	--     14: Alliance
	--     13: Contested (Alliance, Assulted by the Horde)
	--
	-- Tower
	--      9: Horde
	--      8: Contested (Horde, Assulted by the Alliance)
	--     10: Alliance
	--     11: Contested (Alliance, Assulted by the Horde)

	-- TEXTURE KEY (Arathi Basin)
	--
	-- Lumber Mill
	--     21: None
	--     25: Horde
	--     22: Contested (Horde, Assulted by the Alliance)
	--     23: Alliance
	--     24: Contested (Alliance, Assulted by the Horde)
	--
	-- Mine
	--     16: None
	--     20: Horde
	--     17: Contested (Horde, Assulted by the Alliance)
	--     18: Alliance
	--     19: Contested (Alliance, Assulted by the Horde)
	--
	-- Blacksmith
	--     26: None
	--     30: Horde
	--     27: Contested (Horde, Assulted by the Alliance)
	--     28: Alliance
	--     29: Contested (Alliance, Assulted by the Horde)
	--
	-- Farm
	--     31: None
	--     35: Horde
	--     32: Contested (Horde, Assulted by the Alliance)
	--     33: Alliance
	--     34: Contested (Alliance, Assulted by the Horde)
	--
	-- Stables
	--     36: None
	--     40: Horde
	--     37: Contested (Horde, Assulted by the Alliance)
	--     38: Alliance
	--     39: Contested (Alliance, Assulted by the Horde)


	-- ## Determines who the point will be capped by. If not under contestation, return a zero length string.
	function TBG_Battleground_MapPointCappedBy(t)
		if (TBG_IsBattleground(TBG.Var.Player.zone) and not TBG.Var.Const.BATTLEGROUNDS_INFORMATION[TBG.Var.Player.zone].has_points) then return nil; end

		local capped = "";
		local text   = "";

		if (t == 3 or t == 8 or t == 22 or t == 17 or t == 27 or t == 32 or t == 37) then
			capped = "Alliance";
			text   = FACTION_ALLIANCE;
		elseif (t == 13 or t == 11 or t == 24 or t == 19 or t == 29 or t == 34 or t == 39) then
			capped = "Horde";
			text   = FACTION_HORDE;
		end

		return capped, text;
	end


	-- ## Determines who owns the point, if it is not under contestation. Otherwise, returns a zero length string.
	function TBG_Battleground_MapPointOwnedBy(t)
		if (not TBG_IsBattleground(TBG.Var.Player.zone))                                 then return nil; end
		if (not TBG.Var.Const.BATTLEGROUNDS_INFORMATION[TBG.Var.Player.zone].has_points) then return nil; end

		local owned = "";

		if (t == 2 or t == 14 or t == 10 or t == 23 or t == 18 or t == 28 or t == 33 or t == 38) then
			owned = "Alliance";
		elseif (t == 1 or t == 12 or t == 9 or t == 25 or t == 20 or t == 30 or t == 35 or t == 40) then
			owned = "Horde";
		end

		return owned;
	end


-- ##################
-- ## ARATHI BASIN ##
-- ##################


	-- ## Determines who is likely to win the Arathi Basin or Eye of the Storm game.
	-- ##
	-- ## Variables
	-- ##     * b_alliance: Number of bases held by the Alliance.
	-- ##     * b_horde: Number of bases held by the Horde.
	-- ##     * a_extra: Extra resources to add to the alliance side when performing a future prediction check.
	-- ##     * h_extra: Extra resources to add to the horde side when performing a future prediction check.
	-- ##
	-- ## Returns
	-- ##     * The team that will win. Nil/0/1. Reference TBG.Var.Const.NAME_FACTION.
	-- ##     * The amount of time it will take them to win. Nil/time in seconds.
	-- ##     * Final score held by the Alliance.
	-- ##     * Final score held by the Horde.
	function TBG_Battleground_Resources_GetWinner(b_alliance, b_horde, a_extra, h_extra)
		if (not TBG_IsBattleground(TBG.Var.Player.zone))                                               then return nil; end
		if (TBG.Var.Player.zone ~= TBG.Text.Ref.BG_AB and TBG.Var.Player.zone ~= TBG.Text.Ref.BG_EOTS) then return nil; end

		if (TBG.DB.world_state[1] and TBG.DB.world_state[1].active) then
			local a_resources = TBG.DB.world_state[1].resources;
			local h_resources = TBG.DB.world_state[2].resources;

			-- If we are checking possible future scenarios, add the extra resources.
			if (a_extra ~= nil and h_extra ~= nil) then
				a_resources = a_resources + a_extra;
				h_resources = h_resources + h_extra;
			end

			-- Get the amount of resources per second.
			local a_res_sec = TBG_Battleground_Resources_GetResourcesPerSecond(b_alliance);
			local h_res_sec = TBG_Battleground_Resources_GetResourcesPerSecond(b_horde);

			-- The amount of time left until the team gathers 2000 resources.
			local a_timeleft = -1;
			local h_timeleft = -1;

			-- Catch division by zero exceptions.
			if (a_res_sec > 0) then
				a_timeleft = (2000 - a_resources) / a_res_sec;
			end

			if (h_res_sec > 0) then
				h_timeleft = (2000 - h_resources) / h_res_sec;
			end

			-- Determine final scores at the end.
			local a_final = a_resources;
			local h_final = h_resources;

			if (a_res_sec > 0) then
				a_final = math.floor((a_final + (math.floor(h_timeleft / 30) * (a_res_sec * 30))) / 10) * 10;
			end

			if (h_res_sec > 0) then
				h_final = math.floor((h_final + (math.floor(a_timeleft / 30) * (h_res_sec * 30))) / 10) * 10;
			end

			-- Unknown who will win.
			if (a_timeleft == -1 and h_timeleft == -1) then
				return nil, nil, nil, nil;

			-- Alliance will win.
			elseif ((a_timeleft > -1 and a_timeleft < h_timeleft) or (a_timeleft > -1 and h_timeleft == -1)) then
				return "Alliance", a_timeleft, 2000, h_final;

			-- Horde will win.
			elseif ((h_timeleft > -1 and h_timeleft < a_timeleft) or (h_timeleft > -1 and a_timeleft == -1)) then
				return "Horde", h_timeleft, a_final, 2000;
			end
		end

		-- Otherwise return unknown.
		return nil, nil, nil, nil;
	end


	-- ## Determines how many resources per second a team gets with the amount of bases they're holding.
	function TBG_Battleground_Resources_GetResourcesPerSecond(bases)
		if (not TBG_IsBattleground(TBG.Var.Player.zone)) then return 0; end
		if (not bases)                                   then return 0; end

		bases = bases + 0;

		if (TBG.Var.Player.zone == TBG.Text.Ref.BG_AB) then
			if (bases == 1) then
				return (25.0 / 30.0);
			elseif (bases == 2) then
				return (300.0 / 9.0) / 30.0;
			elseif (bases == 3) then
				return (50.0  / 30.0);
			elseif (bases == 4) then
				return (100.0 / 30.0);
			elseif (bases == 5) then
				return (900.0 / 30.0);
			else
				return 0;
			end

		elseif (TBG.Var.Player.zone == TBG.Text.Ref.BG_EOTS) then
			if (bases == 1) then
				return (0.5);
			elseif (bases == 2) then
				return (1);
			elseif (bases == 3) then
				return (2.5);
			elseif (bases == 4) then
				return (5);
			else
				return 0;
			end

		else
			return 0;
		end
	end


	-- ## Determines how many nodes are required to win.
	-- ##
	-- ## Returns
	-- ##     * TBG.Text.Label.Tooltip.AB_IMPOSSIBLE: If not possible to win.
	-- ##                          OR: Number of nodes required to win.
	function TBG_Battleground_Resources_NodesRequiredToWin(faction, a_resources, h_resources)
		if (not TBG_IsBattleground(TBG.Var.Player.zone))                                               then return 0; end
		if (TBG.Var.Player.zone ~= TBG.Text.Ref.BG_AB and TBG.Var.Player.zone ~= TBG.Text.Ref.BG_EOTS) then return 0; end

		if (TBG.DB.world_state[1] and TBG.DB.world_state[1].active) then
			local nodes = TBG.Text.Label.Tooltip.AB_IMPOSSIBLE;
			local bases = 0;
			local e     = 0;

			-- Determine which faction we are checking is.
			if (faction == TBG.Var.Const.ALLIANCE) then
				bases = TBG.DB.world_state[1].bases;
			else
				bases = TBG.DB.world_state[2].bases;
			end

			if (TBG.Var.Player.zone ~= TBG.Text.Ref.BG_AB) then
				e = 5 - bases;
			elseif (TBG.Var.Player.zone ~= TBG.Text.Ref.BG_EOTS) then
				e = 4 - bases;
			end

			-- Try different base numbers.
			for i = 1, e do
				local winner = nil;

				if (faction == TBG.Var.Const.ALLIANCE) then
					winner = TBG_Battleground_Resources_GetWinner(bases + i, TBG.DB.world_state[2].bases - i, a_resources, h_resources);
				else
					winner = TBG_Battleground_Resources_GetWinner(TBG.DB.world_state[1].bases - i, bases + i, a_resouces, h_resources);
				end

				-- If the winner is the faction, then stop.
				if (winner == faction) then
					nodes = i;
					break;
				end
			end

			return nodes;
		end
	end


	-- ## Determines how long it will take until the amount of nodes needing to be captured in order to win increases.
	-- ##
	-- ## Variables
	-- ##     * faction: The faction we are checking.
	-- ##     * initial: The number of nodes that need to be captured by the faction in order to win.
	function TBG_Battleground_Resources_TimeLeftToCapture(faction, initial, seconds)
		if (not TBG_IsBattleground(TBG.Var.Player.zone))                                               then return false; end
		if (TBG.Var.Player.zone ~= TBG.Text.Ref.BG_AB and TBG.Var.Player.zone ~= TBG.Text.Ref.BG_EOTS) then return false; end

		local nodes       = nil;
		local complete    = false;

		local a_resources = 0;
		local h_resources = 0;

		-- Get the amount of resources per second.
		local a_res_sec = TBG_Battleground_Resources_GetResourcesPerSecond(TBG.DB.world_state[1].bases);
		local h_res_sec = TBG_Battleground_Resources_GetResourcesPerSecond(TBG.DB.world_state[2].bases);

		-- Get the amount of resources produced in this time period.
		a_resources = a_res_sec * seconds;
		h_resources = h_res_sec * seconds;

		-- Check the amount of nodes
		nodes = TBG_Battleground_Resources_NodesRequiredToWin(faction, a_resources, h_resources);

		if (nodes ~= initial) then
			complete = true;
		end

		return complete;
	end


	-- ## Checks the amount of time until the amount of nodes needing to be captured increases.
	-- ## Cannot be done as a basic loop as it can lock up the game.
	function TBG_Battleground_Resources_CheckTimeUntilNodeIncrease()
		if (not TBG_IsBattleground(TBG.Var.Player.zone))                                               then return false; end
		if (TBG.Var.Player.zone ~= TBG.Text.Ref.BG_AB and TBG.Var.Player.zone ~= TBG.Text.Ref.BG_EOTS) then return false; end

		if (GetTime() - TBG.Var.BG.AB.timeinc_update >= TBG.Var.BG.AB.timeing_update_delay) then
			TBG.Var.BG.AB.timeinc_update = GetTime();

			if (not TBG.Var.BG.AB.timeinc_check) then
				 TBG.Var.BG.AB.timeinc_sec   = 0;
				 TBG.Var.BG.AB.timeinc_check = true;
			end

			if (TBG.Var.Player.zone ~= TBG.Text.Ref.BG_AB) then
				TBG.Var.BG.AB.timeinc_sec = TBG.Var.BG.AB.timeinc_sec + 30;
			elseif (TBG.Var.Player.zone ~= TBG.Text.Ref.BG_EOTS) then
				TBG.Var.BG.AB.timeinc_sec = TBG.Var.BG.AB.timeinc_sec + 2;
			end

			local winner, win_time = TBG_Battleground_Resources_GetWinner(TBG.DB.world_state[1].bases, TBG.DB.world_state[2].bases);

			if (winner ~= nil) then
				local complete = false;

				-- If the player's faction isn't going to win.
				if (winner ~= UnitFactionGroup("player")) then
					local bases    = TBG_Battleground_Resources_NodesRequiredToWin(TBG.Var.Player.faction);
					      complete = TBG_Battleground_Resources_TimeLeftToCapture(TBG.Var.Player.faction, bases, TBG.Var.BG.AB.timeinc_sec);

				-- Otherwise, if their faction is going to win.
				else
					local bases    = TBG_Battleground_Resources_NodesRequiredToWin(TBG.Var.Player.enemy_faction);
					      complete = TBG_Battleground_Resources_TimeLeftToCapture(TBG.Var.Player.enemy_faction, bases, TBG.Var.BG.AB.timeinc_sec);
				end

				if (complete) then
					TBG.Var.BG.AB.timeinc_check         = false;
					TBG.Var.BG.AB.time_to_node_increase = TBG.Var.BG.AB.timeinc_sec;
				end

				if (win_time < TBG.Var.BG.AB.timeinc_sec) then
					TBG.Var.BG.AB.timeinc_check         = false;
					TBG.Var.BG.AB.time_to_node_increase = win_time;
				end
			end
		end
	end


-- ##############
-- ## PRINTING ##
-- ##############


	-- ## Prints who is carrying the specified flag to the chat log.
	function TBG_Battleground_PrintFlagCarrier(faction, name, class)
		if (not TBG_IsBattleground(TBG.Var.Player.zone))                                                then return nil; end
		if (TBG.Var.Player.zone ~= TBG.Text.Ref.BG_WSG and TBG.Var.Player.zone ~= TBG.Text.Ref.BG_EOTS) then return nil; end

		local message = "";

		if (TBG.Var.Player.zone == TBG.Text.Ref.BG_EOTS) then
			message = "* " .. string.gsub(TBG.Text.Message.CHAT_EOTS_FLAG_CARRIED_BY, "{name}", name .. " (" .. faction .. " - " .. class .. ")", 1);
		else
			message = "* " .. string.gsub(string.gsub(TBG.Text.Message.CHAT_WSG_FLAG_CARRIED_BY, "{faction}", faction, 1), "{name}", name .. " (" .. class .. ")", 1);
		end

		SendChatMessage(message, "BATTLEGROUND");
		--SendChatMessage(message, "WHISPER", "Common", "Nirme");
	end


	-- ## Prints the movement timers for Lokholar and Ivus in Alterav Valley.
	function TBG_Battleground_PrintMovementTimer(name, eta)
		if (not TBG_IsBattleground(TBG.Var.Player.zone)) then return nil; end
		if (TBG.Var.Player.zone ~= TBG.Text.Ref.BG_AV)   then return nil; end

		local message = "* " .. string.gsub(string.gsub(TBG.Text.Message.CHAT_AV_MOVEMENT_TIMER, "{name}", name, 1), "{time}", eta, 1);

		SendChatMessage(message, "BATTLEGROUND");
		--SendChatMessage(message, "WHISPER", "Common", "Maina");
	end


	-- ## Prints the specified capture timers to the chat log.
	function TBG_Battleground_PrintCaptureTimer(faction, point, eta)
		local message = "* (" .. faction .. ") " .. point .. " - " .. eta;

		SendChatMessage(message, "BATTLEGROUND");
		--SendChatMessage(message, "WHISPER", "Common", "Maina");
	end


	-- ## Prints all of the capture timers.
	-- ##
	-- ## Variables
	-- ##     total: The maximum amount of captures to be printed.
	function TBG_Battleground_PrintCaptureTimers(total)
		local count = 0;

		for i = 1, table.getn(TBG.DB.active.captures) do
			local eta = "";

			-- If the capture was done while the player was not there, show unknown.
			if (not TBG_Saved("MD_AllowInaccurateTimers") and TBG.DB.active.captures[i].enter) then
				eta = TBG.Text.Label.Tooltip.CAPTURE_TIMERS_UNKNOWN;

			-- Otherwise, show the timer.
			else
				eta = TBG_SecondsToTime(TBG.DB.active.captures[i].cap - GetTime());

				-- If there is a capture imminent, indicate.
				if (eta == "") then
					eta = TBG.Text.Label.Tooltip.CAPTURE_TIMERS_IMINENT;
				end
			end

			-- If not unknown, display timers.
			if (eta ~= TBG.Text.Label.Tooltip.CAPTURE_TIMERS_UNKNOWN) then
				TBG_Battleground_PrintCaptureTimer(TBG.DB.active.captures[i].who, TBG.DB.active.captures[i].name, eta);

				count = count + 1;

				if (total and tonumber(count) >= tonumber(total)) then
					break;
				end
			end
		end
	end


-- ##################
-- ## OTHER PANELS ##
-- ##################

	-- ## Updates the on the screen display of the capture timers.
	function TBG_Battleground_Other_UpdateCaptures()
		local caps     = 0;
		local display  = TBG_Saved("MDB_Other_CaptureTimers_OnScreen_DisplayHowMany");
		local shown    = 1;
		local i        = 1;

		-- Hide all of the frames first.
		if (not TBG_IsBattleground(TBG.Var.Player.zone)) then
			for i = 1, TBG.Var.Display.captures_max do
				getglobal("TBGX_Display_Capture_" .. i):Hide();
			end
		end

		if (not TBG_IsBattleground(TBG.Var.Player.zone))                                 then return nil; end
		if (not TBG.Var.Const.BATTLEGROUNDS_INFORMATION[TBG.Var.Player.zone].has_points) then return nil; end

		if (GetBattlefieldWinner() == nil) then
			if (TBG.DB.active.captures ~= nil) then
				caps = table.getn(TBG.DB.active.captures); -- Number of captures.

				if (table.getn(TBG.DB.active.captures) > TBG_Saved("MDB_Other_CaptureTimers_OnScreen_DisplayHowMany")) then
					display = caps;
				end

				local _, relativeTo, _, _, _ = TBGX_Display_Capture_1:GetPoint();
				local nRelativeTo            = nil;

				if (TBG_Display_MovementTimer_2:IsVisible()) then
					nRelativeTo = TBG_Display_MovementTimer_2;
				end

				if (TBG_Display_MovementTimer_1:IsVisible() and nRelativeTo == nil) then
					nRelativeTo = TBG_Display_MovementTimer_1;
				end

				-- If movement timers are displayed, change attachment of capture timers.
				if (nRelativeTo ~= nil and relativeTo ~= nRelativeTo) then
					TBGX_Display_Capture_1:ClearAllPoints();

					if (TBG_Saved("MDB_Other_CaptureTimers_OnScreen_Invert")) then
						TBGX_Display_Capture_1:SetPoint("BOTTOMLEFT", nRelativeTo, "TOPLEFT", 0, 10);
					else
						TBGX_Display_Capture_1:SetPoint("TOPLEFT", nRelativeTo, "BOTTOMLEFT", 0, -10);
					end

				-- If movement timers are not displayed, change attachment of capture timers.
				elseif (nRelativeTo == nil and relativeTo ~= TBGX_Display_Capture) then
					TBGX_Display_Capture_1:ClearAllPoints();

					if (TBG_Saved("MDB_Other_CaptureTimers_OnScreen_Invert")) then
						TBGX_Display_Capture_1:SetPoint("BOTTOMLEFT", TBGX_Display_Capture, "BOTTOMLEFT", 0, 0);
					else
						TBGX_Display_Capture_1:SetPoint("TOPLEFT", TBGX_Display_Capture, "TOPLEFT", 0, 0);
					end
				end
			end

			while (i <= display) do
				local frame = "TBGX_Display_Capture_" .. shown;

				-- If this slot is not active, turn it off.
				if (TBG.DB.active.captures[i] == nil or not TBG_Saved("MDB_Other_CaptureTimers_OnScreen_Display")) then

					-- Blank out the points.
					getglobal(frame .. "_Icon"):SetTexCoord(TBG.Var.Const.TEXTURE_CAPTURES_BLANK[1], TBG.Var.Const.TEXTURE_CAPTURES_BLANK[2], TBG.Var.Const.TEXTURE_CAPTURES_BLANK[3], TBG.Var.Const.TEXTURE_CAPTURES_BLANK[4]);
					getglobal(frame .. "_ButtonText"):SetText("");
					getglobal(frame .. "_Time"):SetText("");

					getglobal(frame .. "_Button").faction = "";
					getglobal(frame .. "_Button").name    = "";
					getglobal(frame .. "_Button").eta     = "";

					-- Hide the frame.
					getglobal(frame):Hide();

					shown = shown + 1;

				-- Update this slot with a timer if this is a valid capture.
				else
					local eta = "";

					-- If the capture was done while the player was not there, show unknown.
					if (not TBG_Saved("MD_AllowInaccurateTimers") and TBG.DB.active.captures[i].enter) then
						eta = TBG.Text.Label.Tooltip.CAPTURE_TIMERS_UNKNOWN;

					-- Otherwise, show the timer.
					else
						eta = TBG_SecondsToTime(TBG.DB.active.captures[i].cap - GetTime());

						-- If there is a capture imminent, indicate.
						if (eta == "") then
							eta = TBG.Text.Label.Tooltip.CAPTURE_TIMERS_IMINENT;
						end
					end

					-- If not unknown, display timers.
					if (eta ~= TBG.Text.Label.Tooltip.CAPTURE_TIMERS_UNKNOWN) then
						local tOne, tTwo, tThree, tFour = WorldMap_GetPOITextureCoords(TBG.DB.active.captures[i].t);

						-- Update the points.
						getglobal(frame .. "_Icon"):SetTexCoord(tOne, tTwo, tThree, tFour);
						getglobal(frame .. "_ButtonText"):SetText(TBG.DB.active.captures[i].name);
						getglobal(frame .. "_Time"):SetText(eta);

						getglobal(frame .. "_Button").faction = TBG.DB.active.captures[i].text;
						getglobal(frame .. "_Button").name    = TBG.DB.active.captures[i].name;
						getglobal(frame .. "_Button").eta     = eta;

						-- Show the frame.
						getglobal(frame):Show();

						shown = shown + 1;
					end
				end

				if (shown == (TBG_Saved("MDB_Other_CaptureTimers_OnScreen_DisplayHowMany") + 1)) then
					break;
				end

				i = i + 1;
			end

			-- Hide those items not being used.
			for i = shown, TBG.Var.Display.captures_max do
				getglobal("TBGX_Display_Capture_" .. i):Hide();
			end
		end
	end


	-- ## Updates the on the screen display of the Warsong Gulch flag tracker.
	function TBG_Battleground_Other_UpdateFlags()
		if (not TBG_IsBattleground(TBG.Var.Player.zone))                               then return nil; end
		if (not TBG.Var.Const.BATTLEGROUNDS_INFORMATION[TBG.Var.Player.zone].has_flag) then return nil; end

		local faction = { };

		faction[1] = { name = TBG.DB.active.stats.bg.stats.alliance.player, faction = FACTION_ALLIANCE, };
		faction[2] = { name = TBG.DB.active.stats.bg.stats.horde.player,    faction = FACTION_HORDE,    };

		-- Return the actual frame number whether inverted or not.
		local invert = { [1] = 1, [2] = 2, };

		if (TBG_Saved("MDB_Other_FlagTracker_OnScreen_Invert")) then
			invert[1] = 2;
			invert[2] = 1;
		end

		-- If this is the Eye of the Storm, then there is only one flag. No need for two slots.
--		if (TBG.Var.Player.zone == TBG.Text.Ref.BG_EOTS and (faction[1].name == nil or faction[1].name == "")) then
--			if (faction[2].name ~= nil and faction[2].name ~= "") then
--				local old        = faction[1];
--				      faction[1] = faction[2];
--				      faction[2] = old;
--			end
--		end

		for f = 1, table.getn(faction) do
			local frame = "TBG_Display_Flag";

			if (faction[f].name ~= "" and faction[f].name ~= nil) then
				local color   = "";
				local class   = "";
				local player  = TBG.DB.players.names[faction[f].name .. "|" .. TBG.Var.Const.FACTION_NAME.from[faction[f].faction]];

				if (player) then

					-- If the class name should be colored.
					if (TBG_Saved("MDB_Other_FlagTracker_OnScreen_ColorClass")) then
						color = TBG.Var.Const.COLOR_CLASS[TBG.Var.Const.NAME_CLASS[player.class]].color;
					end

					-- If the class text should be displayed.
					if (TBG_Saved("MDB_Other_FlagTracker_OnScreen_TextClass")) then
						class = TBG.Var.Const.COLOR_WHITE .. " (" .. player.class .. ")";
					end

					-- The texture of the flag icon.
					local texture = "Interface\\WorldStateFrame\\" .. TBG.Var.Const.FACTION_NAME.from[TBG.Var.Const.FACTION_NAME.enemy[faction[f].faction]]  .. "Flag";

					if (TBG.Var.Player.zone == TBG.Text.Ref.BG_EOTS) then
						texture = "Interface\\WorldStateFrame\\" .. TBG.Var.Const.FACTION_NAME.from[faction[f].faction] .. "Flag";
					end

					-- Set icon if not attached to the score frame.
					if (TBG_Saved("MDB_Other_FlagTracker_OnScreen_AttachToScore")) then
						getglobal(frame .. "_" .. invert[f] .. "_Icon"):Hide();
					else
						getglobal(frame .. "_" .. invert[f] .. "_Icon"):SetTexture(texture);
						getglobal(frame .. "_" .. invert[f] .. "_Icon"):Show();
					end

					if (TBG_Saved("MDB_Other_FlagTracker_OnScreen_AttachToScore") and TBG.Var.Player.zone == TBG.Text.Ref.BG_EOTS) then
						getglobal(frame .. "_" .. invert[f] .. "_IconScoreFrame"):SetTexture(texture);
						getglobal(frame .. "_" .. invert[f] .. "_IconScoreFrame"):Show();
					else
						getglobal(frame .. "_" .. invert[f] .. "_IconScoreFrame"):Hide();
					end

					-- Set text.
					getglobal(frame .. "_" .. invert[f] .. "_ButtonText"):SetText(color .. player.name .. class);
					getglobal(frame .. "_" .. invert[f] .. "_Button"):SetWidth(getglobal(frame .. "_" .. invert[f] .. "_ButtonText"):GetWidth());

					-- Set on click script.
					getglobal(frame .. "_" .. invert[f] .. "_Button"):SetAttribute("type1", "macro");
					getglobal(frame .. "_" .. invert[f] .. "_Button"):SetAttribute("macrotext1", "/target " .. faction[f].name);

					-- Set correct flag faction.
					local flag = player.faction;
					if (TBG.Var.Player.zone == TBG.Text.Ref.BG_WSG) then flag = TBG.Var.Const.FACTION_NAME.enemy[flag]; end

					getglobal(frame .. "_" .. invert[f] .. "_Button"):SetAttribute("shift-type1", "macro");
					getglobal(frame .. "_" .. invert[f] .. "_Button"):SetAttribute("shift-macrotext1", "/script TBG_Battleground_PrintFlagCarrier('" .. flag .. "', '" .. player.name .. "', '" .. player.class .. "');");

					getglobal(frame .. "_" .. invert[f] .. "_ButtonText"):Show();
					getglobal(frame .. "_" .. invert[f]):Show();
				end
			else
				getglobal(frame .. "_" .. invert[f] .. "_ButtonText"):SetText("");
				getglobal(frame .. "_" .. invert[f]):Hide();
			end
		end
	end


	-- ## Updates movement timers of Lokholar and Ivus in Alterac Valley.
	function TBG_Battleground_Other_UpdateMovementTimers()
		local i = 0;

		if (TBG_Display_MovementTimer_2:IsVisible() or TBG_Display_MovementTimer_2_ButtonText:GetText() ~= "") then
			TBG_Display_MovementTimer_2:Hide();
			TBG_Display_MovementTimer_2_ButtonText:SetText("");
		end

		if (TBG_Display_MovementTimer_1:IsVisible() or TBG_Display_MovementTimer_1_ButtonText:GetText() ~= "") then
			TBG_Display_MovementTimer_1:Hide();
			TBG_Display_MovementTimer_1_ButtonText:SetText("");
		end

		if (not TBG_IsBattleground(TBG.Var.Player.zone)) then return nil; end
		if (TBG.Var.Player.zone ~= TBG.Text.Ref.BG_AV)   then return nil; end

		if (TBG_Saved("MDB_Other_MovementTimers_OnScreen_Display") and (TBG.DB.active.stats.bg.stats.summons["IVUS"] ~= 0 or TBG.DB.active.stats.bg.stats.summons["LOK"] ~= 0)) then
			if (TBG.DB.active.stats.bg.stats.summons["IVUS"] ~= 0) then
				i = i + 1;

				local mtime = TBG.Text.Label.Button.READY;
				local color = TBG.Var.Const.COLOR_BLUE;

				if (TBG.Var.BG.AV.time_to_movement - (GetTime() - TBG.DB.active.stats.bg.stats.summons["IVUS"]) > 0) then
					mtime = TBG_SecondsToTime(TBG.Var.BG.AV.time_to_movement - (GetTime() - TBG.DB.active.stats.bg.stats.summons["IVUS"]));
				end

				getglobal("TBG_Display_MovementTimer_" .. i .. "_ButtonText"):SetText(color .. TBG.Text.Label.Other.IVUS);
				getglobal("TBG_Display_MovementTimer_" .. i .. "_Time"):SetText(mtime);

				if (TBG_Saved("MDB_Other_MovementTimers_OnScreen_ClickPrint")) then
					getglobal("TBG_Display_MovementTimer_" .. i .. "_Button"):SetScript("OnClick", function()
						if (IsShiftKeyDown()) then
							TBG_Battleground_PrintMovementTimer(TBG.Text.Label.Other.IVUS, mtime);
						end
					end);
				end

				getglobal("TBG_Display_MovementTimer_" .. i):Show();
			end

			if (TBG.DB.active.stats.bg.stats.summons["LOK"] ~= 0) then
				i = i + 1;

				local mtime = TBG.Text.Label.Button.READY;
				local color = TBG.Var.Const.COLOR_RED;

				if (TBG.Var.BG.AV.time_to_movement - (GetTime() - TBG.DB.active.stats.bg.stats.summons["LOK"]) > 0) then
					mtime = TBG_SecondsToTime(TBG.Var.BG.AV.time_to_movement - (GetTime() - TBG.DB.active.stats.bg.stats.summons["LOK"]));
				end

				getglobal("TBG_Display_MovementTimer_" .. i .. "_ButtonText"):SetText(color .. TBG.Text.Label.Other.LOKHOLAR);
				getglobal("TBG_Display_MovementTimer_" .. i .. "_Time"):SetText(mtime);

				if (TBG_Saved("MDB_Other_MovementTimers_OnScreen_ClickPrint")) then
					getglobal("TBG_Display_MovementTimer_" .. i .. "_Button"):SetScript("OnClick", function()
						if (IsShiftKeyDown()) then
							TBG_Battleground_PrintMovementTimer(TBG.Text.Label.Other.LOKHOLAR, mtime);
						end
					end);
				end

				getglobal("TBG_Display_MovementTimer_" .. i):Show();
			end
		end
	end


-- #############
-- ## HISTORY ##
-- #############


	-- ## Checks the history making certain it is not corrupted.
	function TBG_Battleground_HistoryValidate()

		-- Loop through all of the battlegrounds and if they store history validate it is correct.
		for i = 1, table.getn(TBG.Var.Const.BATTLEGROUNDS) do
			local bg = TBG.Var.Const.BATTLEGROUNDS[i];

			if (TBG.Var.Const.BATTLEGROUNDS_INFORMATION[bg].store_history) then
				if (not TBG_Saved("BattlegroundHistory")[bg]) then
					TBG_Init_CreateBattlegroundHistory();
					TBG_Print(TBG.Text.Message.HISTORY_CORRUPT_AND_RESET);
				end
			end
		end
	end


	-- ## Stores battleground history.
	function TBG_Battleground_HistoryUpdate()
		if (not TBG_IsBattleground(TBG.Var.Player.zone)) then return nil; end

		if (not TBG.Var.BG.stat_information_stored and GetBattlefieldWinner() ~= nil) then
			local bg = TBG.Var.Player.zone;

			TBG_Battleground_HistoryValidate(); -- Validate the history to be certain everything is correct.

			if (TBG.Var.Const.BATTLEGROUNDS_INFORMATION[bg].store_history) then
				local winner  = GetBattlefieldWinner();
				local length  = TBG.DB.active.stats.bg.run;
				local percent = { [1] = 0, [0] = 0, };

				if (TBG.DB.active.stats.bg.servers.alliance and TBG.DB.active.stats.bg.alliance) then
					if (TBG.DB.active.stats.bg.servers.alliance[1]) then
						percent[1] = TBG_MathRound((TBG.DB.active.stats.bg.servers.alliance[1].count / TBG.DB.active.stats.bg.alliance) * 100, 0);
					end
				end

				if (TBG.DB.active.stats.bg.servers.horde and TBG.DB.active.stats.bg.horde) then
					if (TBG.DB.active.stats.bg.servers.horde[1]) then
						percent[0] = TBG_MathRound((TBG.DB.active.stats.bg.servers.horde[1].count / TBG.DB.active.stats.bg.horde) * 100, 0);
					end
				end

				-- Alliance.
				if (percent[1] >= 80) then
					TBG_Saved("BattlegroundHistory").total.team[1]     = TBG_Saved("BattlegroundHistory").total.team[1] + 1;
					TBG_Saved("BattlegroundHistory")[bg].total.team[1] = TBG_Saved("BattlegroundHistory")[bg].total.team[1] + 1;
				else
					TBG_Saved("BattlegroundHistory").total.pug[1]     = TBG_Saved("BattlegroundHistory").total.pug[1] + 1;
					TBG_Saved("BattlegroundHistory")[bg].total.pug[1] = TBG_Saved("BattlegroundHistory")[bg].total.pug[1] + 1;
				end

				-- Horde.
				if (percent[0] >= 80) then
					TBG_Saved("BattlegroundHistory").total.team[0]     = TBG_Saved("BattlegroundHistory").total.team[0] + 1;
					TBG_Saved("BattlegroundHistory")[bg].total.team[0] = TBG_Saved("BattlegroundHistory")[bg].total.team[0] + 1;
				else
					TBG_Saved("BattlegroundHistory").total.pug[0]     = TBG_Saved("BattlegroundHistory").total.pug[0] + 1;
					TBG_Saved("BattlegroundHistory")[bg].total.pug[0] = TBG_Saved("BattlegroundHistory")[bg].total.pug[0] + 1;
				end

				-- Winner.
				if (percent[winner] >= 80) then
					TBG_Saved("BattlegroundHistory").won.team[winner]     = TBG_Saved("BattlegroundHistory").won.team[winner] + 1;
					TBG_Saved("BattlegroundHistory")[bg].won.team[winner] = TBG_Saved("BattlegroundHistory")[bg].won.team[winner] + 1;

					if ((TBG_Saved("BattlegroundHistory")[bg].shortest_game.team.length == 0 or TBG_Saved("BattlegroundHistory")[bg].shortest_game.team.length > length) and not TBG.DB.active.stats.bg.run_estimate) then
						TBG_Saved("BattlegroundHistory")[bg].shortest_game.team.length  = length;
						TBG_Saved("BattlegroundHistory")[bg].shortest_game.team.faction = winner;
					end

					if (TBG.Var.Const.BATTLEGROUNDS_INFORMATION[bg].time_is_limited and not TBG.DB.active.stats.bg.run_estimate and (TBG_Saved("BattlegroundHistory")[bg].longest_game.team.length == 0 or TBG_Saved("BattlegroundHistory")[bg].longest_game.team.length < length)) then
						TBG_Saved("BattlegroundHistory")[bg].longest_game.team.length  = length;
						TBG_Saved("BattlegroundHistory")[bg].longest_game.team.faction = winner;
					end
				else
					TBG_Saved("BattlegroundHistory").won.pug[winner]     = TBG_Saved("BattlegroundHistory").won.pug[winner] + 1;
					TBG_Saved("BattlegroundHistory")[bg].won.pug[winner] = TBG_Saved("BattlegroundHistory")[bg].won.pug[winner] + 1;

					if ((TBG_Saved("BattlegroundHistory")[bg].shortest_game.pug.length == 0 or TBG_Saved("BattlegroundHistory")[bg].shortest_game.pug.length > length) and not TBG.DB.active.stats.bg.run_estimate) then
						TBG_Saved("BattlegroundHistory")[bg].shortest_game.pug.length  = length;
						TBG_Saved("BattlegroundHistory")[bg].shortest_game.pug.faction = winner;
					end

					if (TBG.Var.Const.BATTLEGROUNDS_INFORMATION[bg].time_is_limited and not TBG.DB.active.stats.bg.run_estimate and (TBG_Saved("BattlegroundHistory")[bg].longest_game.pug.length == 0 or TBG_Saved("BattlegroundHistory")[bg].longest_game.pug.length < length)) then
						TBG_Saved("BattlegroundHistory")[bg].longest_game.pug.length  = length;
						TBG_Saved("BattlegroundHistory")[bg].longest_game.pug.faction = winner;
					end
				end

				if (TBG.Var.Const.BATTLEGROUNDS_INFORMATION[bg].has_score) then
					if (bg == TBG.Text.Ref.BG_AB or bg == TBG.Text.Ref.BG_EOTS) then
						TBG_Saved("BattlegroundHistory")[bg].total_score[1][1] = TBG_Saved("BattlegroundHistory")[bg].total_score[1][1] + TBG.DB.world_state[1].resources;
						TBG_Saved("BattlegroundHistory")[bg].total_score[0][1] = TBG_Saved("BattlegroundHistory")[bg].total_score[0][1] + TBG.DB.world_state[2].resources;

						TBG_Saved("BattlegroundHistory")[bg].total_score[1][2] = TBG_Saved("BattlegroundHistory")[bg].total_score[1][2] + TBG.DB.world_state[1].bases;
						TBG_Saved("BattlegroundHistory")[bg].total_score[0][2] = TBG_Saved("BattlegroundHistory")[bg].total_score[0][2] + TBG.DB.world_state[2].bases;
					elseif (bg == TBG.Text.Ref.BG_WSG) then
						TBG_Saved("BattlegroundHistory")[bg].total_score[1][1] = TBG_Saved("BattlegroundHistory")[bg].total_score[1][1] + TBG.DB.world_state[1].flags;
						TBG_Saved("BattlegroundHistory")[bg].total_score[0][1] = TBG_Saved("BattlegroundHistory")[bg].total_score[0][1] + TBG.DB.world_state[2].flags;
					else
					end
				end
			end

			TBG.Var.BG.stat_information_stored = true;
		end

	end


	-- ## Scans the player's inventory for battleground items.
	function TBG_Battleground_ItemsScan()
		local items     = { index = { }, text = { } };
		local inventory = { };

		local bag = { [0] = GetContainerNumSlots(0),
			          [1] = GetContainerNumSlots(1),
			          [2] = GetContainerNumSlots(2),
			          [3] = GetContainerNumSlots(3),
			          [4] = GetContainerNumSlots(4), };

		-- Create a fake tooltip.
		if (not TBGX_TempTooltip) then CreateFrame("GameTooltip", "TBGX_TempTooltip", nil, "GameTooltipTemplate"); end
		TBGX_TempTooltip:SetOwner(UIParent, "ANCHOR_NONE");

		-- Loop through all of the player's buffs and check for a mount.
		for b = 0, 4 do
			for i = 1, bag[b] do
				local link = GetContainerItemLink(b, i);

				if (link) then
					TBGX_TempTooltip:SetBagItem(b, i);

					local name  = TBGX_TempTooltipTextLeft1:GetText();
					local bg    = TBGX_TempTooltipTextLeft2:GetText();
					local count = GetItemCount(link);

					if (name) then
						if (bg and bg == TBG.Text.Ref.BG_AV and inventory[name] == nil) then
							local index = { };

							index.name  = name;
							index.count = count;

							table.insert(items.index, index);
							items.text = table.getn(items.index);
						end

						inventory[name] = count;
					end
				end
			end
		end

		TBGX_TempTooltip:Hide();

		items.index                   = TBG_Battleground_ItemsSort(items.index);
		TBG.DB.inventory.battleground = items;
		TBG.DB.inventory.all          = inventory;
	end

	-- ## Sorts the list of items alphabetically.
	function TBG_Battleground_ItemsSort(list)
		if (not TBG_IsBattleground(TBG.Var.Player.zone)) then return list; end

		table.sort (list, function (v1, v2)
			if (TBG_Saved("MDB_TT_BattlegroundItemsSort") == "NAME") then
				return v1.name < v2.name;
			end

			if (TBG_Saved("MDB_TT_BattlegroundItemsSort") == "AMOUNT") then
				return v1.count > v2.count;
			end
		end);

		return list;
	end


	-- ## Returns the date the battleground weekend started for the current week.
	-- ## Date begins the day following maintenance.
	function TBG_GetBattlegroundWeekendStartDate()
		local cdate   = { d = date("%d") + 0, m = date("%m") + 0, y = date("%Y") + 0, };
		local weekday = date("%w") + 0;

		if (weekday == 0) then weekday = 7; end

		if (weekday ~= TBG_Saved("MDB_TT_WeekStart")) then
			local subtract = 0;

			if (weekday < TBG_Saved("MDB_TT_WeekStart")) then
				subtract = weekday;
				weekday  = 7;
			end

			subtract = subtract + (weekday - 1) - (TBG_Saved("MDB_TT_WeekStart") - 1);
			cdate    = TBG_DateSubtractDays(cdate, subtract);
		end

		return cdate;
	end


	-- ## Calculates what battleground weekend it is.
	function TBG_GetBattlegroundWeekend()
		local days      = TBG_GetDateDifference(TBG.Var.Const.BATTLEGROUND_WEEKEND_START_DATE, TBG_GetBattlegroundWeekendStartDate());
		local weeks     = days / 7;
		local remainder = TBG_MathRound(weeks % 4, 0);

		-- Make certain any rounding does not cause problems.
		if (remainder > 7) then remainder = 7; end
		if (remainder < 1) then remainder = 1; end

		TBG.Var.BG.battleground_weekend = TBG.Var.Const.BATTLEGROUND_WEEKEND[remainder];
	end


	-- ## Attempts to guess what the day following maintenance is.
	function TBG_GuessBattlegroundWeekend()
		local weekend = TBG_Saved("MDB_TT_WeekStart");
		local start   = 0;

		-- Loop through all of the days.
		for i = 1, 7 do
			TBG_Saved_Set("MDB_TT_WeekStart", i);

			local days  = TBG_GetDateDifference(TBG.Var.Const.BATTLEGROUND_WEEKEND_START_DATE, TBG_GetBattlegroundWeekendStartDate());
			local weeks = days / 7;

			if (TBG.Var.Const.BATTLEGROUND_WEEKEND[weeks % 4] ~= nil) then
				start = i;
				break;
			end
		end

		-- If the addon could not guess the correct day, just default and allow an inaccurate estimate.
		if (start == 0) then start = 2; end
		return start;
	end