TBG_Files_Init_IsLoaded = true; -- Indicates this file has been loaded into memory.

-- ## Performed when the addon is loaded.
function TBG_Init()
	this.registry = {
		id                  = TBG.Var.Const.ADDON_ID,
		menuText            = TBG.Var.Const.ADDON_ID,
		--tooltipTitle        = TBG.Text.Label.Tooltip.HEADER,
		buttonTextFunction  = "TBG_Button_SetText",
		--tooltipTextFunction = "TBG_Display_SetText",
		icon                = TBG.Var.Const.ADDON_ARTWORK_PATH,
		iconWidth           = 16,
		frequency           = TBG.Var.Const.ADDON_UPDATE_FREQ;

		savedVariables = {
			ShowIcon = 1,
		}
	};

	this:RegisterEvent("ADDON_LOADED");
	this:RegisterEvent("VARIABLES_LOADED");
	this:RegisterEvent("PLAYER_ENTERING_WORLD");
	this:RegisterEvent("FRIENDLIST_UPDATE");
	this:RegisterEvent("PLAYER_ENTERING_WORLD");
	this:RegisterEvent("ZONE_CHANGED");
	this:RegisterEvent("ZONE_CHANGED_NEW_AREA");
	this:RegisterEvent("ZONE_CHANGED_INDOORS");
	this:RegisterEvent("UPDATE_BATTLEFIELD_STATUS");
	this:RegisterEvent("UPDATE_WORLD_STATES");
	this:RegisterEvent("BATTLEFIELDS_SHOW");
	this:RegisterEvent("UPDATE_CHAT_WINDOWS");
	this:RegisterEvent("CHAT_MSG_BG_SYSTEM_ALLIANCE");
	this:RegisterEvent("CHAT_MSG_BG_SYSTEM_HORDE");
	this:RegisterEvent("CHAT_MSG_BG_SYSTEM_NEUTRAL");
	this:RegisterEvent("CHAT_MSG_MONSTER_YELL");
	this:RegisterEvent("CHAT_MSG_BATTLEGROUND");
	this:RegisterEvent("CHAT_MSG_BATTLEGROUND_LEADER");
	this:RegisterEvent("CHAT_MSG_PARTY");
	this:RegisterEvent("CHAT_MSG_RAID");
	this:RegisterEvent("CHAT_MSG_RAID_LEADER");
	this:RegisterEvent("CHAT_MSG_RAID_WARNING");
	this:RegisterEvent("CHAT_MSG_WHISPER");
	this:RegisterEvent("RAID_ROSTER_UPDATE");
	this:RegisterEvent("BAG_UPDATE");
	this:RegisterEvent("GOSSIP_SHOW");
	this:RegisterEvent("GOSSIP_CLOSED");

	-- Register slash commands.
	SLASH_TITANBG1 = "/titanbg";
	SLASH_TITANBG2 = "/tbg";

	--[[
	    **
	    ** SLASH COMMANDS
	    ** Arguments preceded by a * are optional. The argument 'n' represents a number.
	    **
	    ** /tbg * (help|reset|print)
	    ** /tbg reset (pos|history|save)
	    ** /tbg print (flag|cap|move)
	    ** /tbg print flag (a|h)
	    ** /tbg print cap (all|top|n)
	    ** /tbg print cap top (n)
	    ** /tbg print move (ivus|lok)
	    **
	]]

	-- Register slash command handler.
	SlashCmdList["TITANBG"] = function(command)
		TBG_SlashProcess(command);
	end

	-- Shows the status of each queue point returned by the server. If the slot is not taken up by a battleground,
	-- it will return the zone the user is currently in.
	-- !! DEBUG START

	--	DEFAULT_CHAT_FRAME:AddMessage("Printing queue status ... ");

	--	for i = 1, MAX_BATTLEFIELD_QUEUES do
	--		local _, name, _ = GetBattlefieldStatus(i);

	--		DEFAULT_CHAT_FRAME:AddMessage("BG = " .. name);
	--	end

	-- !! DEBUG END
end


-- ## Run when the addon is completely loaded in the game.
function TBG_Init_OnLoad()
	if (TBG.Var.Addon.ADDON_LOADED and TBG.Var.Addon.VARIABLES_LOADED) then
		TBG_Init_DependenciesDetect();

		-- Load player data.
		TBG_Init_CreatePlayerInformation();

		-- Load saved variables.
		TBG_Init_CreateSavedVariables();

		TBG_Init_CreateActiveBattlegroundInformation();
		TBG_Init_CreatePlayerList();
		TBG_Init_CreateInventory();
		TBG_Init_CreateBattlegroundInformation();

		-- Initialise display functions.
		TBG_ToggleInvertCaptureTimers();

		-- Initialise the drop down menu.
		UIDropDownMenu_Initialize(TBGX_RightClickMenu, TBG_Menu_RightClickMenu_Load, "MENU");

		TBG_ToggleScoreFrameVisible();
		TBG_ToggleMaxCameraDistanceIncrease();

		-- If the battlefield minimap hasn't been loaded, load it.
		TBG_Hooks_StoreBlizzard();

		if (TBG_Saved("MG_ShowStartMessage")) then
			local message = string.gsub(TBG.Text.Message.START, "{addon}", TBG.Var.Const.COLOR_BLUE .. TBG.Text.ADDON_NAME, 1);
			      message = string.gsub(message, "{version}", TBG.Var.Const.COLOR_WHITE .. "v" .. TBG.Text.ADDON_VERSION .. TBG.Var.Const.COLOR_YELLOW, 1);
			      message = string.gsub(message, "/tbg", TBG.Var.Const.COLOR_RED .. "/tbg" .. TBG.Var.Const.COLOR_YELLOW, 1);
			      message = string.gsub(message, "/tbg help", TBG.Var.Const.COLOR_RED .. "/tbg help" .. TBG.Var.Const.COLOR_YELLOW, 1);

			DEFAULT_CHAT_FRAME:AddMessage(message);
		end

		if (TBG.Var.Addon.first_load) then
			StaticPopup_Show(TBG.Var.Const.DIALOG_INTRODUCTION);
		end

		if (TBG_Saved("AddonVersion") == nil or
		    TBG_Saved("AddonVersion") ~= TBG.Text.ADDON_VERSION) then

			-- Reset history if this version is earlier than the version that last changed history.
			if (not TBG.Var.Addon.first_load and (not TBG_Saved("AddonVersion") or TBG_Saved("AddonVersion") < "1.3.1" or string.find(TBG_Saved("AddonVersion"), "ALPHA") or string.find(TBG_Saved("AddonVersion"), "BETA"))) then
				TBG_Init_CreateBattlegroundHistory();
				StaticPopup_Show(TBG.Var.Const.DIALOG_RELOAD_HISTORY);
			end

			TBG_Saved_Set("AddonVersion", TBG.Text.ADDON_VERSION);
		end

		if (GetLocale() == "frFR" or GetLocale() == "deDE") then
			local text     = "|cff0097d0Titan Battleground\n\n" .. HIGHLIGHT_FONT_COLOR_CODE;
			local file     = "";
			local language = "";

			if (GetLocale() == "frFR") then
				file     = "Localization.fr.lua";
				language = "French";
			end

			if (GetLocale() == "deDE") then
				file     = "Localization.de.lua";
				language = "German";
			end

			text = text .. "You are using the " .. TBG.Var.Const.COLOR_YELLOW .. language .. TBG.Var.Const.COLOR_WHITE .. " localization of the addon. For this version to work, you will need to perform the following steps.\n\n";
			text = text .. "Access the file " .. TBG.Var.Const.COLOR_GREEN .. file .. TBG.Var.Const.COLOR_WHITE .. " in the addon directory. Scroll down to the following heading.\n\n";
			text = text .. TBG.Var.Const.COLOR_YELLOW .. "-- ## REFERENCES ##" .. TBG.Var.Const.COLOR_WHITE .. "\n\n";
			text = text .. "Many new variables under this heading require translation, particularly those relating to ".. TBG.Var.Const.COLOR_GREEN .. "Alterac Valley" .. TBG.Var.Const.COLOR_WHITE .. ". Please follow the instructions and make certain capitalization, spelling and punctuation is exactly as it appears in the game.\n\n";
			text = text .. "Once you have performed the translation, access the following website and post the translation in the comments. This will mean the translations are included in the following version and you will no longer get this message.\n\n";
			text = text .. TBG.Var.Const.COLOR_GREEN .. TBG.Text.ADDON_WEBSITE .. TBG.Var.Const.COLOR_WHITE .. "\n\n";
			text = text .. "If any other functions fail to work, make sure to check the other translations under the " .. TBG.Var.Const.COLOR_GREEN .. "References" .. TBG.Var.Const.COLOR_WHITE .. " heading in the file.";

			TBG_MessagePopup(text);
		end

		-- Load the menu and other xml.
		TBG_Menu_OnLoad();
		TBGX_QuestFrame_AutoQuestComplete_Add:SetText(TBG.Text.Label.Other.AUTO_QUEST_COMPLETE_ADD);
		TBGX_QuestFrame_AutoQuestComplete_Remove:SetText(TBG.Text.Label.Other.AUTO_QUEST_COMPLETE_REMOVE);

		-- Set what battleground weekend it is.
		TBG_GetBattlegroundWeekend();

		-- Improve garbage collection.
		collectgarbage("setpause", 110);

		TBG.Var.Addon.LOADED = true;
	end
end


function TBG_Init_DependenciesDetect()
	for i = 1, GetNumAddOns() do
		local name = GetAddOnInfo(i);

		if (name == "Titan") then
			TBG.Var.Addon.TITAN.detected = true;
			TBG_Hooks_StoreTitan();
			TBG_Hooks_HookTitan();
		end
	end
end


-- ## Performs setup functions when a player changes zones.
function TBG_Init_AddonSetup()

	-- Hide and show items as necessary.
	TBG_ToggleLockCaptures();
	TBG_ToggleScoreFrameLock();
	TBG_ToggleMiniMapIcon();
	TBG_ToggleDetachScoreFrame();
	TBG_ChangeAttachFlagTracker();
	TBG_ToggleClickFlagRunner();
	TBG_ToggleClickCaptureTimer();
	TBG_ToggleClickMovementTimer();
	TBG_InvertFlagTracker();

	TBG_Tooltip_Load();
	TBG_Tooltip_Hide();
	TBG_Tooltip_AlwaysShow();
	TBG_Tooltip_LockTooltip();
	TBG_Tooltip_SetTooltipPosition();
	TBG_Tooltip_FrameStrata();

	if (WorldStateAlwaysUpFrame) then
		WorldStateAlwaysUpFrame:SetFrameStrata("LOW");
	end
end


-- ## Resets panels.
function TBG_Init_ResetPanels()
	TBG.DB.active.captures = { };

	RaidWarningFrame:Clear();
	TBG_Display_MovementTimer_1:Hide();
	TBG_Display_MovementTimer_1:Hide();
	TBG_Display_Flag_1:Hide();
	TBG_Display_Flag_2:Hide();
	TBGX_Display_MinimapFlags_1:Hide();
	TBGX_Display_MinimapFlags_2:Hide();
	TBG_Arena_Other_UpdateHostilePlayers(true);

	for i = 1, 14 do
		getglobal("TBGX_Display_Capture_" .. i):Hide();
	end

	local i = 1;

	while (true) do
		if (getglobal("TBGX_Assist_" .. i)) then
			getglobal("TBGX_Assist_" .. i):Hide();
		else
			break;
		end

		i = i + 1;
	end
end


-- ## GLOBAL CREATION


	-- ## Resets battleground status information to original values.
	function TBG_Init_CreateBattlegroundInformation()
		TBG.DB.bgs          = { };
		TBG.DB.bgs.queue    = 0;   -- The number of battegrounds that are in queue or active status.
		TBG.DB.bgs.blizzard = { }; -- Table which links the blizzard battleground index to the correct information.
		TBG.DB.bgs.names    = { }; -- Battleground information stored by name.

		TBG_Init_CreateBattlegrounds();
		TBG_Init_CreateArenas();
	end


	function TBG_Init_CreateBattlegrounds()
		for i = 1, table.getn(TBG.Var.Const.BATTLEGROUNDS) do
			local n = TBG.Var.Const.BATTLEGROUNDS[i];

			TBG.DB.bgs.names[n]                = { };
			TBG.DB.bgs.names[n].type           = TBG.Var.Const.BATTLEGROUND;
			TBG.DB.bgs.names[n].reference      = i;
			TBG.DB.bgs.names[n].index          = 0;
			TBG.DB.bgs.names[n].short          = TBG.Text.Ref.BGS_SHORT[n];
			TBG.DB.bgs.names[n].name           = n;
			TBG.DB.bgs.names[n].status         = TBG.Var.Const.BG_STATUS_NONE;
			TBG.DB.bgs.names[n].instance       = 0;
			TBG.DB.bgs.names[n].expires        = 0;
			TBG.DB.bgs.names[n].start_delay    = TBG.Var.BG.start_delay;
			TBG.DB.bgs.names[n].open_instances = 0;
			TBG.DB.bgs.names[n].open_age       = 0;
			TBG.DB.bgs.names[n].update         = nil;
			TBG.DB.bgs.names[n].sound          = false;
			TBG.DB.bgs.names[n].sound_timer    = 0;
			TBG.DB.bgs.names[n].sound_queue    = false;
			TBG.DB.bgs.names[n].auto           = 0;
		end
	end


	function TBG_Init_CreateArenas()
		for i = 1, table.getn(TBG.Var.Const.ARENAS) do
			local n = TBG.Var.Const.ARENAS[i];

			TBG.DB.bgs.names[n]        = { };
			TBG.DB.bgs.names[n].type   = TBG.Var.Const.ARENA; --
			TBG.DB.bgs.names[n].name   = n; --
			TBG.DB.bgs.names[n].queue  = false;
			TBG.DB.bgs.names[n].arenas = { };

			for s = 1, table.getn(TBG.Var.Const.ARENA_SIZES) do
				local a = TBG.Var.Const.ARENA_SIZES[s];

				TBG.DB.bgs.names[n].arenas[a]             = { };

				TBG.DB.bgs.names[n].arenas[a].type        = TBG.Var.Const.ARENA;
				TBG.DB.bgs.names[n].arenas[a].name        = n;
				TBG.DB.bgs.names[n].arenas[a].team_size   = a;
				TBG.DB.bgs.names[n].arenas[a].short       = TBG.Text.Ref.BGS_SHORT[n];
				TBG.DB.bgs.names[n].arenas[a].reference   = i;
				TBG.DB.bgs.names[n].arenas[a].start_delay = TBG.Var.BG.start_delay_arena;
				TBG.DB.bgs.names[n].arenas[a].index       = 0;
				TBG.DB.bgs.names[n].arenas[a].status      = TBG.Var.Const.BG_STATUS_NONE;
				TBG.DB.bgs.names[n].arenas[a].instance    = 0;
				TBG.DB.bgs.names[n].arenas[a].expires     = 0;
				TBG.DB.bgs.names[n].arenas[a].update      = nil;
				TBG.DB.bgs.names[n].arenas[a].sound       = false;
				TBG.DB.bgs.names[n].arenas[a].sound_timer = 0;
				TBG.DB.bgs.names[n].arenas[a].sound_queue = false;
				TBG.DB.bgs.names[n].arenas[a].auto        = 0;
			end
		end
	end


	-- ## Resets information on the battleground player list.
	function TBG_Init_CreatePlayerList()
		TBG.DB.players          = { };
		TBG.DB.players.list     = { };
		TBG.DB.players.names    = { };
		TBG.DB.players.assists  = { };
		TBG.DB.players.horde    = 0;
		TBG.DB.players.alliance = 0;

		TBG.DB.players.classes = {
			[1] = {
				["WARRIOR"] = 0,
				["DRUID"]   = 0,
				["MAGE"]    = 0,
				["WARLOCK"] = 0,
				["ROGUE"]   = 0,
				["HUNTER"]  = 0,
				["PRIEST"]  = 0,
				["PALADIN"] = 0,
				["SHAMAN"]  = 0,
			},

			[0] = {
				["WARRIOR"] = 0,
				["DRUID"]   = 0,
				["MAGE"]    = 0,
				["WARLOCK"] = 0,
				["ROGUE"]   = 0,
				["HUNTER"]  = 0,
				["PRIEST"]  = 0,
				["PALADIN"] = 0,
				["SHAMAN"]  = 0,
			},
		};
	end


	-- ## Resets inventory data.
	function TBG_Init_CreateInventory()
		TBG.DB.inventory                    = { };
		TBG.DB.inventory.all                = { };
		TBG.DB.inventory.battleground       = { };
		TBG.DB.inventory.battleground.index = { };
		TBG.DB.inventory.battleground.text  = { };
	end


	-- ## Resets information on the current active battleground.
	function TBG_Init_CreateActiveBattlegroundInformation()
		TBG.DB.active                             = { };
		TBG.DB.active.stats                       = { };
		TBG.DB.active.stats.bg                    = { };
		TBG.DB.active.stats.bg.close_estimate     = nil;
		TBG.DB.active.stats.bg.run                = 0;                                                    -- How long the battlefield has been open.
		TBG.DB.active.stats.bg.run_estimate       = nil;                                                  -- What time the battlefield run estimations were begun.
		TBG.DB.active.stats.bg.servers            = { [FACTION_ALLIANCE] = { index = { }, reference = { }, },
			                                          [FACTION_HORDE]    = { index = { }, reference = { }, }, }; -- List of servers in the battleground.
		TBG.DB.active.captures                    = { };                                                  -- Capture timers.

		-- Store information on the battleground, such as whether or not it handles flags or capture points.
		if (TBG.Var.Const.BATTLEGROUNDS_INFORMATION[TBG.Var.Player.zone]) then
			TBG.DB.active.stats.bg.stats = TBG.Var.Const.BATTLEGROUNDS_INFORMATION[TBG.Var.Player.zone].stats;
		end
	end


	-- ## Resets player information.
	function TBG_Init_CreatePlayerInformation()

		if (TBG.Var.Player.faction == UNKNOWN) then
			while (true) do
				TBG.Var.Player.faction = UnitFactionGroup("player");

				if (TBG.Var.Player.faction ~= UNKNOWN) then
					if (TBG.Var.Player.faction == TBG.Var.Const.ALLIANCE) then
						TBG.Var.Player.enemy_faction = TBG.Var.Const.HORDE;
					else
						TBG.Var.Player.enemy_faction = TBG.Var.Const.ALLIANCE;
					end

					break;
				end
			end
		end

		if (TBG.Var.Player.name == UNKNOWN) then
			while (true) do
				TBG.Var.Player.name = UnitName("player");
				if (TBG.Var.Player.name ~= UNKNOWN) then break; end
			end
		end

		if (TBG.Var.Player.realm == UNKNOWN) then
			while (true) do
				TBG.Var.Player.realm = GetRealmName();
				if (TBG.Var.Player.realm ~= UNKNOWN) then break; end
			end
		end
	end


	-- ## Resets battleground history.
	function TBG_Init_CreateBattlegroundHistory()
		TBG_Saved_Set("BattlegroundHistory", { });

		TBG_Saved("BattlegroundHistory").total         = { };
		TBG_Saved("BattlegroundHistory").total.team    = { };
		TBG_Saved("BattlegroundHistory").total.team[0] = 0;
		TBG_Saved("BattlegroundHistory").total.team[1] = 0;
		TBG_Saved("BattlegroundHistory").total.pug     = { };
		TBG_Saved("BattlegroundHistory").total.pug[0]  = 0;
		TBG_Saved("BattlegroundHistory").total.pug[1]  = 0;

		TBG_Saved("BattlegroundHistory").won         = { };
		TBG_Saved("BattlegroundHistory").won.team    = { };
		TBG_Saved("BattlegroundHistory").won.team[0] = 0;
		TBG_Saved("BattlegroundHistory").won.team[1] = 0;
		TBG_Saved("BattlegroundHistory").won.pug     = { };
		TBG_Saved("BattlegroundHistory").won.pug[0]  = 0;
		TBG_Saved("BattlegroundHistory").won.pug[1]  = 0;


		for i = 1, table.getn(TBG.Var.Const.BATTLEGROUNDS) do
			local n = TBG.Var.Const.BATTLEGROUNDS[i];

			TBG_Saved("BattlegroundHistory")[n]      = { };
			TBG_Saved("BattlegroundHistory")[n].n    = n;
			TBG_Saved("BattlegroundHistory")[n].name = n;

			TBG_Saved("BattlegroundHistory")[n].total         = { };
			TBG_Saved("BattlegroundHistory")[n].total.team    = { };
			TBG_Saved("BattlegroundHistory")[n].total.team[0] = 0;
			TBG_Saved("BattlegroundHistory")[n].total.team[1] = 0;
			TBG_Saved("BattlegroundHistory")[n].total.pug     = { };
			TBG_Saved("BattlegroundHistory")[n].total.pug[0]  = 0;
			TBG_Saved("BattlegroundHistory")[n].total.pug[1]  = 0;

			TBG_Saved("BattlegroundHistory")[n].won         = { };
			TBG_Saved("BattlegroundHistory")[n].won.team    = { };
			TBG_Saved("BattlegroundHistory")[n].won.team[0] = 0;
			TBG_Saved("BattlegroundHistory")[n].won.team[1] = 0;
			TBG_Saved("BattlegroundHistory")[n].won.pug     = { };
			TBG_Saved("BattlegroundHistory")[n].won.pug[0]  = 0;
			TBG_Saved("BattlegroundHistory")[n].won.pug[1]  = 0;

			TBG_Saved("BattlegroundHistory")[n].shortest_game              = { };
			TBG_Saved("BattlegroundHistory")[n].shortest_game.team         = { };
			TBG_Saved("BattlegroundHistory")[n].shortest_game.team.faction = nil;
			TBG_Saved("BattlegroundHistory")[n].shortest_game.team.length  = 0;
			TBG_Saved("BattlegroundHistory")[n].shortest_game.pug          = { };
			TBG_Saved("BattlegroundHistory")[n].shortest_game.pug.faction  = nil;
			TBG_Saved("BattlegroundHistory")[n].shortest_game.pug.length   = 0;

			TBG_Saved("BattlegroundHistory")[n].longest_game              = { };
			TBG_Saved("BattlegroundHistory")[n].longest_game.team         = { };
			TBG_Saved("BattlegroundHistory")[n].longest_game.team.faction = nil;
			TBG_Saved("BattlegroundHistory")[n].longest_game.team.length  = 0;
			TBG_Saved("BattlegroundHistory")[n].longest_game.pug          = { };
			TBG_Saved("BattlegroundHistory")[n].longest_game.pug.faction  = nil;
			TBG_Saved("BattlegroundHistory")[n].longest_game.pug.length   = 0;

			TBG_Saved("BattlegroundHistory")[n].total_score       = { };
			TBG_Saved("BattlegroundHistory")[n].total_score[0]    = { };
			TBG_Saved("BattlegroundHistory")[n].total_score[0][1] = 0;
			TBG_Saved("BattlegroundHistory")[n].total_score[0][2] = 0;
			TBG_Saved("BattlegroundHistory")[n].total_score[1]    = { };
			TBG_Saved("BattlegroundHistory")[n].total_score[1][1] = 0;
			TBG_Saved("BattlegroundHistory")[n].total_score[1][2] = 0;
		end
	end

	-- ## Resets world state always up frame data.
	function TBG_Init_CreateWorldState()
		TBG.DB.world_state[1] = { };
		TBG.DB.world_state[2] = { };
	end

	-- ## Sets up saved variables.
	function TBG_Init_CreateSavedVariables()

		-- Clear old saved variables.
		if (sv_menu) then sv_menu = nil; end

		-- If the addon is being loaded for the first time, make sure TBGSV exists.
		if (TBGSV == nil) then
			TBGSV                                                  = { };
			TBGSV["List"]                                          = { };
			TBGSV["Profiles"]                                      = { };
			TBGSV["Profiles"][TBG.Var.Const.ADDON_DEFAULT_PROFILE] = { };

			TBG.Var.Addon.first_load = true;
		end

		-- Create the player.
		if (TBGSV["List"][TBG.Var.Player.realm]                      == nil) then TBGSV["List"][TBG.Var.Player.realm]                      = { }; end
		if (TBGSV["List"][TBG.Var.Player.realm][TBG.Var.Player.name] == nil) then TBGSV["List"][TBG.Var.Player.realm][TBG.Var.Player.name] = { }; end

		-- Set the profile for any new users.
		if (TBG_Saved_GetProfile() == nil) then TBG_Saved_SetProfile(); end

		-- Import old settings.
		if (TBGS ~= nil) then
			TBGSV["Profiles"][TBG_Saved_GetProfile()] = TBGS;
			TBGS = nil;
		end

		-- Work through an make sure all applicable variables are set for this profile.

		if (TBG_Saved("BattlegroundHistory") == nil) then
			TBG_Init_CreateBattlegroundHistory();
		end

		if (TBG_Saved("AddonEnabled") == nil)     then TBG_Saved_Set("AddonEnabled",     true); end
		if (TBG_Saved("AutoTurnInQuests") == nil) then TBG_Saved_Set("AutoTurnInQuests", { });  end

		if (TBG_Saved("M_ShowButtonText") == nil) then TBG_Saved_Set("M_ShowButtonText", true); end

		-- General
		if (TBG_Saved("MG_ShowStartMessage") == nil)       then TBG_Saved_Set("MG_ShowStartMessage",       true); end
		if (TBG_Saved("MG_ShowTips") == nil)               then TBG_Saved_Set("MG_ShowTips",               true); end
		if (TBG_Saved("MG_OverwriteOtherSettings") == nil) then TBG_Saved_Set("MG_OverwriteOtherSettings", true); end
		if (TBG_Saved("MG_AllowGarbageCollection") == nil) then TBG_Saved_Set("MG_AllowGarbageCollection", true); end

			-- Battleground
			if (TBG_Saved("MGB_AutoQueue") == nil)                then TBG_Saved_Set("MGB_AutoQueue",                false);   end
			if (TBG_Saved("MGB_AutoJoin") == nil)                 then TBG_Saved_Set("MGB_AutoJoin",                 false);   end
			if (TBG_Saved("MGB_AutoJoin_NotIfAFK") == nil)        then TBG_Saved_Set("MGB_AutoJoin_NotIfAFK",        false);   end
			if (TBG_Saved("MGB_AutoJoin_NotInArena") == nil)      then TBG_Saved_Set("MGB_AutoJoin_NotInArena",      true);    end
			if (TBG_Saved("MGB_AutoJoin_Always") == nil)          then TBG_Saved_Set("MGB_AutoJoin_Always",          false);   end
			if (TBG_Saved("MGB_AutoJoin_Priority") == nil)        then TBG_Saved_Set("MGB_AutoJoin_Priority",        false);   end
			if (TBG_Saved("MGB_AutoLeave") == nil)                then TBG_Saved_Set("MGB_AutoLeave",                false);   end
			if (TBG_Saved("MGB_AutoReleaseGhost") == nil)         then TBG_Saved_Set("MGB_AutoReleaseGhost",         false);   end
			if (TBG_Saved("MGB_AutoReleaseCheck") == nil)         then TBG_Saved_Set("MGB_AutoReleaseCheck",         true);    end
			if (TBG_Saved("MGB_AutoDisableKey") == nil)           then TBG_Saved_Set("MGB_AutoDisableKey",           "SHIFT"); end
			if (TBG_Saved("MGB_AutoTurnInQuests") == nil)         then TBG_Saved_Set("MGB_AutoTurnInQuests",         true);    end
			if (TBG_Saved("MGB_AutoTurnInQuests_Warning") == nil) then TBG_Saved_Set("MGB_AutoTurnInQuests_Warning", true);    end
			if (TBG_Saved("MGB_SkipBattlemasterGossip") == nil)   then TBG_Saved_Set("MGB_SkipBattlemasterGossip",   true);    end
			if (TBG_Saved("MGB_SkipBankGossip") == nil)           then TBG_Saved_Set("MGB_SkipBankGossip",           true);    end
			if (TBG_Saved("MGB_SkipTaxiGossip") == nil)           then TBG_Saved_Set("MGB_SkipTaxiGossip",           true);    end
			if (TBG_Saved("MGB_SkipVendorGossip") == nil)         then TBG_Saved_Set("MGB_SkipVendorGossip",         true);    end

			if (TBG_Saved("MGB_AutoQueue_Which") == nil)                                                                   then TBG_Saved_Set("MGB_AutoQueue_Which",                     { });  end
			if (TBG_Saved("MGB_AutoQueue_Which") ~= nil and TBG_Saved("MGB_AutoQueue_Which")[TBG.Text.Ref.BG_AB] == nil)   then TBG_Saved("MGB_AutoQueue_Which")[TBG.Text.Ref.BG_AB]   = true; end
			if (TBG_Saved("MGB_AutoQueue_Which") ~= nil and TBG_Saved("MGB_AutoQueue_Which")[TBG.Text.Ref.BG_WSG] == nil)  then TBG_Saved("MGB_AutoQueue_Which")[TBG.Text.Ref.BG_WSG]  = true; end
			if (TBG_Saved("MGB_AutoQueue_Which") ~= nil and TBG_Saved("MGB_AutoQueue_Which")[TBG.Text.Ref.BG_AV] == nil)   then TBG_Saved("MGB_AutoQueue_Which")[TBG.Text.Ref.BG_AV]   = true; end
			if (TBG_Saved("MGB_AutoQueue_Which") ~= nil and TBG_Saved("MGB_AutoQueue_Which")[TBG.Text.Ref.BG_EOTS] == nil) then TBG_Saved("MGB_AutoQueue_Which")[TBG.Text.Ref.BG_EOTS] = true; end

			if (TBG_Saved("MGB_AutoJoin_PriorityList") ~= nil and TBG_Saved("MGB_AutoJoin_PriorityList")[TBG.Text.Ref.BG_AB] == nil)   then TBG_Saved_Set("MGB_AutoJoin_PriorityList", nil); end
			if (TBG_Saved("MGB_AutoJoin_PriorityList") ~= nil and TBG_Saved("MGB_AutoJoin_PriorityList")[TBG.Text.Ref.BG_WSG] == nil)  then TBG_Saved_Set("MGB_AutoJoin_PriorityList", nil); end
			if (TBG_Saved("MGB_AutoJoin_PriorityList") ~= nil and TBG_Saved("MGB_AutoJoin_PriorityList")[TBG.Text.Ref.BG_AV] == nil)   then TBG_Saved_Set("MGB_AutoJoin_PriorityList", nil); end
			if (TBG_Saved("MGB_AutoJoin_PriorityList") ~= nil and TBG_Saved("MGB_AutoJoin_PriorityList")[TBG.Text.Ref.BG_EOTS] == nil) then TBG_Saved_Set("MGB_AutoJoin_PriorityList", nil); end

			if (TBG_Saved("MGB_AutoJoin_PriorityList") == nil) then
				TBG_Saved_Set("MGB_AutoJoin_PriorityList", {
					list = {
						[1] = TBG.Text.Ref.BG_AB,
						[2] = TBG.Text.Ref.BG_WSG,
						[3] = TBG.Text.Ref.BG_AV,
						[4] = TBG.Text.Ref.BG_EOTS,
					},

					[TBG.Text.Ref.BG_AB]   = { i = 1, id = TBG.Text.Ref.BG_AB,   text = TBG.Text.Ref.BG_AB, },
					[TBG.Text.Ref.BG_WSG]  = { i = 2, id = TBG.Text.Ref.BG_WSG,  text = TBG.Text.Ref.BG_WSG, },
					[TBG.Text.Ref.BG_AV]   = { i = 3, id = TBG.Text.Ref.BG_AV,   text = TBG.Text.Ref.BG_AV, },
					[TBG.Text.Ref.BG_EOTS] = { i = 3, id = TBG.Text.Ref.BG_EOTS, text = TBG.Text.Ref.BG_EOTS, },
				});
			end

			if (TBG_Saved("MGB_AutoJoinTime") == nil)  then TBG_Saved_Set("MGB_AutoJoinTime",  10); end
			if (TBG_Saved("MGB_AutoLeaveTime") == nil) then TBG_Saved_Set("MGB_AutoLeaveTime", 10); end

			-- Interface
			if (TBG_Saved("MGI_HideMinimapButton") == nil)                	then TBG_Saved_Set("MGI_HideMinimapButton",                	false); end
			if (TBG_Saved("MGI_RedirectMessageAlliance") == nil)          	then TBG_Saved_Set("MGI_RedirectMessageAlliance",          	true);  end
			if (TBG_Saved("MGI_RedirectMessageHorde") == nil)             	then TBG_Saved_Set("MGI_RedirectMessageHorde",             	true);  end
			if (TBG_Saved("MGI_RedirectMessageNeutral") == nil)           	then TBG_Saved_Set("MGI_RedirectMessageNeutral",           	false); end
			if (TBG_Saved("MGI_BG_DoNotShowLoot") == nil)           		then TBG_Saved_Set("MGI_BG_DoNotShowLoot",          		false); end
			if (TBG_Saved("MGI_BG_DoNotShowDeaths") == nil)           		then TBG_Saved_Set("MGI_BG_DoNotShowDeaths",         	 	false); end
			if (TBG_Saved("MGI_BG_DoNotShowRaidChanges") == nil)          	then TBG_Saved_Set("MGI_BG_DoNotShowRaidChanges",           true);  end
			if (TBG_Saved("MGI_BG_DoNotShowBattlegroundChanges") == nil)	then TBG_Saved_Set("MGI_BG_DoNotShowBattlegroundChanges",	false); end
			if (TBG_Saved("MGI_BG_DoNotShow_AVOnly") == nil)           		then TBG_Saved_Set("MGI_BG_DoNotShow_AVOnly",           	false); end

			if (GetLocale() == "enUS") then
				if (TBG_Saved("MGI_AV_YellConvertHerald") == nil)    then TBG_Saved_Set("MGI_AV_YellConvertHerald",    true);  end
				if (TBG_Saved("MGI_AV_YellConvertImportant") == nil) then TBG_Saved_Set("MGI_AV_YellConvertImportant", true);  end
				if (TBG_Saved("MGI_AV_YellPreventOther") == nil)     then TBG_Saved_Set("MGI_AV_YellPreventOther",     true);  end
			else
				if (TBG_Saved("MGI_AV_YellConvertHerald") == nil)    then TBG_Saved_Set("MGI_AV_YellConvertHerald",    false);  end
				if (TBG_Saved("MGI_AV_YellConvertImportant") == nil) then TBG_Saved_Set("MGI_AV_YellConvertImportant", false);  end
				if (TBG_Saved("MGI_AV_YellPreventOther") == nil)     then TBG_Saved_Set("MGI_AV_YellPreventOther",     false);  end
			end

			if (TBG_Saved("MGI_AutoShowMinimap") == nil)                  	then TBG_Saved_Set("MGI_AutoShowMinimap",                 	true);  end
			if (TBG_Saved("MGI_HideJoinPopup") == nil)                    	then TBG_Saved_Set("MGI_HideJoinPopup",                    	false); end
			if (TBG_Saved("MGI_PlaySoundOnConfirm") == nil)               	then TBG_Saved_Set("MGI_PlaySoundOnConfirm",               	false); end
			if (TBG_Saved("MGI_LeaderBoard_UseCustomLeaderBoard") == nil) 	then TBG_Saved_Set("MGI_LeaderBoard_UseCustomLeaderBoard", 	true);  end
			if (TBG_Saved("MGI_ScoreFrame_ColorNamesByClass") == nil)     	then TBG_Saved_Set("MGI_ScoreFrame_ColorNamesByClass",     	true);  end
			if (TBG_Saved("MGI_ScoreFrame_ColorNamesByClassAll") == nil)  	then TBG_Saved_Set("MGI_ScoreFrame_ColorNamesByClassAll",  	true);  end
			if (TBG_Saved("MGI_ScoreFrame_ColorNamesNotPlayer") == nil)   	then TBG_Saved_Set("MGI_ScoreFrame_ColorNamesNotPlayer",   	false); end
			if (TBG_Saved("MGI_ScoreFrame_PlayerNameMark") == nil)        	then TBG_Saved_Set("MGI_ScoreFrame_PlayerNameMark",        	true);  end
			if (TBG_Saved("MGI_PlaySoundWhenQueue") == nil)               	then TBG_Saved_Set("MGI_PlaySoundWhenQueue",               	true);  end
			if (TBG_Saved("MGI_RepeatQueuedSound") == nil)                	then TBG_Saved_Set("MGI_RepeatQueuedSound",                	false); end
			if (TBG_Saved("MGI_RepeatConfirmSound") == nil)               	then TBG_Saved_Set("MGI_RepeatConfirmSound",                false); end

			if (TBG_Saved("MGI_PlayWhatSoundOnConfirm") == nil) then TBG_Saved_Set("MGI_PlayWhatSoundOnConfirm", 1); end

		-- Display
		if (TBG_Saved("MD_ShowSpacers") == nil)           then TBG_Saved_Set("MD_ShowSpacers",           true);  end
		if (TBG_Saved("MD_DisablePercentInAV") == nil)    then TBG_Saved_Set("MD_DisablePercentInAV",    true);  end
		if (TBG_Saved("MD_AllowInaccurateTimers") == nil) then TBG_Saved_Set("MD_AllowInaccurateTimers", false); end

		if (TBG_Saved("MD_TT_AlwaysShow") == nil)       then TBG_Saved_Set("MD_TT_AlwaysShow",       false); end
		if (TBG_Saved("MD_TT_WhenNoQueueHide") == nil)  then TBG_Saved_Set("MD_TT_WhenNoQueueHide",  true);  end
		if (TBG_Saved("MD_TT_ShowOverTooltip") == nil)  then TBG_Saved_Set("MD_TT_ShowOverTooltip",  true);  end
		if (TBG_Saved("MD_TT_StaticWidth") == nil)      then TBG_Saved_Set("MD_TT_StaticWidth",      false); end
		if (TBG_Saved("MD_TT_AllowMouseClicks") == nil) then TBG_Saved_Set("MD_TT_AllowMouseClicks", true);  end

		if (TBG_Saved("MD_TT_Detach") == nil)                 then TBG_Saved_Set("MD_TT_Detach",                 false); end
		if (TBG_Saved("MD_TT_Lock") == nil)                   then TBG_Saved_Set("MD_TT_Lock",                   false); end
		if (TBG_Saved("MD_TT_Detach_SecondaryButton") == nil) then TBG_Saved_Set("MD_TT_Detach_SecondaryButton", true);  end

		if (TBG_Saved("MD_TT_AllowAutoScroll") == nil)        then TBG_Saved_Set("MD_TT_AllowAutoScroll",        false); end
		if (TBG_Saved("MD_TT_AllowManualScroll") == nil)      then TBG_Saved_Set("MD_TT_AllowManualScroll",      true);  end

		if (TBG_Saved("MD_TT_DetachCorner") == nil) then TBG_Saved_Set("MD_TT_DetachCorner", "TOPRIGHT"); end

		if (TBG_Saved("MD_TT_Color_Background") == nil) then
			TBG_Saved_Set("MD_TT_Color_Background", {
				r = 0, g = 0, b = 0, a = 0.6, });
		end

		if (TBG_Saved("MD_TT_Color_Border") == nil) then
			TBG_Saved_Set("MD_TT_Color_Border", {
				r = 0.341, g = 0.341, b = 0.341, a = 1, });
		end

		if (TBG_Saved("MD_TT_Lines") == nil)           then TBG_Saved_Set("MD_TT_Lines",           TBG.Var.Display.tt_lines_max);    end
		if (TBG_Saved("MD_TT_MaxWidth") == nil)        then TBG_Saved_Set("MD_TT_MaxWidth",        500);                             end
		if (TBG_Saved("MD_TT_HideDelay") == nil)       then TBG_Saved_Set("MD_TT_HideDelay",       0.5);                             end
		if (TBG_Saved("MD_TT_AutoScrollSpeed") == nil) then TBG_Saved_Set("MD_TT_AutoScrollSpeed", TBG.Var.Display.tt_update_delay); end
		if (TBG_Saved("MD_TT_FrameStrata") == nil)     then TBG_Saved_Set("MD_TT_FrameStrata",     "MEDIUM");                        end

		if (TBG_Saved("MD_TT_DisplayOrder") ~= nil and TBG_Saved("MD_TT_DisplayOrder")["PLAYER_STATISTICS"] == nil)    then TBG_Saved_Set("MD_TT_DisplayOrder", nil); end
		if (TBG_Saved("MD_TT_DisplayOrder") ~= nil and TBG_Saved("MD_TT_DisplayOrder")["AB_WIN_ESTIMATES"] == nil)     then TBG_Saved_Set("MD_TT_DisplayOrder", nil); end
		if (TBG_Saved("MD_TT_DisplayOrder") ~= nil and TBG_Saved("MD_TT_DisplayOrder")["MOVEMENT_TIMERS"] == nil)      then TBG_Saved_Set("MD_TT_DisplayOrder", nil); end
		if (TBG_Saved("MD_TT_DisplayOrder") ~= nil and TBG_Saved("MD_TT_DisplayOrder")["PROGRESS_ESTIMATES"] == nil)   then TBG_Saved_Set("MD_TT_DisplayOrder", nil); end
		if (TBG_Saved("MD_TT_DisplayOrder") ~= nil and TBG_Saved("MD_TT_DisplayOrder")["CAPTURE_TIMERS"] == nil)       then TBG_Saved_Set("MD_TT_DisplayOrder", nil); end
		if (TBG_Saved("MD_TT_DisplayOrder") ~= nil and TBG_Saved("MD_TT_DisplayOrder")["WSG_FLAG_TRACKER"] == nil)     then TBG_Saved_Set("MD_TT_DisplayOrder", nil); end
		if (TBG_Saved("MD_TT_DisplayOrder") ~= nil and TBG_Saved("MD_TT_DisplayOrder")["OBJECTIVE_STATISTICS"] == nil) then TBG_Saved_Set("MD_TT_DisplayOrder", nil); end
		if (TBG_Saved("MD_TT_DisplayOrder") ~= nil and TBG_Saved("MD_TT_DisplayOrder")["FRIENDS"] == nil)              then TBG_Saved_Set("MD_TT_DisplayOrder", nil); end
		if (TBG_Saved("MD_TT_DisplayOrder") ~= nil and TBG_Saved("MD_TT_DisplayOrder")["SERVER_STATISTICS"] == nil)    then TBG_Saved_Set("MD_TT_DisplayOrder", nil); end
		if (TBG_Saved("MD_TT_DisplayOrder") ~= nil and TBG_Saved("MD_TT_DisplayOrder")["CLASS_STATISTICS"] == nil)     then TBG_Saved_Set("MD_TT_DisplayOrder", nil); end
		if (TBG_Saved("MD_TT_DisplayOrder") ~= nil and TBG_Saved("MD_TT_DisplayOrder")["INACTIVE_PLAYERS"] == nil)     then TBG_Saved_Set("MD_TT_DisplayOrder", nil); end
		if (TBG_Saved("MD_TT_DisplayOrder") ~= nil and TBG_Saved("MD_TT_DisplayOrder")["BATTLEGROUND_ITEMS"] == nil)   then TBG_Saved_Set("MD_TT_DisplayOrder", nil); end

		if (TBG_Saved("MD_TT_DisplayOrder") == nil) then
			local array = {
				list = {
					[1]  = "PLAYER_STATISTICS",
					[2]  = "AB_WIN_ESTIMATES",
					[3]  = "MOVEMENT_TIMERS",
					[4]  = "PROGRESS_ESTIMATES",
					[5]  = "CAPTURE_TIMERS",
					[6]  = "WSG_FLAG_TRACKER",
					[7]  = "OBJECTIVE_STATISTICS",
					[8]  = "FRIENDS",
					[9]  = "SERVER_STATISTICS",
					[10]  = "CLASS_STATISTICS",
					[11] = "BATTLEGROUND_ITEMS",
					[12] = "INACTIVE_PLAYERS",
				},

				["PLAYER_STATISTICS"]    = { i = 1,  id = "PLAYER_STATISTICS",    text = TBG.Text.Label.Tooltip.PLAYER_STATS, },
				["AB_WIN_ESTIMATES"]     = { i = 2,  id = "AB_WIN_ESTIMATES",     text = TBG.Text.Label.Tooltip.AB_WIN_ESTIMATE, },
				["MOVEMENT_TIMERS"]      = { i = 3,  id = "MOVEMENT_TIMERS",      text = TBG.Text.Label.Tooltip.MOVEMENT_TIMERS, },
				["PROGRESS_ESTIMATES"]   = { i = 4,  id = "PROGRESS_ESTIMATES",   text = TBG.Text.Label.Tooltip.AV_PROGRESS_ESTIMATE, },
				["CAPTURE_TIMERS"]       = { i = 5,  id = "CAPTURE_TIMERS",       text = TBG.Text.Label.Tooltip.CAPTURE_TIMERS, },
				["WSG_FLAG_TRACKER"]     = { i = 6,  id = "WSG_FLAG_TRACKER",     text = TBG.Text.Label.Tooltip.WSG_FLAG_TRACKER, },
				["OBJECTIVE_STATISTICS"] = { i = 7,  id = "OBJECTIVE_STATISTICS", text = TBG.Text.Label.Tooltip.OBJECTIVE_STATS, },
				["FRIENDS"]              = { i = 8,  id = "FRIENDS",              text = TBG.Text.Label.Tooltip.FRIENDS, },
				["SERVER_STATISTICS"]    = { i = 9,  id = "SERVER_STATISTICS",    text = TBG.Text.Label.Tooltip.SERVER_STATS, },
				["CLASS_STATISTICS"]     = { i = 10, id = "CLASS_STATISTICS",     text = TBG.Text.Label.Tooltip.CLASS_STATS, },
				["BATTLEGROUND_ITEMS"]   = { i = 11, id = "BATTLEGROUND_ITEMS",   text = TBG.Text.Label.Tooltip.BGITEMS, },
				["INACTIVE_PLAYERS"]     = { i = 12, id = "INACTIVE_PLAYERS",     text = TBG.Text.Label.Tooltip.INACTIVES, },
			};

			TBG_Saved_Set("MD_TT_DisplayOrder", array);
		end

			-- World
			if (TBG_Saved("MDW_Button_DisplayScore") == nil)           then TBG_Saved_Set("MDW_Button_DisplayScore",           false); end
			if (TBG_Saved("MDW_Button_TimeInQueue") == nil)            then TBG_Saved_Set("MDW_Button_TimeInQueue",            true);  end
			if (TBG_Saved("MDW_Button_TimeBeforeQueueExpires") == nil) then TBG_Saved_Set("MDW_Button_TimeBeforeQueueExpires", true);  end

			if (TBG_Saved("MDW_TT_DisplayNotInQueueText") == nil)  then TBG_Saved_Set("MDW_TT_DisplayNotInQueueText",  true);  end
			if (TBG_Saved("MDW_TT_DisplayHeaders") == nil)         then TBG_Saved_Set("MDW_TT_DisplayHeaders",         true); end
			if (TBG_Saved("MDW_TT_DisplayScore") == nil)           then TBG_Saved_Set("MDW_TT_DisplayScore",           false); end
			if (TBG_Saved("MDW_TT_TimeBeforeQueueExpires") == nil) then TBG_Saved_Set("MDW_TT_TimeBeforeQueueExpires", true);  end
			if (TBG_Saved("MDW_TT_RememberOpenBGs") == nil)        then TBG_Saved_Set("MDW_TT_RememberOpenBGs",        false); end

			if (TBG_Saved("MDW_Other_ModifyPopupQueueExpire") == nil) then TBG_Saved_Set("MDW_Other_ModifyPopupQueueExpire", true); end

			-- Battleground
			if (TBG_Saved("MDB_Button_ReplaceActiveWhenConfirm") == nil) then TBG_Saved_Set("MDB_Button_ReplaceActiveWhenConfirm", true);  end
			if (TBG_Saved("MDB_Button_NumberOfPlayers") == nil)          then TBG_Saved_Set("MDB_Button_NumberOfPlayers",          true);  end
			if (TBG_Saved("MDB_Button_TopServerPercentage") == nil)      then TBG_Saved_Set("MDB_Button_TopServerPercentage",      false); end
			if (TBG_Saved("MDB_Button_ServerPercentageColor") == nil)    then TBG_Saved_Set("MDB_Button_ServerPercentageColor",    false); end
			if (TBG_Saved("MDB_Button_TimeToStart") == nil)              then TBG_Saved_Set("MDB_Button_TimeToStart",              true);  end
			if (TBG_Saved("MDB_Button_Standing") == nil)                 then TBG_Saved_Set("MDB_Button_Standing",                 true);  end
			if (TBG_Saved("MDB_Button_Kills") == nil)                    then TBG_Saved_Set("MDB_Button_Kills",                    true);  end
			if (TBG_Saved("MDB_Button_KillingBlows") == nil)             then TBG_Saved_Set("MDB_Button_KillingBlows",             true);  end
			if (TBG_Saved("MDB_Button_Deaths") == nil)                   then TBG_Saved_Set("MDB_Button_Deaths",                   true);  end
			if (TBG_Saved("MDB_Button_DamageDone") == nil)               then TBG_Saved_Set("MDB_Button_DamageDone",               true);  end
			if (TBG_Saved("MDB_Button_HealingDone") == nil)              then TBG_Saved_Set("MDB_Button_HealingDone",              true);  end
			if (TBG_Saved("MDB_Button_Honor") == nil)                    then TBG_Saved_Set("MDB_Button_Honor",                    true);  end

			if (TBG_Saved("MDB_TT_SeperateActiveAndQueue") == nil) then TBG_Saved_Set("MDB_TT_SeperateActiveAndQueue", false);  end
			if (TBG_Saved("MDB_TT_TimeToStart") == nil)            then TBG_Saved_Set("MDB_TT_TimeToStart",            true);   end
			if (TBG_Saved("MDB_TT_InactivePlayers") == nil)        then TBG_Saved_Set("MDB_TT_InactivePlayers",        true);   end
			if (TBG_Saved("MDB_TT_CaptureTimers") == nil)          then TBG_Saved_Set("MDB_TT_CaptureTimers",          true);   end
			if (TBG_Saved("MDB_TT_BattlegroundItems") == nil)      then TBG_Saved_Set("MDB_TT_BattlegroundItems",      true);   end
			if (TBG_Saved("MDB_TT_BattlegroundItemsSort") == nil)  then TBG_Saved_Set("MDB_TT_BattlegroundItemsSort",  "NAME"); end
			if (TBG_Saved("MDB_TT_MovementTimers") == nil)         then TBG_Saved_Set("MDB_TT_MovementTimers",         true);   end
			if (TBG_Saved("MDB_TT_ProgressEstimates") == nil)      then TBG_Saved_Set("MDB_TT_ProgressEstimates",      true);   end
			if (TBG_Saved("MDB_TT_ABWinEstimates") == nil)         then TBG_Saved_Set("MDB_TT_ABWinEstimates",         true);   end
			if (TBG_Saved("MDB_TT_FlagTracker") == nil)            then TBG_Saved_Set("MDB_TT_FlagTracker",            true);   end
			if (TBG_Saved("MDB_TT_ServerStats_A") == nil)          then TBG_Saved_Set("MDB_TT_ServerStats_A",          false);  end
			if (TBG_Saved("MDB_TT_ServerStats_H") == nil)          then TBG_Saved_Set("MDB_TT_ServerStats_H",          false);  end
			if (TBG_Saved("MDB_TT_ServerStats_Top") == nil)        then TBG_Saved_Set("MDB_TT_ServerStats_Top",        true);   end
			if (TBG_Saved("MDB_TT_ServerStats_Percent") == nil)    then TBG_Saved_Set("MDB_TT_ServerStats_Percent",    true);   end
			if (TBG_Saved("MDB_TT_ClassStats_A") == nil)           then TBG_Saved_Set("MDB_TT_ClassStats_A",           true);   end
			if (TBG_Saved("MDB_TT_ClassStats_H") == nil)           then TBG_Saved_Set("MDB_TT_ClassStats_H",           true);   end
			if (TBG_Saved("MDB_TT_ClassStats_Name") == nil)        then TBG_Saved_Set("MDB_TT_ClassStats_Name",        false);  end
			if (TBG_Saved("MDB_TT_PlayerStats") == nil)            then TBG_Saved_Set("MDB_TT_PlayerStats",            true);   end
			if (TBG_Saved("MDB_TT_ObjectiveStats") == nil)         then TBG_Saved_Set("MDB_TT_ObjectiveStats",         false);  end
			if (TBG_Saved("MDB_TT_Friends") == nil)                then TBG_Saved_Set("MDB_TT_Friends",                false);  end
			if (TBG_Saved("MDB_TT_BattlegroundWeekend") == nil)    then TBG_Saved_Set("MDB_TT_BattlegroundWeekend",    true);   end
			if (TBG_Saved("MDB_TT_WeekStart") == nil) 			  then TBG_Saved_Set("MDB_TT_WeekStart",               TBG_GuessBattlegroundWeekend()); end
			if (TBG_Saved("MDB_TT_NoHints") == nil)                then TBG_Saved_Set("MDB_TT_NoHints",                true);   end

			if (TBG_Saved("MDB_Other_LockOnScreenDisplay") == nil)                then TBG_Saved_Set("MDB_Other_LockOnScreenDisplay",                false); end
			if (TBG_Saved("MDB_Other_CaptureTimers_OnScreen_Display") == nil)     then TBG_Saved_Set("MDB_Other_CaptureTimers_OnScreen_Display",     true);  end
			if (TBG_Saved("MDB_Other_CaptureTimers_OnScreen_Invert") == nil)      then TBG_Saved_Set("MDB_Other_CaptureTimers_OnScreen_Invert",      false); end
			if (TBG_Saved("MDB_Other_CaptureTimers_OnScreen_ClickPrint") == nil)  then TBG_Saved_Set("MDB_Other_CaptureTimers_OnScreen_ClickPrint",  true);  end
			if (TBG_Saved("MDB_Other_MovementTimers_OnScreen_Display") == nil)    then TBG_Saved_Set("MDB_Other_MovementTimers_OnScreen_Display",    true);  end
			if (TBG_Saved("MDB_Other_MovementTimers_OnScreen_ClickPrint") == nil) then TBG_Saved_Set("MDB_Other_MovementTimers_OnScreen_ClickPrint", true);  end
			if (TBG_Saved("MDB_Other_FlagTracker_OnScreen_Show") == nil)          then TBG_Saved_Set("MDB_Other_FlagTracker_OnScreen_Show",          true);  end
			if (TBG_Saved("MDB_Other_FlagTracker_OnScreen_TextClass") == nil)     then TBG_Saved_Set("MDB_Other_FlagTracker_OnScreen_TextClass",     true);  end
			if (TBG_Saved("MDB_Other_FlagTracker_OnScreen_ColorClass") == nil)    then TBG_Saved_Set("MDB_Other_FlagTracker_OnScreen_ColorClass",    true);  end
			if (TBG_Saved("MDB_Other_FlagTracker_OnScreen_ClickTarget") == nil)   then TBG_Saved_Set("MDB_Other_FlagTracker_OnScreen_ClickTarget",   true);  end
			if (TBG_Saved("MDB_Other_FlagTracker_OnScreen_AttachToScore") == nil) then TBG_Saved_Set("MDB_Other_FlagTracker_OnScreen_AttachToScore", false); end
			if (TBG_Saved("MDB_Other_FlagTracker_OnScreen_Invert") == nil)        then TBG_Saved_Set("MDB_Other_FlagTracker_OnScreen_Invert",        false); end
			if (TBG_Saved("MDB_Other_MinimapFlag_Horde") == nil)                  then TBG_Saved_Set("MDB_Other_MinimapFlag_Horde",                  true);  end
			if (TBG_Saved("MDB_Other_MinimapFlag_Alliance") == nil)               then TBG_Saved_Set("MDB_Other_MinimapFlag_Alliance",               true);  end
			if (TBG_Saved("MDB_Other_MinimapFlag_AllowMouse") == nil)             then TBG_Saved_Set("MDB_Other_MinimapFlag_AllowMouse",             true);  end
			if (TBG_Saved("MDB_Other_MinimapFlag_AllowOutOfBounds") == nil)       then TBG_Saved_Set("MDB_Other_MinimapFlag_AllowOutOfBounds",       true);  end
			if (TBG_Saved("MDB_Other_MinimapFlag_OutOfBoundsFade") == nil)        then TBG_Saved_Set("MDB_Other_MinimapFlag_OutOfBoundsFade",        true);  end
			if (TBG_Saved("MDB_Other_ScoreFrame_Detach") == nil)                  then TBG_Saved_Set("MDB_Other_ScoreFrame_Detach",                  true);  end
			if (TBG_Saved("MDB_Other_ScoreFrame_Lock") == nil)                    then TBG_Saved_Set("MDB_Other_ScoreFrame_Lock",                    true);  end
			if (TBG_Saved("MDB_Other_ScoreFrame_Hide") == nil)                    then TBG_Saved_Set("MDB_Other_ScoreFrame_Hide",                    false); end
			if (TBG_Saved("MDB_Other_MaxCameraDistanceIncrease") == nil)          then TBG_Saved_Set("MDB_Other_MaxCameraDistanceIncrease",          true);  end
			if (TBG_Saved("MDB_Other_PvPFrame_ArenaPoints") == nil)               then TBG_Saved_Set("MDB_Other_PvPFrame_ArenaPoints",               true);  end
			if (TBG_Saved("MDB_Other_PlayersList") == nil)                        then TBG_Saved_Set("MDB_Other_PlayersList",                        true);  end
			if (TBG_Saved("MDB_Other_PlayersList_PrintClass") == nil)             then TBG_Saved_Set("MDB_Other_PlayersList_PrintClass",             true);  end
			if (TBG_Saved("MDB_Other_PlayersList_ColorClass") == nil)             then TBG_Saved_Set("MDB_Other_PlayersList_ColorClass",             true);  end

			if (TBG_Saved("MDB_Other_CaptureTimers_OnScreen_DisplayHowMany") == nil) then TBG_Saved_Set("MDB_Other_CaptureTimers_OnScreen_DisplayHowMany", 14); end
	end