TBG_Files_Settings_IsLoaded = true; -- Indicates this file has been loaded into memory.

-- ## Toggles the locking of the capture timers on the screen.
function TBG_ToggleLockCaptures()

	-- Lock the on screen capture timers.
	if (TBG_Saved("MDB_Other_LockOnScreenDisplay") or not TBG_IsEnabled()) then
		TBGX_Display_Capture:Hide();

	-- Unlock the on screen capture timers.
	else
		TBGX_Display_Capture:Show();
	end
end


-- ## Hides or shows the battleground mini map button.
function TBG_ToggleMiniMapIcon()
	if (TBG_IsEnabled()) then
		if (TBG_Saved("MGI_HideMinimapButton")) then
			MiniMapBattlefieldFrame:Hide();
		elseif (TBG_IsQueuedActive()) then
			MiniMapBattlefieldFrame:Show();
		end

	else
		if (not TBG_IsQueuedActive()) then
			MiniMapBattlefieldFrame:Hide();
		else
			MiniMapBattlefieldFrame:Show();
		end
	end
end


-- ## Enables and disables the ability of the addon to modify the location of the miniature score frame.
function TBG_ToggleDetachScoreFrame()
	if (TBG_Saved("MDB_Other_ScoreFrame_Detach") and TBG_IsEnabled()) then
		if (AlwaysUpFrame1) then
			local attach, relativeTo, relativePoint, x, y = AlwaysUpFrame1:GetPoint();

			if (x ~= nil) then
				x = TBG_MathRound(x, 0);
			end

			if (y ~= nil) then
				y = TBG_MathRound(y, 0);
			end

			if (x ~= 2 or y ~= -8 or attach ~= "TOPLEFT" or relativePoint ~= "TOPLEFT" or relativeTo ~= TBGX_Display_ScoreFrame) then
				AlwaysUpFrame1:ClearAllPoints();
				AlwaysUpFrame1:SetPoint("TOPLEFT", TBGX_Display_ScoreFrame, "TOPLEFT", 2, -8);
			end
		end
	elseif (not TBG_Saved("MDB_Other_ScoreFrame_Detach") or not TBG_IsEnabled()) then
		if (AlwaysUpFrame1) then
			local attach, relativeTo, relativePoint, x, y = AlwaysUpFrame1:GetPoint();

			if (x ~= nil) then
				x = TBG_MathRound(x, 0);
			end

			if (y ~= nil) then
				y = TBG_MathRound(y, 0);
			end

			if (x ~= -23 or y ~= -20 or attach ~= "TOP" or relativeTo ~= WorldStateAlwaysUpFrame) then
				AlwaysUpFrame1:ClearAllPoints();
				AlwaysUpFrame1:SetPoint("TOP", WorldStateAlwaysUpFrame, -23 , -20);
			end
		end
	end
end


-- ## Enables and disables click targetting of the flag runner.
function TBG_ToggleClickFlagRunner()
	if (TBG_Saved("MDB_Other_FlagTracker_OnScreen_ClickTarget")) then
		TBG_Display_Flag_1_Button:EnableMouse(true);
		TBG_Display_Flag_2_Button:EnableMouse(true);
	else
		TBG_Display_Flag_1_Button:EnableMouse(false);
		TBG_Display_Flag_2_Button:EnableMouse(false);
	end
end


-- ## Enables and disables click posting of the capture timers.
function TBG_ToggleClickCaptureTimer()
	if (TBG_Saved("MDB_Other_CaptureTimers_OnScreen_ClickPrint")) then
		for i = 1, TBG.Var.Display.captures_max do
			getglobal("TBGX_Display_Capture_" .. i .. "_Button"):EnableMouse(true);
		end
	else
		for i = 1, TBG.Var.Display.captures_max do
			getglobal("TBGX_Display_Capture_" .. i .. "_Button"):EnableMouse(false);
		end
	end
end

-- ## Enables and disables click posting of the movement timers.
function TBG_ToggleClickMovementTimer()
	if (TBG_Saved("MDB_Other_MovementTimers_OnScreen_ClickPrint")) then
		TBG_Display_MovementTimer_1_Button:EnableMouse(true);
		TBG_Display_MovementTimer_2_Button:EnableMouse(true);
	else
		TBG_Display_MovementTimer_1_Button:EnableMouse(false);
		TBG_Display_MovementTimer_2_Button:EnableMouse(false);
	end
end


-- ## Changes the attachment points of the flag tracker.
function TBG_InvertFlagTracker()
	if (not TBG_Saved("MDB_Other_FlagTracker_OnScreen_Invert")) then
		TBG_Display_Flag_1:ClearAllPoints();
		TBG_Display_Flag_2:ClearAllPoints();

		TBG_Display_Flag_1:SetPoint("TOPLEFT", "TBGX_Display_Capture", "TOPLEFT");
		TBG_Display_Flag_2:SetPoint("TOPLEFT", "TBG_Display_Flag_1", "BOTTOMLEFT");
	else
		TBG_Display_Flag_1:ClearAllPoints();
		TBG_Display_Flag_2:ClearAllPoints();

		TBG_Display_Flag_1:SetPoint("BOTTOMLEFT", "TBG_Display_Flag_2", "TOPLEFT");
		TBG_Display_Flag_2:SetPoint("BOTTOMLEFT", "TBGX_Display_Capture", "BOTTOMLEFT");

	end
end


-- ## Changes the attachment of the on screen capture timers.
function TBG_ToggleInvertCaptureTimers()
	if (not TBG_Saved("MDB_Other_CaptureTimers_OnScreen_Invert")) then
		for i = 1, TBG.Var.Display.captures_max do
			frame = getglobal("TBGX_Display_Capture_" .. i);

			attach        = "TOPLEFT";
			relativeTo    = "TBGX_Display_Capture_" .. (i - 1);
			relativePoint = "BOTTOMLEFT";
			x             = 0;
			y             = 0;

			if (i == 1) then
				attach        = "TOPLEFT";
				relativeTo    = "TBGX_Display_Capture";
				relativePoint = "TOPLEFT";
			end

			frame:ClearAllPoints();
			frame:SetPoint(attach, relativeTo, relativePoint, x, y);
		end

		TBG_Display_MovementTimer_1:ClearAllPoints();
		TBG_Display_MovementTimer_1:SetPoint("TOPLEFT", TBGX_Display_Capture, "TOPLEFT", 0, 0);

		TBG_Display_MovementTimer_2:ClearAllPoints();
		TBG_Display_MovementTimer_2:SetPoint("TOPLEFT", TBG_Display_MovementTimer_1, "BOTTOMLEFT", 0, 0);
	else
		for i = 1, TBG.Var.Display.captures_max do
			frame = getglobal("TBGX_Display_Capture_" .. i);

			attach        = "BOTTOMLEFT";
			relativeTo    = "TBGX_Display_Capture_" .. (i - 1);
			relativePoint = "TOPLEFT";
			x             = 0;
			y             = 0;

			if (i == 1) then
				attach        = "BOTTOMLEFT";
				relativeTo    = "TBGX_Display_Capture";
				relativePoint = "BOTTOMLEFT";
			end

			frame:ClearAllPoints();
			frame:SetPoint(attach, relativeTo, relativePoint, x, y);
		end

		TBG_Display_MovementTimer_1:ClearAllPoints();
		TBG_Display_MovementTimer_1:SetPoint("BOTTOMLEFT", TBGX_Display_Capture, "BOTTOMLEFT", 0, 0);

		TBG_Display_MovementTimer_2:ClearAllPoints();
		TBG_Display_MovementTimer_2:SetPoint("BOTTOMLEFT", TBG_Display_MovementTimer_1, "TOPLEFT", 0, 0);
	end
end

-- ## Attaches the flag tracker to the score frame.
function TBG_ChangeAttachFlagTracker()
	if (TBG.Var.Player.zone ~= TBG.Text.Ref.BG_WSG and TBG.Var.Player.zone ~= TBG.Text.Ref.BG_EOTS) then return nil; end

	if (not TBG_Saved("MDB_Other_FlagTracker_OnScreen_AttachToScore")) then
		TBG_Display_Flag_1_Button:ClearAllPoints();
		TBG_Display_Flag_2_Button:ClearAllPoints();

		TBG_Display_Flag_1_Button:SetPoint("TOPLEFT", TBG_Display_Flag_1, "TOPRIGHT", 30, -2);
		TBG_Display_Flag_2_Button:SetPoint("TOPLEFT", TBG_Display_Flag_2, "TOPRIGHT", 30, -2);

	else
		if (AlwaysUpFrame1DynamicIconButton ~= nil and AlwaysUpFrame2DynamicIconButton ~= nil) then
			if (TBG.Var.Player.zone == TBG.Text.Ref.BG_EOTS or TBG.Var.Player.zone == TBG.Text.Ref.BG_WSG) then
				if (TBG_Display_Flag_1:IsVisible()) then
					TBG_Display_Flag_1:Hide();
					TBG_Display_Flag_1:Show();
				end

				if (TBG_Display_Flag_2:IsVisible()) then
					TBG_Display_Flag_2:Hide();
					TBG_Display_Flag_2:Show();
				end

			else
				TBG_Display_Flag_1_Button:ClearAllPoints();
				TBG_Display_Flag_2_Button:ClearAllPoints();

				TBG_Display_Flag_1_Button:SetPoint("TOPLEFT", AlwaysUpFrame1, "TOPRIGHT", 0, 0);
				TBG_Display_Flag_2_Button:SetPoint("TOPLEFT", AlwaysUpFrame2, "TOPRIGHT", 0, 0);
			end
		end
	end
end


-- ## Toggles whether or not the user can drag the score frame.
function TBG_ToggleScoreFrameLock()
	if (TBG_Saved("MDB_Other_ScoreFrame_Lock") or not TBG_IsEnabled()) then
		TBGX_Display_ScoreFrame:Hide();

	else
		TBGX_Display_ScoreFrame:Show();
	end
end


-- ## Toggles whether or not the score frame is visible.
function TBG_ToggleScoreFrameVisible(menu)
	if (TBG_Saved("MDB_Other_ScoreFrame_Hide") and TBG_IsEnabled()) then
		WorldStateAlwaysUpFrame:Hide();

		Hook_WorldStateAlwaysUpFrame_OnShow = WorldStateAlwaysUpFrame:GetScript("OnShow");

		WorldStateAlwaysUpFrame:SetScript("OnShow", function()
			if (TBG_Saved("MDB_Other_ScoreFrame_Hide")) then
				this:Hide();
			end
		end);

	elseif (menu) then
		WorldStateAlwaysUpFrame:SetScript("OnShow", Hook_WorldStateAlwaysUpFrame_OnShow);
		WorldStateAlwaysUpFrame:Show();
		WorldStateScoreFrame_OnLoad();
		WorldStateAlwaysUpFrame_Update();
	end
end


-- ## Toggles whether or not the maximum camera distance in interface options is increased.
function TBG_ToggleMaxCameraDistanceIncrease()
	local original = 2;

	for index, value in pairs(UIOptionsFrameSliders) do
		if (value.text == "MAX_FOLLOW_DIST") then
			if (TBG_Saved("MDB_Other_MaxCameraDistanceIncrease")) then
				value.maxValue = 3.4;
			else
				value.maxValue = original;
			end

			if ((GetCVar("cameraDistanceMaxFactor") + 0) > value.maxValue) then
				SetCVar("cameraDistanceMaxFactor", value.maxValue);
			end

			break;
		end
	end
end


-- ## Toggles what profile is used.
function TBG_Settings_ModifyProfile(profile)
	TBG.Var.Addon.LOADED = false;
	TBG_Saved_SetProfile(profile);
	TBG_Init_CreateSavedVariables();
	TBG.Var.Addon.LOADED = true;
	TBG_Menu_SetTab(-1);
	TBG_Menu_SetTab(1);
end


function TBG_Settings_ToggleTitan(set)
	if (not TBG.Var.Addon.TITAN.detected) then return nil; end

	if (TitanPanelSettings) then
		stack = debugstack(2);

		if (TitanPanel_IsPluginShown(TBG.Var.Const.ADDON_ID)) then
			TBG_Hooks_HookBlizzard();

			TBG.Var.Player.zone         = nil;
			TBG.Var.Addon.TITAN.enabled = true;

			TBG_ZonesChanged();
		else
			TBG.Var.Addon.TITAN.enabled = false;

			if (TBG.Var.Addon.LOADED and TBG.Var.Addon.TITAN.scanned and string.find(stack, ":OnClick")) then
				TBG_Hooks_RestoreBlizzard();
				TBG_Init_AddonSetup();
				TBGX_Menu:Hide();
				StaticPopup_Show(TBG.Var.Const.DIALOG_CONSOLE_RELOADUI);
			end
		end

		TBG.Var.Addon.TITAN.scanned = true;
	end
end


-- ## Resets the position of draggable on screen frames.
function TBG_ResetOnScreenDragFramePositions()
	local _, scoreFrameRelativeTo, _, _, _    = TBGX_Display_ScoreFrame:GetPoint();
	local _, tooltipAttachRelativeTo, _, _, _ = TBGX_Tooltip_Attach:GetPoint();

	TBGX_Display_Capture:ClearAllPoints();
	TBGX_Display_Capture:SetPoint("CENTER", UIParent, "CENTER", 0, 0);

	TBGX_Display_ScoreFrame:ClearAllPoints();
	TBGX_Display_ScoreFrame:SetPoint("TOPLEFT", scoreFrameRelativeTo, "TOPLEFT", 743, -24);

	TBGX_Tooltip_Attach:ClearAllPoints();
	TBGX_Tooltip_Attach:SetPoint("CENTER", tooltipAttachRelativeTo, "CENTER", 0, 0);

	TBGX_Menu:ClearAllPoints();
	TBGX_Menu:SetPoint("CENTER", UIParent, "CENTER", 0, 0);
end

-- ## Resets saved variables.
function TBG_ResetSavedVariables()
	TBG_Saved_Reset();
end