TBG_Files_Tooltip_IsLoaded = true; -- Indicates this file has been loaded into memory.

-- #############################
-- ## INITALISATION FUNCTIONS ##
-- #############################

-- ## Loads the tooltip, setting positioning.
function TBG_Tooltip_Load()
	TBGX_Tooltip_Background:SetBackdropColor(TBG_Saved("MD_TT_Color_Background").r, TBG_Saved("MD_TT_Color_Background").g, TBG_Saved("MD_TT_Color_Background").b, TBG_Saved("MD_TT_Color_Background").a);
	TBGX_Tooltip_Background:SetBackdropBorderColor(TBG_Saved("MD_TT_Color_Border").r, TBG_Saved("MD_TT_Color_Border").g, TBG_Saved("MD_TT_Color_Border").b, TBG_Saved("MD_TT_Color_Border").a);

	if (TBG_Saved("MD_TT_AlwaysShow") and (not TBG_Saved("MD_TT_WhenNoQueueHide") or (TBG_Saved("MD_TT_WhenNoQueueHide") and (TBG_IsQueuedActive() or TBG_IsBattleground(TBG.Var.Player.zone))))) then
		TBGX_Tooltip:EnableMouse(0);
	else
		TBGX_Tooltip:EnableMouse(1);
	end

	TBGX_Tooltip_Background:EnableMouse(0);
end


-- ## Tooltip update function.
function TBG_Tooltip_Update()
	if (not TBG.Var.Addon.LOADED) then return nil; end
	if (not TBG_IsEnabled())      then return nil; end

	if (TBGX_Tooltip:IsVisible()) then

		-- Check if the mouse is no longer over the tooltip.
		if ((not TBG_Saved("MD_TT_AlwaysShow") or (TBG_Saved("MD_TT_WhenNoQueueHide") and not TBG_IsQueuedActive() and not TBG_IsSupported(TBG.Var.Player.zone))) and TBG_Saved("MD_TT_ShowOverTooltip")) then
			local mouse = GetMouseFocus();

			if (mouse) then
				mouse = mouse:GetName();

				if (mouse) then
					if (string.find(mouse, "TBGX_Tooltip")) then
						TBG.Var.Display.tt_mouse_over_last = 0;
					end
				end
			end
		end

		-- Update the text of the tooltip.
		if (GetTime() - TBG.Var.Display.tt_update >= TBG.Var.Display.tt_update_delay) then
			TBG.Var.Display.tt_update = GetTime();
			TBG_Display_SetText();
		end

		-- Scroll lines if required.
		if (TBG.Var.Display.tt_more_lines and TBG_Saved("MD_TT_AllowAutoScroll") and (not TBG_Saved("MD_TT_AlwaysShow") or (TBG_Saved("MD_TT_WhenNoQueueHide") and not TBG_IsQueuedActive() and not TBG_IsSupported(TBG.Var.Player.zone)))) then
			if (GetTime() - TBG.Var.Display.tt_scroll >= TBG_Saved("MD_TT_AutoScrollSpeed")) then
				TBG.Var.Display.tt_scroll = GetTime();

				TBG.Var.Display.tt_start_line = TBG.Var.Display.tt_start_line + 1;
			end
		end

		-- Check if timeout occured and the tooltip needs to be hidden.
		if (TBG.Var.Display.tt_mouse_over_last > 0) then
			if (GetTime() - TBG.Var.Display.tt_mouse_over_last >= TBG_Saved("MD_TT_HideDelay")) then
				TBG_Tooltip_Hide();
			end
		end
	end
end


-- ## Sets the text being displayed in the tooltip.
function TBG_Tooltip_SetText(text)
	local count   = 1;
	local fcount  = 1;
	local f       = "TBGX_Tooltip";
	local fleft   = getglobal(f .. "_L" .. fcount);
	local fright  = getglobal(f .. "_R" .. fcount);
	local height  = 0;
	local width   = TBG.Var.Display.tt_width_min;
	local stop    = false;
	local adding  = false;

	if (TBG_Saved("MD_TT_StaticWidth")) then
		width = TBG_Saved("MD_TT_MaxWidth");
	end

	-- If the start line is out of bounds, adjust it.
	if ((TBG.Var.Display.tt_start_line - 1) + TBG_Saved("MD_TT_Lines") > (TBG.Var.Display.tt_total_lines - 1)) then
		TBG.Var.Display.tt_start_line = TBG.Var.Display.tt_total_lines - TBG_Saved("MD_TT_Lines")

		if (TBG.Var.Display.tt_start_line < 1) then
			TBG.Var.Display.tt_start_line = 1;
		end
	end

	-- Show header if this is the first line.
	if (TBG.Var.Display.tt_start_line == 1) then
		fleft:SetText(TBG.Var.Const.COLOR_BLUE .. TBG.Text.Menu.HEADER);
		fright:SetText("");

		fleft:Show();
		fright:Show();

		fcount = fcount + 1;
		height = height + math.ceil(fleft:GetHeight()) + 2;
	end

	count = count + 1;

	-- Loop through all of the lines of text.
	while (true) do
		local left  = text;
		local right = "";

		-- If there are more lines of text after this line, separate them.
		if (string.find(text, "\n")) then
			left = string.sub(text, 0, string.find(text, "\n") - 1);
			text = string.sub(text, string.find(text, "\n") + 1, string.len(text));
		else
			stop = true;
		end

		-- If there are two columns of text in this line.
		if (string.find(left, "\t")) then
			right = string.sub(left, string.find(left, "\t") + 1, string.len(left));
			left  = string.sub(left, 0, string.find(left, "\t") - 1);
		end

		-- If we are showing the text from this line.
		if (count >= TBG.Var.Display.tt_start_line and not adding) then

			-- Blank lines only print correctly if they contain a space.
			if (left  == "") then left  = " "; end
			if (right == "") then right = " "; end

			getglobal("TBGX_Tooltip_Button_" .. fcount):Hide();

			-- If this is the capture timers header.
			if (string.find(left, TBG.Var.Const.REF_CAPTURE_TIMERS)) then
				left = string.gsub(left, TBG.Var.Const.REF_CAPTURE_TIMERS, "", 1);

				if (TBG_Saved("MD_TT_AllowMouseClicks")) then
					getglobal("TBGX_Tooltip_Button_" .. fcount):SetScript("OnClick", function()
						if (IsShiftKeyDown()) then
							TBG_Battleground_PrintCaptureTimers();
						end
					end);

					getglobal("TBGX_Tooltip_Button_" .. fcount):Show();
				end

			-- If this is a capture timer.
			elseif (string.find(left, TBG.Var.Const.REF_CAPTURE_TIMER)) then
				      left = string.gsub(left, TBG.Var.Const.REF_CAPTURE_TIMERS, "", 1);
				local str  = left;
				      left = string.sub(left, string.find(left, "}}") + 2, string.len(left));

				if (TBG_Saved("MD_TT_AllowMouseClicks")) then
					local who  = string.sub(str, string.find(str, "who=") + 4, string.find(str, "|name=") - 1);
					local name = string.sub(str, string.find(str, "name=") + 5, string.find(str, "|eta=") - 1);
					local eta  = string.sub(str, string.find(str, "eta=") + 4, string.find(str, "}}") - 1);

					getglobal("TBGX_Tooltip_Button_" .. fcount):SetScript("OnClick", function()
						if (IsShiftKeyDown()) then
							TBG_Battleground_PrintCaptureTimer(who, name, eta);
						end
					end);

					getglobal("TBGX_Tooltip_Button_" .. fcount):Show();
				end

			-- If this is a movement timer.
			elseif (string.find(left, TBG.Var.Const.REF_MOVEMENT_TIMER)) then
				      left = string.gsub(left, TBG.Var.Const.REF_MOVEMENT_TIMER, "", 1);
				local str  = left;
				      left = string.sub(left, string.find(left, "}}") + 2, string.len(left));

				if (TBG_Saved("MD_TT_AllowMouseClicks")) then
					local name = string.sub(str, string.find(str, "name=") + 5, string.find(str, "|eta=") - 1);
					local eta  = string.sub(str, string.find(str, "eta=") + 4, string.find(str, "}}") - 1);

					getglobal("TBGX_Tooltip_Button_" .. fcount):SetScript("OnClick", function()
						if (IsShiftKeyDown()) then
							TBG_Battleground_PrintMovementTimer(name, eta);
						end
					end);

					getglobal("TBGX_Tooltip_Button_" .. fcount):Show();
				end

			-- If this is a flag tracker.
			elseif (string.find(left, TBG.Var.Const.REF_FLAG)) then
				      left = string.gsub(left, TBG.Var.Const.REF_FLAG, "", 1);
				local str  = left;
				      left = string.sub(left, string.find(left, "}}") + 2, string.len(left));

				if (TBG_Saved("MD_TT_AllowMouseClicks")) then
					local faction = string.sub(str, string.find(str, "faction=") + 8, string.find(str, "|player=") - 1);
					local player  = string.sub(str, string.find(str, "player=") + 7, string.find(str, "|class=") - 1);
					local class   = string.sub(str, string.find(str, "class=") + 6, string.find(str, "}}") - 1);

					if (TBG.Var.Player.zone == TBG.Text.Ref.BG_EOTS) then
						faction = TBG.Var.Const.FACTION_NAME.enemy[faction];
					end

					getglobal("TBGX_Tooltip_Button_" .. fcount):SetScript("OnClick", function()
						if (IsShiftKeyDown()) then
							TBG_Battleground_PrintFlagCarrier(faction, player, class);
						end
					end);

					getglobal("TBGX_Tooltip_Button_" .. fcount):Show();
				end
			end


			fleft  = getglobal(f .. "_L" .. fcount);
			fright = getglobal(f .. "_R" .. fcount);

			fleft:SetWidth(0);
			fright:SetWidth(0);
			fleft:SetHeight(0);
			fright:SetHeight(0);

			-- Print the lines and show the strings.
			fleft:SetText(left);
			fright:SetText(right);

			fleft:Show();
			fright:Show();

			-- Find out how wide this text will make the frame.
			local w = math.ceil(fleft:GetWidth() + fright:GetWidth()) + 4;
			height  = height + math.ceil(fleft:GetHeight()) + 2;

			-- If this is the widest text so far, this is how wide the frame will be.
			if (w > width) then
				if (w > TBG_Saved("MD_TT_MaxWidth")) then
					width = TBG_Saved("MD_TT_MaxWidth");

					if (right ~= "" and right ~= " ") then
						fleft:SetWidth(TBG_Saved("MD_TT_MaxWidth") - fright:GetWidth());
					else
						fleft:SetWidth(TBG_Saved("MD_TT_MaxWidth"));
					end
				else
					width = w;
				end
			end

			fcount = fcount + 1;
		end

		count = count + 1;

		-- If no more frames to display lines in.
		if (fcount > TBG_Saved("MD_TT_Lines")) then
			adding = true;
		end

		-- If no more lines.
		if (stop) then
			break;
		end
	end

	-- Set the width of the frame.
	getglobal(f):SetWidth(width);
	getglobal(f):SetHeight(height);
	TBGX_Tooltip_Dummy:SetWidth(width);

	fcount                         = fcount - 1;
	TBG.Var.Display.tt_total_lines = count;

	local s  = f .. "_Slider";
	local bg = getglobal(f .. "_Background");
	local tr = f .. "_R" .. 1;
	local br = f .. "_R" .. fcount;

	-- If the user wants to allow the frame to be manually scrolled, then set up the scroll bar.
	if (TBG_Saved("MD_TT_AllowManualScroll")) then
		tr = s;
		br = s;

		if (fcount == 0) then fcount = 1; end

		-- Place the scroll bar.
		getglobal(s):ClearAllPoints();
		getglobal(s):SetPoint("TOPLEFT", f .. "_R" .. 1, "TOPRIGHT", 4, -16);
		getglobal(s):SetPoint("BOTTOMLEFT", f .. "_R" .. fcount, "BOTTOMRIGHT", 4, 16);

		-- How many lines are there that the user can click down to.
		local clicks = TBG.Var.Display.tt_total_lines - TBG_Saved("MD_TT_Lines");

		-- If the up button should be enabled.
		if (TBG.Var.Display.tt_start_line > 1) then
			TBGX_Tooltip_Slider_Up:Enable();
		else
			TBGX_Tooltip_Slider_Up:Disable();
		end

		-- If the down button should be enabled.
		if (TBG.Var.Display.tt_start_line < (TBG.Var.Display.tt_total_lines - TBG_Saved("MD_TT_Lines"))) then
			TBGX_Tooltip_Slider_Down:Enable();

			TBG.Var.Display.tt_more_lines = true;
		else
			TBGX_Tooltip_Slider_Down:Disable();

			TBG.Var.Display.tt_more_lines = false;
		end

		-- If only one page, disable the scroll bar.
		if (TBG.Var.Display.tt_total_lines < TBG_Saved("MD_TT_Lines")) then
			getglobal(s):SetMinMaxValues(1, 1);
			getglobal(s):SetValue(1);
		else
			if (clicks == 1) then
				clicks = 2;
			end

			getglobal(s):SetMinMaxValues(1, clicks);
			getglobal(s):SetValueStep(1);

			if (not TBG.Var.Display.tt_slider_moved) then
				getglobal(s):SetValue(TBG.Var.Display.tt_start_line);
			end
		end

		getglobal(s):Show();
	else
		getglobal(s):Hide();
	end

	-- Set the background of the tooltip.
	bg:ClearAllPoints();
	bg:SetPoint("TOPLEFT", f .. "_L" .. 1, "TOPLEFT",  -10, 10);
	bg:SetPoint("TOPRIGHT", tr, "TOPRIGHT",  10, 10);
	bg:SetPoint("BOTTOMLEFT", f .. "_L" .. fcount, "BOTTOMLEFT",  -10, -10);
	bg:SetPoint("BOTTOMRIGHT", br, "BOTTOMRIGHT",  10, -10);

	-- Hide unused text frames.
	for i = fcount + 1, TBG.Var.Display.tt_lines_max do
		local fleft  = getglobal(f .. "_L" .. i);
		local fright = getglobal(f .. "_R" .. i);

		fleft:SetText("");
		fright:SetText("");
		fleft:Hide();
		fright:Hide();
	end
end


-- ## Shows the tooltip.
function TBG_Tooltip_Show()
	if (TBG_IsEnabled() and not TBG.Var.Addon.drop_down_menu_open) then
		TBGX_Tooltip:Show();
		TBGX_Tooltip_Background:Show();
	end
end


-- ## Hides the tooltip.
function TBG_Tooltip_Hide(force)
	if (force or not TBG_Saved("MD_TT_AlwaysShow") or not TBG_IsEnabled() or (TBG_Saved("MD_TT_AlwaysShow") and TBG_Saved("MD_TT_WhenNoQueueHide") and not TBG_IsQueuedActive() and not TBG_IsBattleground(_player_zone))) then
		TBGX_Tooltip:Hide();
		TBGX_Tooltip_Background:Hide();

		TBG.Var.Display.tt_start_line = 1;
	end
end


-- ## Hides and shows the tooltip when it is always shown.
function TBG_Tooltip_AlwaysShow()
	local show = false;

	if (TBG_Saved("MD_TT_AlwaysShow") and TBG_IsEnabled()) then
		if (not TBG_Saved("MD_TT_WhenNoQueueHide")) then
			show = true;
		else
			if (TBG_IsQueuedActive() or TBG_IsBattleground(TBG.Var.Player.zone)) then
				show = true;
			end
		end
	end

	if (TBG_Saved("MD_TT_AlwaysShow") and TBG_IsEnabled() and (not TBG_Saved("MD_TT_WhenNoQueueHide") or (TBG_Saved("MD_TT_WhenNoQueueHide") and (TBG_IsQueuedActive() or TBG_IsBattleground(TBG.Var.Player.zone))))) then
		TBG_Tooltip_Show();
	else
		TBG_Tooltip_Hide();
	end
end


-- ## When the up scroll button is clicked in the tooltip.
function TBG_Tooltip_ClickUp()
	if (TBG.Var.Display.tt_start_line > 1) then
		TBG.Var.Display.tt_start_line = TBG.Var.Display.tt_start_line - 1;
		TBG_Display_SetText();
	end
end


-- ## When the down scroll button is clicked in the tooltip.
function TBG_Tooltip_ClickDown()
	if (TBG.Var.Display.tt_start_line < (TBG.Var.Display.tt_total_lines - TBG_Saved("MD_TT_Lines"))) then
		TBG.Var.Display.tt_start_line = TBG.Var.Display.tt_start_line + 1;
		TBG_Display_SetText();
	end
end


-- ## When the tooltip slider is moved.
function TBG_Tooltip_MoveSlider()
	TBG.Var.Display.tt_start_line = this:GetValue();
	TBG_Display_SetText();
end


-- ## Returns how far the tooltip is off screen.
function TBG_Tooltip_GetOffScreen()

	-- The tooltip is too far too the left.
	if (TBGX_Tooltip_Dummy and TBGX_Tooltip_Dummy:GetLeft() and TBGX_Tooltip_Dummy:GetLeft() * TBGX_Tooltip_Dummy:GetEffectiveScale() < UIParent:GetLeft() * UIParent:GetEffectiveScale()) then
		return "LEFT", TBGX_Tooltip_Dummy:GetLeft() - 10;

	-- The tooltip is too far to the right.
	elseif (TBGX_Tooltip_Dummy and TBGX_Tooltip_Dummy:GetRight() and TBGX_Tooltip_Dummy:GetRight() * TBGX_Tooltip_Dummy:GetEffectiveScale() > UIParent:GetRight() * UIParent:GetEffectiveScale()) then
		return "RIGHT", (UIParent:GetRight() * UIParent:GetEffectiveScale() - TBGX_Tooltip_Dummy:GetRight() * TBGX_Tooltip_Dummy:GetEffectiveScale()) + 10;
	end

	return "NONE";
end


-- ## Changes whether the tooltip is attached to the Titan Panel button, or otherwise.
function TBG_Tooltip_SetTooltipPosition()
	if (TBG_Saved("MD_TT_Detach")) then
		TBG_Tooltip_ChangeTooltipAttachment(TBG_Saved("MD_TT_DetachCorner"));

		if ((not TBG_Saved("MD_TT_Lock") or TBG_Saved("MD_TT_Detach_SecondaryButton")) and TBG_IsEnabled()) then
			TBGX_Tooltip_Attach:Show();
		end

	else
		if (TitanPanelSettings and TitanUtils_GetWhichBar(TBG.Var.Const.ADDON_ID) == "AuxBar") then
			TBGX_Tooltip:ClearAllPoints();
			TBGX_Tooltip:SetPoint("BOTTOMRIGHT", TitanPanelTitanBGButton, "TOPRIGHT", 0, 26);

			TBGX_Tooltip_Dummy:ClearAllPoints();
			TBGX_Tooltip_Dummy:SetPoint("BOTTOMRIGHT", TitanPanelTitanBGButton, "TOPRIGHT", 0, 26);

		else
			TBGX_Tooltip:ClearAllPoints();
			TBGX_Tooltip:SetPoint("TOPRIGHT", TitanPanelTitanBGButton, "BOTTOMRIGHT", 0, -14);

			TBGX_Tooltip_Dummy:ClearAllPoints();
			TBGX_Tooltip_Dummy:SetPoint("TOPRIGHT", TitanPanelTitanBGButton, "BOTTOMRIGHT", 0, -14);

		end

		TBGX_Tooltip_Attach:Hide();
	end
end


-- ## Changes the layer level at which the tooltip is displayed.
function TBG_Tooltip_FrameStrata()
	TBGX_Tooltip_Background:SetFrameStrata(TBG_Saved("MD_TT_FrameStrata"));
	TBGX_Tooltip:SetFrameStrata(TBG_Saved("MD_TT_FrameStrata"));

	for i = 1, 35 do
		getglobal("TBGX_Tooltip_Button_" .. i):SetFrameStrata(TBG_Saved("MD_TT_FrameStrata"));
	end
end


-- ## Changes by what corner to tooltip is attached to the drag frame.
function TBG_Tooltip_ChangeTooltipAttachment(attach)
	local relativePoint = "";
	local x             = 0;
	local y             = 0;
	local slider        = 0;

	-- If the slider is being shown.
	if (TBG_Saved("MD_TT_AllowManualScroll")) then
		slider = TBGX_Tooltip_Slider:GetWidth() + 3;
	end

	if (attach == "TOPLEFT") then
		relativePoint = "BOTTOMRIGHT";
		y             = -4;
		x             = 4;

	elseif (attach == "TOPRIGHT") then
		relativePoint = "BOTTOMLEFT";
		y             = -4;
		x             = -4 - slider;

	elseif (attach == "BOTTOMLEFT") then
		relativePoint = "TOPRIGHT";
		y             = -4;
		x             = 4;

	elseif (attach == "BOTTOMRIGHT") then
		relativePoint = "TOPLEFT";
		y             = -4;
		x             = -4 - slider;
	end

	TBGX_Tooltip:ClearAllPoints();
	TBGX_Tooltip:SetPoint(attach, "TBGX_Tooltip_Attach", relativePoint, x, y);
end


-- ## Prevents the drag frame from being moved around.
function TBG_Tooltip_LockTooltip()
	if (TBG_Saved("MD_TT_Detach") and TBG_IsEnabled()) then
		if (not TBG_Saved("MD_TT_Lock") or TBG_Saved("MD_TT_Detach_SecondaryButton")) then
			TBGX_Tooltip_Attach:Show();
		else
			TBGX_Tooltip_Attach:Hide();
		end
	else
		TBGX_Tooltip_Attach:Hide();
	end
end