TBG_Files_Utils_IsLoaded = true; -- Indicates this file has been loaded into memory.


-- ## Returns the profile the player is currently using.
function TBG_Saved_GetProfile()
	if (TBGSV["List"] and TBGSV["List"][TBG.Var.Player.realm] and TBGSV["List"][TBG.Var.Player.realm][TBG.Var.Player.name]) then
		return TBGSV["List"][TBG.Var.Player.realm][TBG.Var.Player.name]["Profile"];
	else
		return nil;
	end
end


-- ## Sets the profile.
function TBG_Saved_SetProfile(profile)
	if (TBGSV["List"] and TBGSV["List"][TBG.Var.Player.realm] and TBGSV["List"][TBG.Var.Player.realm][TBG.Var.Player.name]) then
		if (profile == nil) then profile = TBG.Var.Const.ADDON_DEFAULT_PROFILE; end
		TBGSV["List"][TBG.Var.Player.realm][TBG.Var.Player.name]["Profile"] = profile;
	end
end


-- ## Resets the values for the current profile to the default.
function TBG_Saved_Reset()
	TBGSV["Profiles"][TBG_Saved_GetProfile()] = { };
	TBG_Init_CreateSavedVariables();
end


-- ## Returns a saved variable.
function TBG_Saved(var)
	if (TBGSV and TBGSV["Profiles"] and TBGSV["Profiles"][TBG_Saved_GetProfile()]) then
		return TBGSV["Profiles"][TBG_Saved_GetProfile()][var];
	else
		return nil;
	end
end


-- ## Sets a saved variable.
function TBG_Saved_Set(var, value)
	if (TBGSV and TBGSV["Profiles"] and TBGSV["Profiles"][TBG_Saved_GetProfile()]) then
		TBGSV["Profiles"][TBG_Saved_GetProfile()][var] = value;
	else
	end
end


-- ## Indicates if the zone can be processed by the addon.
function TBG_IsSupported(zone)
	if (TBG_IsBattleground(zone) or TBG_IsArena(zone)) then
		return true;
	else
		return false;
	end
end


-- ## Determines if a given zone is an arena.
function TBG_IsArena(zone)
	if (TBG.DB.bgs.names[zone] ~= nil and TBG.DB.bgs.names[zone].type == TBG.Var.Const.ARENA) then
		return true;
	else
		return false;
	end
end


-- ## Determines if a given zone is a battleground.
function TBG_IsBattleground(zone)
	if (TBG.DB.bgs.names[zone] ~= nil and TBG.DB.bgs.names[zone].type == TBG.Var.Const.BATTLEGROUND) then
		return true;
	else
		return false;
	end
end


-- ## Indicates if the addon is enabled.
function TBG_IsEnabled()
	return TBG.Var.Addon.TITAN.enabled;
end


-- ## Indicates if the person is currently in a battleground queue or in a battleground.
function TBG_IsQueuedActive()
	local in_queue = false;

	-- Loop through and check on the status of the battlegrounds.
	for i = 1, MAX_BATTLEFIELD_QUEUES do
		local status, _, _ = GetBattlefieldStatus(i);

		-- If the user is in a queue or in a battleground.
		if (status ~= TBG.Var.Const.BG_STATUS_NONE) then

			in_queue = true;
			break;
		end
	end

	return in_queue;
end


-- ## Indicates if the person is currently in queue for a battleground or arena.
function TBG_IsQueued()
	local in_queue = false;

	-- Loop through and check on the status of the battlegrounds.
	for i = 1, MAX_BATTLEFIELD_QUEUES do
		local status, _, _ = GetBattlefieldStatus(i);

		-- If the user is in a queue or in a battleground.
		if (status == TBG.Var.Const.BG_STATUS_QUEUED) then
			in_queue = true;
			break;
		end
	end

	return in_queue;
end


-- ## Indicates if all team sizes for a given arena are inactive.
function TBG_IsArenaActive(name)
	local n = TBG.DB.bgs.names[name];

	for s = 1, table.getn(TBG.Var.Const.ARENA_SIZES) do
		local a = n.arenas[TBG.Var.Const.ARENA_SIZES[s]];

		if (a.status ~= TBG.Var.Const.BG_STATUS_NONE) then
			return true;
		end
	end

	return false;
end


-- ## Determines if the specified battleground or arena are the one that is being joined or left.
function TBG_IsAuto(type, name)
	if (name ~= type.name) then
		return false;
	end

	return true;
end


-- ## Determines if it is okay to automatically join a battleground at this time.
function TBG_IsAutoJoinAllowed(name)
	if (TBG_IsArena(name)) then
		return false;
	end

	-- If the user does not want to auto join.
	if (not TBG_Saved("MGB_AutoJoin")) then
		return false;
	end

	-- If the user is currently in a rated arena and does not want to auto join.
	if (TBG_Saved("MGB_AutoJoin_NotInArena")) then
		local arena, rated = IsActiveBattlefieldArena();

		if (arena and rated) then
			return false;
		end
	end

	-- Do not auto join if afk.
	if (TBG_Saved("MGB_AutoJoin_NotIfAFK") and UnitIsAFK("player")) then
		return false;
	end

	-- Battleground zone not supported.
	if (not name or not TBG_IsSupported(name)) then
		return false;
	end

	-- Player is in a battleground.
	if (TBG_IsSupported(TBG.Var.Player.zone)) then

		-- Player does not want to always auto join.
		if (not TBG_Saved("MGB_AutoJoin_Always")) then
			return false;
		end

		-- No priority list set up.
		if (not TBG_Saved("MGB_AutoJoin_PriorityList")) then
			return false;
		end

		-- If the new battleground is not on the priority list.
		if (not TBG_Saved("MGB_AutoJoin_PriorityList")[name]) then
			return false;
		end

		-- If the current zone is on the priority list.
		if (TBG_Saved("MGB_AutoJoin_PriorityList")[TBG.Var.Player.zone]) then

			-- If the new zone is of a lower priority than the current zone.
			if (TBG_Saved("MGB_AutoJoin_PriorityList")[name].i > TBG_Saved("MGB_AutoJoin_PriorityList")[TBG.Var.Player.zone].i) then
				return false;
			end
		end
	end

	return true;
end


-- ## Returns if the modifier key to skip automatic quest, gossip and queue actions is pressed.
function TBG_IsModifierKeyDown()
	if (TBG_Saved("MGB_AutoDisableKey") == "ALT") then
		return IsAltKeyDown();

	elseif (TBG_Saved("MGB_AutoDisableKey") == "CTRL") then
		return IsControlKeyDown();

	elseif (TBG_Saved("MGB_AutoDisableKey") == "SHIFT") then
		return IsShiftKeyDown();

	else
		return false;
	end
end


-- ## Prints a message to the user's console.
function TBG_Print(s)
	local str  = TBG.Var.Const.COLOR_BLUE .. "[" .. TBG.Text.ADDON_NAME_SHORT .. "): " .. TBG.Var.Const.COLOR_WHITE;

	if (s == nil)   then s = "nil";   end
	if (s == true)  then s = "true";  end
	if (s == false) then s = "false"; end

	DEFAULT_CHAT_FRAME:AddMessage(str .. s);
end


-- ## Prints a message to a popup frame.
function TBG_MessagePopup(s)
	if (s == nil)   then s = "nil";   end
	if (s == true)  then s = "true";  end
	if (s == false) then s = "false"; end

	TBGX_Message_Text:SetText(s);
	TBGX_Message:SetHeight(TBGX_Message_Text:GetHeight() + 90);
	TBGX_Message:Show();
end


-- ## Prints a tip to the user's console.
function TBG_Print_Tip(s)
	local str  = TBG.Var.Const.COLOR_BLUE .. "[" .. TBG.Text.ADDON_NAME_SHORT .. "): " .. TBG.Var.Const.COLOR_GREEN .. string.upper(TBG.Text.Message.TIP) .. "! " .. TBG.Var.Const.COLOR_WHITE;
	DEFAULT_CHAT_FRAME:AddMessage(str .. s);
end

-- ## Returns the printed time. Removes white space around the text.
function TBG_SecondsToTime(t)
	return strtrim(SecondsToTime(t));
end


-- ## Returns the short printed time. Removes white space around text.
function TBG_SecondsToTimeAbbrev(t)
	return strtrim(SecondsToTimeAbbrev(t));
end


-- ## Returns the full name of a battleground or arena.
function TBG_GetInstanceName(bg, short)
	local name = bg.name;

	if (short) then
		name = bg.short;
	end

	if (bg.type == TBG.Var.Const.ARENA) then
		if (short) then
			name = name .. " (" .. bg.team_size .. ")";
		else
			name = name .. " (" .. bg.team_size .. "v" .. bg.team_size .. ")";
		end
	end

	return name;
end


-- ## Rounds off 'n' to the 'decimal' place.
function TBG_MathRound(n, decimal)
	local result = n;

	if (decimal == 0 or decimal == nil) then
		if (n - math.floor(n) >=  math.ceil(n) - n) then
			result = math.ceil(n);
		else
			result = math.floor(n);
		end
	else
		local shift  = 10 * decimal;
		      result = floor(n * shift + 0.5) / shift;
	end

	return result;
end


-- ## Creates a percentage value.
function TBG_MathPercent(n, total, decimal)
	local result = TBG_MathRound((n / total) * 100, decimal);

	if (total == 0) then
		result = 0;
	end

	return (result .. "%"), result;
end


-- ## Determines the difference between two numbers.
function TBG_MathDifference(n1, n2)
	local n = n1 - n2;
	if (n < 0) then n = n * -1; end
	return n;
end


-- ## Splits a string by a given pattern or set of characters.
function TBG_Split(text, s, blank, include)
	local array = { };

	while (true) do
		local o    = text;
		local i, e = string.find(text, s);

		if (i ~= nil) then
			local word = string.sub(text, 1, i - 1);
			      text = string.sub(text, e + 1);

			if (word == "" and blank == false) then
			else
				table.insert(array, word);
			end

			if (include ~= nil and include) then
				table.insert(array, string.sub(o, i, e));
			end
		else
			if (text == "" and blank == false) then
			else
				table.insert(array, text);
			end

			break;
		end
	end

	return array;
end


-- ## Determines if the given year is a leap year.
function TBG_IsLeapYear(year)
	if (year % 4 == 0) then
		if (year % 100 == 0) then
			if (year % 400 == 0) then
				return true;
			else
				return false;
			end
		else
			return true;
		end
	else
		return false;
	end
end


-- ## Determines how many days in a given month.
function TBG_DaysInMonth(month, year)
	if (month == 2) then
		if (TBG_IsLeapYear(year)) then
			return 29;
		else
			return 28;
		end

	elseif ((month <= 7 and month % 2 == 1) or (month > 7 and month % 2 == 0)) then
		return 31;
	else
		return 30;
	end
end


-- ## Returns the date following the specified amount of days subtracted.
-- ## Limited to subtracting 7 days.
function TBG_DateSubtractDays(date1, days)
	if (days > 7) then return nil; end

	-- If the subtracted days are within this month.
	if (date1.d - days >= 1) then
		date1.d = date1.d - days;

	-- If the subtracted days result in it not being this month.
	else
		date1.d = TBG_DaysInMonth(date1.m - 1, date1.y) - (days - date1.d);
		date1.m = date1.m - 1;
	end

	return date1;
end


-- ## Returns the difference between two dates. Second date must be after the first.
function TBG_GetDateDifference(date1, date2)
	local total = 0;

	local years  = TBG_MathDifference(date1.y, date2.y);
	local months = TBG_MathDifference(date1.m, date2.m);
	local days   = TBG_MathDifference(date1.d, date2.d);

	-- If this is the same month and year, find the difference between the two days.
	if (years == 0 and months == 0) then
		total = days;

	-- If this is the same year, but different months, find the difference between.
	elseif (years == 0) then

		-- Loop through and add the whole months.
		if (months > 1) then
			for m = date1.m + 1, date2.m - 1 do
				total = total + TBG_DaysInMonth(m, date1.y);
			end
		end

	-- If this is different years.
	else

		-- Loop through and add the days for whole years.
		if (TBG_MathDifference(date1.y, date2.y) > 1) then
			for y = date1.y + 1, date2.y - 1 do
				total = total + 365;

				if (TBG_IsLeapYear(y)) then
					total = total + 1;
				end
			end
		end

		-- Loop through and add the whole months for the first year.
		for m = date1.m + 1, 12 do
			total = total + TBG_DaysInMonth(m, date1.y);
		end

		-- Loop through and add the whole months for the end year.
		for m = 1, date2.m - 1 do
			total = total + TBG_DaysInMonth(m, date2.y);
		end
	end

	-- If not the same year and month.
	if (months > 0) then

		-- Add the rest of the days for the start date partial month.
		total = total + (TBG_DaysInMonth(date1.m, date1.y) - date1.d);

		-- Add the rest of the days for the end date partial month.
		total = total + date2.d;

	end

	return total;
end