--------------------------------------------------------------------------
-- Globals
--------------------------------------------------------------------------

SLASH_RELOAD1 = "/rl"
SlashCmdList["RELOAD"] = ReloadUI

TITAN_FACTIONS_ID = "Factions"
TITAN_FACTIONS_VERSION = GetAddOnMetadata("TitanFactions","Version")
TITAN_FACTIONS_ICON = "Interface\\AddOns\\TitanFactions\\TitanFactions"

--------------------------------------------------------------------------
-- Locals
--------------------------------------------------------------------------

local TITAN_FACTIONS_SESSION_STARTED = false

local standingLabel = {
	[1] = "|cff8b0000"..FACTION_STANDING_LABEL1.."|r",	-- 36000 Hated
	[2] = "|cffff1919"..FACTION_STANDING_LABEL2.."|r",	-- 3000 Hostile
	[3] = "|cffff8C00"..FACTION_STANDING_LABEL3.."|r",	-- 3000 Unfriendly
	[4] = "|cffc0c0c0"..FACTION_STANDING_LABEL4.."|r",	-- 3000 Neutral
	[5] = "|cffffffff"..FACTION_STANDING_LABEL5.."|r",	-- 6000 Friendly
	[6] = "|cff19ff19"..FACTION_STANDING_LABEL6.."|r",	-- 12000 Honored
	[7] = "|cff4169e1"..FACTION_STANDING_LABEL7.."|r",	-- 21000 Revered
	[8] = "|cff9932cc"..FACTION_STANDING_LABEL8.."|r",	-- 1000 Exalted
}

local runeclothFactions = {
	TITAN_FACTIONS_DARKSPEAR,
	TITAN_FACTIONS_ORGRIMMAR,
	TITAN_FACTIONS_SILVERMOON,
	TITAN_FACTIONS_THUNDERBLUFF,
	TITAN_FACTIONS_UNDERCITY,
	TITAN_FACTIONS_DARNASSUS,
	TITAN_FACTIONS_EXODAR,
	TITAN_FACTIONS_GNOMEREGAN,
	TITAN_FACTIONS_IRONFORGE,
	TITAN_FACTIONS_STORMWIND,
}

local sessionStart = {}

--------------------------------------------------------------------------
-- Core functions
--------------------------------------------------------------------------

function TitanPanelFactionsButton_OnLoad()
	this.registry = {
		id = TITAN_FACTIONS_ID,
		version = TITAN_FACTIONS_VERSION,
		menuText = TITAN_FACTIONS_MENU_TEXT,
		buttonTextFunction = "TitanPanelFactionsButton_GetButtonText",
		tooltipTitle = TITAN_FACTIONS_TOOLTIP,
		tooltipTextFunction = "TitanPanelFactionsButton_GetTooltipText",
		icon = TITAN_FACTIONS_ICON,
		iconWidth = 16,
		category = "Information",
		savedVariables = {
			ShowIcon = 1,
			ShowLabelText = 1,
			ShowPercent = TITAN_NIL,
			ShowRaw = TITAN_NIL,
			HideExalted = TITAN_NIL,
			ShowToGo = TITAN_NIL,
			ShowSession = TITAN_NIL,
			ShowRunecloth = TITAN_NIL,
			ShowMonitor = TITAN_NIL,
			MonitorBlizzard = TITAN_NIL,
			MonitorFaction = TITAN_NIL,
			Debug = TITAN_NIL,
		}
	}
	this:RegisterEvent("UPDATE_FACTION")
end


function TitanPanelFactionsButton_OnEvent()
	if TitanGetVar(TITAN_FACTIONS_ID, "ShowMonitor") == 1 then
		TitanFactions_UpdateMonitor()
		TitanPanelButton_UpdateButton(TITAN_FACTIONS_ID)
	end
	TitanPanelButton_UpdateTooltip()
end

--------------------------------------------------------------------------
-- Titan functions
--------------------------------------------------------------------------

function TitanPanelFactionsButton_GetButtonText(id)

	if not TITAN_FACTIONS_SESSION_STARTED then
		TitanFactions_StartSession()
		TitanFactions_Debug("Titan Factions: Sessions initialized in GetButtonText.")
		TITAN_FACTIONS_SESSION_STARTED = true
	end

	if TitanGetVar(TITAN_FACTIONS_ID, "ShowLabelText") then
		if TitanGetVar(TITAN_FACTIONS_ID, "ShowMonitor") and TitanGetVar(TITAN_FACTIONS_ID, "MonitorFaction") ~= nil then

			if TitanGetVar(TITAN_FACTIONS_ID, "MonitorBlizzard") then
				local name
				local wname = GetWatchedFactionInfo()
				if wname and not string.find(TitanGetVar(TITAN_FACTIONS_ID, "MonitorFaction"), wname) then
					for i = 1, GetNumFactions() do
						name = GetFactionInfo(i)
						if name == wname then
							TitanSetVar(TITAN_FACTIONS_ID, "MonitorFaction", TitanFactions_GetMonitorText(i))
							break
						end
					end
				end
			end

			return format(TITAN_FACTIONS_BUTTON_TEXT, TitanGetVar(TITAN_FACTIONS_ID, "MonitorFaction"))
		else
			return TITAN_FACTIONS_BUTTON_LABEL
		end
	end
end


function TitanPanelFactionsButton_GetTooltipText()

	local tooltipText = {}

	local name, standing, barMin, barMax, barValue, atWarWith, isHeader, isCollapsed, isWatched
	local thisLine, factionText, sessionText, runeclothText, percentText, rawText

	local _, race = UnitRace("player")
	local repPer = string.lower(race) == "human" and 82 or 75
	local rcNeed

	for i = 1, GetNumFactions() do
		name, _, standing, barMin, barMax, barValue, atWarWith, _, isHeader, _, _ = GetFactionInfo(i)
		if name == TITAN_FACTIONS_INACTIVE then
			break
		end
		if not isHeader then
			if TitanGetVar(TITAN_FACTIONS_ID, "HideExalted") and standing == 8 then
				-- do nothing
			else
				-- Faction [gain] NN RC		Standing NN% (NNN/NNNN)

				factionText = ""
				if standing > 3 then
					factionText = "|cff19ff19"..name.."|r"
				elseif standing == 3 then
					factionText = "|cffffff00"..name.."|r"
				elseif standing < 3 then
					factionText = "|cffff1919"..name.."|r"
				end

				runeclothText = ""
				if TitanGetVar(TITAN_FACTIONS_ID, "ShowRunecloth") then
					if name == TITAN_FACTIONS_DARKSPEAR
					or name == TITAN_FACTIONS_ORGRIMMAR
					or name == TITAN_FACTIONS_SILVERMOON
					or name == TITAN_FACTIONS_THUNDERBLUFF
					or name == TITAN_FACTIONS_UNDERCITY
					or name == TITAN_FACTIONS_DARNASSUS
					or name == TITAN_FACTIONS_EXODAR
					or name == TITAN_FACTIONS_GNOMEREGAN
					or name == TITAN_FACTIONS_IRONFORGE
					or name == TITAN_FACTIONS_STORMWIND
					then
						rcNeed = math.ceil( ( (42000 - barValue) / repPer ) + 0.5)
						if rcNeed > 0 then
							runeclothText = " |cffffaec8("..rcNeed.." RC)|r"
						end
					end
				end

				sessionText = ""
				if TitanGetVar(TITAN_FACTIONS_ID, "ShowSession") then
					if not sessionStart[name] then
						sessionStart[name] = 0
					end
					sessionGain = barValue - sessionStart[name]
					if sessionGain > 0 then
						sessionText = " [+"..sessionGain.."]"
					elseif sessionGain < 0 then
						sessionText = " ["..sessionGain.."]"
					end
				end

				percentText = ""
				if TitanGetVar(TITAN_FACTIONS_ID, "ShowPercent") then
					percentText = " "..math.floor( (barValue - barMin)/(barMax - barMin) * 100 ).."%"
				end

				rawText = ""
				if TitanGetVar(TITAN_FACTIONS_ID, "ShowRaw") then
					rawText = " ("..barValue - barMin.."/"..barMax - barMin..")"
				end

				thisLine = factionText..runeclothText.."\t"..sessionText..standingLabel[standing]..percentText..rawText.."\n"
			end
		else
			thisLine = "\n|cffffffff"..name.."|r\n"
		end
		table.insert(tooltipText, thisLine)
	end
	tooltipText = table.concat(tooltipText, "")
	-- remove the last \n
	return string.sub(tooltipText, 1, string.len(tooltipText) - 1)
end

--------------------------------------------------------------------------
-- create 2nd level right-click menu
--------------------------------------------------------------------------
function TitanPanelRightClickMenu_PrepareFactionsMenu()

	local info = {}

	if UIDROPDOWNMENU_MENU_LEVEL == 2 then
		if UIDROPDOWNMENU_MENU_VALUE == "DisplayAbout" then
			info = {}
			info.text = TITAN_FACTIONS_ABOUT_POPUP_TEXT
			info.value = "AboutTextPopUP"
			info.notClickable = 1
			info.isTitle = 0
			UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL)
		end

		if UIDROPDOWNMENU_MENU_VALUE == "Monitor" then

			info = {}
			info.text = TITAN_FACTIONS_MONITOR_TOGGLE_TEXT
			info.func = TitanFactions_MonitorToggle
			info.checked = TitanGetVar(TITAN_FACTIONS_ID, "ShowMonitor")
			UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL)

			info = {}
			info.text = TITAN_FACTIONS_MONITORBLIZZARD_TOGGLE_TEXT
			info.func = TitanFactions_MonitorBlizzardToggle
			info.checked = TitanGetVar(TITAN_FACTIONS_ID, "MonitorBlizzard")
			UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL)

			info = {}
			UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL)

			local name, atWarWith, isHeader

			for i = 1, GetNumFactions() do

				info = {}

				name, _, standing, _, _, _, atWarWith, _, isHeader, _, _ = GetFactionInfo(i)

				if name == TITAN_FACTIONS_INACTIVE then
					break
				end

				if not isHeader then

					if TitanGetVar(TITAN_FACTIONS_ID, "HideExalted") and standing == 8 then
						-- do nothing
					else
						if standing > 3 then
							info.text = "|cff19ff19"..name.."|r"
						elseif standing == 3 then
							info.text = "|cffffff00"..name.."|r"
						elseif standing < 3 then
							info.text = "|cffff1919"..name.."|r"
						else
							info.text = name
						end

						info.value = name

						info.func = TitanFactions_SetMonitorFaction

						if TitanGetVar(TITAN_FACTIONS_ID, "MonitorFaction") and string.find(TitanGetVar(TITAN_FACTIONS_ID, "MonitorFaction"), name) then
							info.checked = 1
						else
							info.checked = nil
						end

						UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL)
					end
				end

			end
		end

		return
	end

	TitanPanelRightClickMenu_AddTitle(TitanPlugins[TITAN_FACTIONS_ID].menuText)

	info = {}
	info.text = TITAN_FACTIONS_MONITOR
	info.value = "Monitor"
	info.hasArrow = 1
	UIDropDownMenu_AddButton(info)

	info = {}
	UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL)

	info = {}
	info.text = TITAN_FACTIONS_SHOW_PERCENT
	info.func = TitanFactions_ShowPercentToggle
	info.checked = TitanGetVar(TITAN_FACTIONS_ID, "ShowPercent")
	UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL)

	info = {}
	info.text = TITAN_FACTIONS_SHOW_RAW
	info.func = TitanFactions_ShowRawToggle
	info.checked = TitanGetVar(TITAN_FACTIONS_ID, "ShowRaw")
	UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL)

	info = {}
	info.text = TITAN_FACTIONS_SHOW_TO_GO
	info.func = TitanFactions_ShowToGoToggle
	info.checked = TitanGetVar(TITAN_FACTIONS_ID, "ShowToGo")
	UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL)

	info = {}
	UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL)

	info = {}
	info.text = TITAN_FACTIONS_SHOW_SESSION
	info.func = TitanFactions_ShowSessionToggle
	info.checked = TitanGetVar(TITAN_FACTIONS_ID, "ShowSession")
	UIDropDownMenu_AddButton(info, UIDROPDOWN_MENU_LEVEL)

	info = {}
	info.text = TITAN_FACTIONS_SHOW_RUNECLOTH
	info.func = TitanFactions_ShowRuneclothToggle
	info.checked = TitanGetVar(TITAN_FACTIONS_ID, "ShowRunecloth")
	UIDropDownMenu_AddButton(info, UIDROPDOWN_MENU_LEVEL)

	info = {}
	UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL)

	info = {}
	info.text = TITAN_FACTIONS_HIDE_EXALTED
	info.func = TitanFactions_HideExaltedToggle
	info.checked = TitanGetVar(TITAN_FACTIONS_ID, "HideExalted")
	UIDropDownMenu_AddButton(info, UIDROPDOWN_MENU_LEVEL)

	info = {}
	UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL)

	TitanPanelRightClickMenu_AddToggleIcon(TITAN_FACTIONS_ID)
	TitanPanelRightClickMenu_AddToggleLabelText(TITAN_FACTIONS_ID)
	TitanPanelRightClickMenu_AddCommand(TITAN_PANEL_MENU_HIDE, TITAN_FACTIONS_ID, TITAN_PANEL_MENU_FUNC_HIDE)

end

--------------------------------------------------------------------------
-- Factions functions
--------------------------------------------------------------------------
-- Round function
function round(x)
	return floor(x + 0.5)
end

function TitanFactions_UpdateMonitor()

	if TitanGetVar(TITAN_FACTIONS_ID, "MonitorFaction") then
		local name
		for i = 1, GetNumFactions() do
			name = GetFactionInfo(i)
			if string.find(TitanGetVar(TITAN_FACTIONS_ID, "MonitorFaction"), name) then
				TitanSetVar(TITAN_FACTIONS_ID, "MonitorFaction", TitanFactions_GetMonitorText(i))
				break
			end
		end

	elseif TitanGetVar(TITAN_FACTIONS_ID, "MonitorBlizzard") then
		local wname = GetWatchedFactionInfo()
		if wname then
			local name
			for i = 1, GetNumFactions() do
				name = GetFactionInfo(i)
				if name == wname then
					TitanSetVar(TITAN_FACTIONS_ID, "MonitorFaction", TitanFactions_GetMonitorText(i))
					break
				end
			end
		end

	end
end

function TitanFactions_MonitorToggle()
	if TitanGetVar(TITAN_FACTIONS_ID, "ShowMonitor") then
		TitanSetVar(TITAN_FACTIONS_ID, "ShowMonitor", nil)
	else
		TitanSetVar(TITAN_FACTIONS_ID, "ShowMonitor", 1)
		if TitanGetVar(TITAN_FACTIONS_ID, "ShowLabelText") ~= 1 then
			TitanSetVar(TITAN_FACTIONS_ID, "ShowLabelText", 1)
		end
	end
	TitanPanelButton_UpdateButton(TITAN_FACTIONS_ID)
end

function TitanFactions_MonitorBlizzardToggle()
	if TitanGetVar(TITAN_FACTIONS_ID, "MonitorBlizzard") then
		TitanSetVar(TITAN_FACTIONS_ID, "MonitorBlizzard", nil)
	else
		TitanSetVar(TITAN_FACTIONS_ID, "MonitorBlizzard", 1)
	end
--	if TitanGetVar(TITAN_FACTIONS_ID, "ShowMonitor") then
			TitanFactions_UpdateMonitor()
--	end
	TitanPanelButton_UpdateButton(TITAN_FACTIONS_ID)
end

function TitanFactions_SetMonitorFaction()

	local wasMonitoringBlizzard = false

	-- temporarily disable watch sync so button text update doesn't prevent us changing monitored faction
	if ( TitanGetVar(TITAN_FACTIONS_ID, "MonitorBlizzard") == 1 ) then
		TitanSetVar(TITAN_FACTIONS_ID, "MonitorBlizzard", nil)
		wasMonitoringBlizzard = true
	end

	local name
	for i = 1, GetNumFactions() do
		name = GetFactionInfo(i)
		if string.find(this.value, name) then
			TitanSetVar(TITAN_FACTIONS_ID, "MonitorFaction", TitanFactions_GetMonitorText(i))
		end
	end

	-- if we disabled watch sync, enable it now and update watched faction to match new monitored faction
	if wasMonitoringBlizzard then
		local name
		for i = 1, GetNumFactions() do
			name = GetFactionInfo(i)
			if string.find(TitanGetVar(TITAN_FACTIONS_ID, "MonitorFaction"), name) then
				SetWatchedFactionIndex(i)
				break
			end
		end
		TitanSetVar(TITAN_FACTIONS_ID, "MonitorBlizzard", 1)
	end

	TitanPanelButton_UpdateButton(TITAN_FACTIONS_ID)
end

function TitanFactions_ShowRawToggle()
	if TitanGetVar(TITAN_FACTIONS_ID, "ShowRaw") then
		TitanSetVar(TITAN_FACTIONS_ID, "ShowRaw", nil)
	else
		TitanSetVar(TITAN_FACTIONS_ID, "ShowRaw", 1)
	end
	if TitanGetVar(TITAN_FACTIONS_ID, "ShowMonitor") then
		TitanFactions_UpdateMonitor()
	end
	TitanPanelButton_UpdateButton(TITAN_FACTIONS_ID)
end

function TitanFactions_ShowPercentToggle()
	if TitanGetVar(TITAN_FACTIONS_ID, "ShowPercent") then
		TitanSetVar(TITAN_FACTIONS_ID, "ShowPercent", nil)
	else
		TitanSetVar(TITAN_FACTIONS_ID, "ShowPercent", 1)
	end
	if TitanGetVar(TITAN_FACTIONS_ID, "ShowMonitor") then
		TitanFactions_UpdateMonitor()
	end
	TitanPanelButton_UpdateButton(TITAN_FACTIONS_ID)
end

function TitanFactions_ShowToGoToggle()
	if TitanGetVar(TITAN_FACTIONS_ID, "ShowToGo") then
		TitanSetVar(TITAN_FACTIONS_ID, "ShowToGo", nil)
	else
		TitanSetVar(TITAN_FACTIONS_ID, "ShowToGo", 1)
	end
	if TitanGetVar(TITAN_FACTIONS_ID, "ShowMonitor") then
		TitanFactions_UpdateMonitor()
	end
	TitanPanelButton_UpdateButton(TITAN_FACTIONS_ID)
end

function TitanFactions_ShowSessionToggle()
	if TitanGetVar(TITAN_FACTIONS_ID, "ShowSession") then
		TitanSetVar(TITAN_FACTIONS_ID, "ShowSession", nil)
	else
		TitanSetVar(TITAN_FACTIONS_ID, "ShowSession", 1)
	end
	TitanPanelButton_UpdateButton(TITAN_FACTIONS_ID)
end

function TitanFactions_ShowRuneclothToggle()
	if TitanGetVar(TITAN_FACTIONS_ID, "ShowRunecloth") then
		TitanSetVar(TITAN_FACTIONS_ID, "ShowRunecloth", nil)
	else
		TitanSetVar(TITAN_FACTIONS_ID, "ShowRunecloth", 1)
	end
	TitanPanelButton_UpdateButton(TITAN_FACTIONS_ID)
end

function TitanFactions_HideExaltedToggle()
	if TitanGetVar(TITAN_FACTIONS_ID, "HideExalted") then
		TitanSetVar(TITAN_FACTIONS_ID, "HideExalted", nil)
	else
		TitanSetVar(TITAN_FACTIONS_ID, "HideExalted", 1)
	end
	TitanPanelButton_UpdateButton(TITAN_FACTIONS_ID)
end

--------------------------------------------------------------------------
-- Function to record beginning reputation values for session gain tracking
-- Contributed by Nyrine of WoWInterface.com in the comments section of FuBar_Factions
-- Adapted for Titan Panel [Factions] by Phanx
--------------------------------------------------------------------------
function TitanFactions_StartSession()
	local name, barValue
	for i = 1, GetNumFactions() do
		name, _, _, _, _, barValue, _, _, _, _, _ = GetFactionInfo(i)
		if not isHeader then
			sessionStart[name] = barValue
			TitanFactions_Debug("Titan Factions: "..name.." starting at "..barValue)
		end
	end
end

function TitanFactions_GetMonitorText(i)
	local name, _, standing, barMin, barMax, barValue, atWarWith, _, isHeader, _, _ = GetFactionInfo(i)
	local factionText, percentText, rawText, toGoText

	if standing > 3 then
		factionText = TitanUtils_GetGreenText(name)
	elseif standing < 3 then
		factionText = TitanUtils_GetRedText(name)
	else
		factionText = TitanUtils_GetNormalText(name)
	end

	if TitanGetVar(TITAN_FACTIONS_ID, "ShowPercent") then
		percentText = " "..math.floor( (barValue - barMin) / (barMax - barMin) * 100 ).."%"
	else
		percentText = ""
	end

	if TitanGetVar(TITAN_FACTIONS_ID, "ShowRaw") then
		rawText = " ("..barValue - barMin.."/"..barMax - barMin..")"
	else
		rawText = ""
	end

	if TitanGetVar(TITAN_FACTIONS_ID, "ShowToGo") then
		toGoText = " "..string.format(TITAN_FACTIONS_TO_GO, barMax - barValue)
	else
		toGoText = ""
	end

	return factionText..": "..standingLabel[standing]..percentText..rawText..toGoText
end

--------------------------------------------------------------------------
-- Enable chatframe output for debugging
-- /script TitanSetVar(TITAN_FACTIONS_ID,"Debug",1)
--------------------------------------------------------------------------
function TitanFactions_Debug(msg)
	if TitanGetVar(TITAN_FACTIONS_ID, "Debug") then
		DEFAULT_CHAT_FRAME:AddMessage(msg)
	end
end