--------------------------------------------------------------------------
-- TitanSkills.lua 
--------------------------------------------------------------------------
--[[

Titan Panel [Skills] plug-in for Titan Panel that displays professions, secondary skills and Weapons Skills when hovered over. 
Skills are color coded so that you can see what needs to be worked on.  Holding SHIFT while hovering over the icon will
display the equivilent character level of the skills.  For example: Fishing 200/300 = 40.  So, to level your fishing skill, you
would need to fish in an area with level 40 mobs.

This mod hadn't been updated in a long time like so many others out there.  Though this mod is very simple, I found that I
really couldn't live without it and decided to update it.  Thanks to Corgi, the original author of this mod and all the other
contributors that helped him make the mod what it was.

Author: Morsultima  

v0.21 (December 11, 2006)
- toc# updated for 2.00 patch
- no longer shows the riding skill in the list

v0.19 (October 16, 2006)
- fixed bugs with menus not remembering settings
- fixed bug related to showing character's level on bar
- reverted to old way of displaying skill bonuses which should fix any problems
   with them not being displayed
- display now defaults to rank and character level on
- Holding SHIFT now toggles display even without moving the mouse off and back
   onto the icon.

v0.15 (October 12, 2006)
- you can now select to display skills two ways: rank (xxx/xxx format), or as 
   equivilent character level
- you can toggle the display of your skills by holding SHIFT while hovering over the icon
- fixed a problem where the tray icon was not showing all the menu options
- changed the tray icon so it matches the default Titan Bar tray icons

v0.10 (September 14, 2006)
- added color coding based on skill in professions and weapons
- miscellaneous UI changes to bring mod up to current version of WoW and Titan
- toc# updated for 1.12 patch

---

(all versions below are by Corgi)

v0.09 (April 5, 2006 22:00 PST)
- fixed level change
- toc# updated for 1.10 patch

v0.08 (January 5, 2006 20:05 PST)
- added ability to toggle player's Level 
- Shift-click opens Skills frame
- toc# updated for 1.9 patch

v0.07 (October 26, 2005 10:18 PST)
- toc# updated for 1.8 patch
- added Skills (Right) options (thanks Forger)
- tooltip will now popup below/above Titan bar (thanks Forger)
- added skill bonuses (e.g. Skill 10(+5)/300) (thanks Forger)

v0.06 (September 24, 2005 19:11 PST)
- toc# updated for 1.7 patch
- added French localization

v0.05 (June 13, 2005 14:30 PST)
- updated for Titan Panel 1.24

v0.04 (June 7, 2005 14:25 PST)
- toc updated for 1.5 patch
- minor french localization

v0.03 (June 6, 2005 12:35 PST)
- complete German translation by Crowley
- added transparent icon

v0.02 (June 3, 2005 20:00 PST)
- list non-passive Class Skills (ie, Rogue's Lockpicking and Poison skills)

v0.01 (May 31, 2005 2:00 PST)
- Initial Release

]]--

TITAN_SKILLS_RED = RED_FONT_COLOR_CODE;
TITAN_SKILLS_ORANGE = "|cffff8020";
TITAN_SKILLS_YELLOW = "|cffffff20";
TITAN_SKILLS_GREEN = GREEN_FONT_COLOR_CODE;
TITAN_SKILLS_WHITE = HIGHLIGHT_FONT_COLOR_CODE;
TITAN_SKILLS_BLUE = "|cff6666ff";
TITAN_SKILLS_GREY = "|cffdddddd";

TITAN_SKILLS_ID = "Skills";
TITAN_SKILLS_RIGHT_ID = "SkillsRight";

TITAN_SKILLS_ICON = "Interface\\Addons\\TitanSkills\\Artwork\\TitanSkills";
TITAN_SKILLS_RIGHT_ICON = "Interface\\Addons\\TitanSkills\\Artwork\\TitanSkillsRight";
--
-- OnFuctions
--
function TitanPanelSkillsButton_OnLoad()
	this.registry = { 
		id = TITAN_SKILLS_ID,
		menuText = TITAN_SKILLS_MENU_TEXT, 
		buttonTextFunction = "TitanPanelSkillsButton_GetButtonText", 
		tooltipTitle = TITAN_SKILLS_TOOLTIP,
		tooltipTextFunction = "TitanPanelSkillsButton_GetTooltipText",
		icon = TITAN_SKILLS_ICON,
		iconWidth = 16,
		frequency = 0.5,
		updateType = TITAN_PANEL_UPDATE_TOOLTIP,
		savedVariables = {
			ShowIcon = 1,
			ShowLabelText = 1,
			ShowLevel = TITAN_NIL,
			ShowRank = 1,
			ShowCharLevel = 1,
		}
	};
	this:RegisterEvent("SKILL_LINES_CHANGED");
	this:RegisterEvent("PLAYER_LEVEL_UP");
end

function TitanPanelSkillsRightButton_OnLoad()
	this.registry = { 
		id = TITAN_SKILLS_RIGHT_ID,
		menuText = TITAN_SKILLS_RIGHT_MENU_TEXT, 
		buttonTextFunction = "TitanPanelSkillsButton_GetButtonText", 
		tooltipTitle = TITAN_SKILLS_TOOLTIP,
		tooltipTextFunction = "TitanPanelSkillsButton_GetTooltipText",
		icon = TITAN_SKILLS_RIGHT_ICON,
		iconWidth = 16,
		frequency = 0.5,
		updateType = TITAN_PANEL_UPDATE_TOOLTIP,
		savedVariables = {
			ShowIcon = 1,
			ShowLabelText = 1,
			ShowLevel = TITAN_NIL,
			ShowRank = 1,
			ShowCharLevel = 1,
		}
	};
end

function TitanPanelSkillsButton_OnEvent()
	TitanPanelButton_UpdateButton(TITAN_SKILLS_ID);	
	TitanPanelButton_UpdateTooltip();
end

function TitanPanelSkillsButton_OnClick()
        if (IsShiftKeyDown()) then
                ToggleCharacter("SkillFrame");
        end
end

--
-- Titan functions
--
function TitanPanelSkillsButton_GetButtonText(id)	
	local buttonRichText = "";
	
	if ( TitanGetVar(TITAN_SKILLS_ID,"ShowCharLevel") ~= nil ) then
		buttonRichText = " "..TitanUtils_GetHighlightText(UnitLevel("player"));
	end

	if ( id == TITAN_SKILLS_RIGHT_ID ) then
		return "", "";
	else
		return TITAN_SKILLS_BUTTON_LABEL, buttonRichText;
	end
end

function TitanPanelSkillsButton_GetTooltipText()
	
	local tooltipRichText = "";
	
	local SkillList = TitanPanelSkills_BuildSkillList();
	
	local numSkills = table.getn(SkillList);

	local i = 0;
	local currentHeader = "";
	local prevHeader = "";

	if ( TitanGetVar(TITAN_SKILLS_ID,"ShowRank") ~= nil ) then
		if (IsShiftKeyDown()) then
			showMethod = "level";
		else
			showMethod = "rank";
		end
	else
		if (IsShiftKeyDown()) then
			showMethod = "rank";
		else
			showMethod = "level";
		end
	end

	for i=1, numSkills do
		
		currentHeader = SkillList[i].stype;

		if ( SkillList[i].stype == TRADE_SKILLS or SkillList[i].stype == TITAN_SKILLS_SECONDARY_TEXT or SkillList[i].stype == TITAN_SKILLS_WEAPON_TEXT or SkillList[i].stype == TITAN_SKILLS_CLASS_TEXT ) then
			if ( currentHeader ~= prevHeader and SkillList[i].maxrank > 1 ) then
				tooltipRichText = tooltipRichText.."\n"..TITAN_SKILLS_WHITE..SkillList[i].stype.."\n";
				prevHeader = currentHeader;
			end
			
			if ( SkillList[i].maxrank > 1 and SkillList[i].name ~= "Riding") then
				digits = 1
				shift = 10 ^ digits
				
				currentRank = SkillList[i].rankbonus + SkillList[i].rank;
				TITAN_SKILLS_LEVEL = string.format ("%1.1f", ( ( ( SkillList[i].rankbonus + SkillList[i].rank ) / ( SkillList[i].rankbonus + 300 ) ) * 60 ) );
				TITAN_SKILLS_PERCENT = floor( ( (currentRank) / SkillList[i].maxrank ) * 100 );

				if (showMethod == "rank") then
					TITAN_SKILLS_SKILL_COLOR = TITAN_SKILLS_RED;
					if ( TITAN_SKILLS_PERCENT > 49) then
						TITAN_SKILLS_SKILL_COLOR = TITAN_SKILLS_ORANGE;
					end
					if ( TITAN_SKILLS_PERCENT > 74 ) then
						TITAN_SKILLS_SKILL_COLOR = TITAN_SKILLS_YELLOW;
					end
					if ( TITAN_SKILLS_PERCENT > 94) then
						TITAN_SKILLS_SKILL_COLOR = TITAN_SKILLS_GREEN;
					end
				else
					TITAN_SKILLS_SKILL_COLOR = TITAN_SKILLS_RED;
					if ( TITAN_SKILLS_PERCENT > 49) then
						TITAN_SKILLS_SKILL_COLOR = TITAN_SKILLS_ORANGE;
					end
					if ( TITAN_SKILLS_PERCENT > 74 ) then
						TITAN_SKILLS_SKILL_COLOR = TITAN_SKILLS_YELLOW;
					end
					if ( TITAN_SKILLS_PERCENT > 94) then
						TITAN_SKILLS_SKILL_COLOR = TITAN_SKILLS_GREEN;
					end
				end				

				-- insert the skill name
				tooltipRichText = tooltipRichText..TITAN_SKILLS_SKILL_COLOR.." "..SkillList[i].name..":\t";

				-- if a skill has a bonus modifier, then color the rank green
				bonusText = "";
				if ( SkillList[i].rankbonus > 0 ) then
					bonusText = "";
					tooltipRichText = tooltipRichText..TITAN_SKILLS_GREEN;
					bonusText = " + "..SkillList[i].rankbonus;
				end

				-- check for display type
				if (showMethod == "level") then
					tooltipRichText = tooltipRichText..TITAN_SKILLS_LEVEL.."\n";
				else
					tooltipRichText = tooltipRichText..SkillList[i].rank..bonusText.."/"..SkillList[i].maxrank.."\n";
				end
			end
		end
	end

	tooltipRichText = tooltipRichText.."\n"..TITAN_SKILLS_BLUE.."Hold SHIFT to toggle display\n"
	-- remove the last \n
	tooltipRichText = string.sub(tooltipRichText, 1, string.len(tooltipRichText)-1);

	return tooltipRichText;
end

--
-- create menus
--
function TitanPanelRightClickMenu_PrepareSkillsMenu()
	TitanPanelRightClickMenu_PrepareMenu(TITAN_SKILLS_ID);
end

function TitanPanelRightClickMenu_PrepareSkillsRightMenu()
	TitanPanelRightClickMenu_PrepareMenu(TITAN_SKILLS_RIGHT_ID);
end

function TitanPanelRightClickMenu_PrepareMenu(id)

	local info = {};
	
	if ( UIDROPDOWNMENU_MENU_LEVEL == 2 ) then
	
		if ( UIDROPDOWNMENU_MENU_VALUE == "DisplayAbout" ) then
			info = {};
			info.text = TITAN_SKILLS_ABOUT_POPUP_TEXT;
			info.value = "AboutTextPopUP";
			info.notClickable = 1;
			info.isTitle = 0;
			UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);
		end
		return;
	end
	
	TitanPanelRightClickMenu_AddTitle(TitanPlugins[id].menuText);
	info = {};
	info.text = TITAN_SKILLS_SHOW_RANK_TEXT;
	info.func = TitanPanelSkills_ShowRankCheck;
	info.checked = TitanGetVar(TITAN_SKILLS_ID, "ShowRank");
	UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);

	info = {};
	info.text = TITAN_SKILLS_SHOW_LEVEL_TEXT;
	info.func = TitanPanelSkills_ShowLevelCheck;
	info.checked = TitanGetVar(TITAN_SKILLS_ID, "ShowLevel");
	UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);
	
	if ( id == TITAN_SKILLS_ID ) then
		TitanPanelRightClickMenu_AddSpacer();
		info = {};
		info.text = TITAN_SKILLS_SHOW_CHAR_LEVEL_TEXT;
		info.value = "ShowCharLevelToggle";
		info.func = TitanPanelSkills_ShowCharLevelToggle;
		info.checked = TitanGetVar(TITAN_SKILLS_ID, "ShowCharLevel");
		UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);
		
		TitanPanelRightClickMenu_AddToggleLabelText(id);
	end

	TitanPanelRightClickMenu_AddSpacer();

	TitanPanelRightClickMenu_AddCommand(TITAN_PANEL_MENU_HIDE, id, TITAN_PANEL_MENU_FUNC_HIDE);

	-- info about plugin
	info = {};
	info.text = TITAN_SKILLS_ABOUT_TEXT;
	info.value = "DisplayAbout";
	info.hasArrow = 1;
	UIDropDownMenu_AddButton(info);
end

--
-- Skills functions
--
function TitanPanelSkills_BuildSkillList()

	local skillName, isHeader, isExpanded, skillRank, numTempPoints, skillModifier, skillMaxRank, isAbandonable;
	local stepCost, rankCost, minLevel, skillCostType;

	local skillType = "";
	local skillIndex = 0;
	local SkillList = { };

	local numSkills = GetNumSkillLines();

	for skillIndex=1, numSkills do
		
		skillName, isHeader, isExpanded, skillRank, numTempPoints, skillModifier, skillMaxRank, isAbandonable, stepCost, rankCost, minLevel, skillCostType = GetSkillLineInfo(skillIndex);
	
		if ( isHeader ) then
			skillType = skillName;
		else
			local entry = { name = skillName, stype = skillType, rank = skillRank, maxrank = skillMaxRank, rankbonus = skillModifier };
			table.insert(SkillList, entry);
		end
	end
	
	--TitanPanelSkills_DisplayTheList(SkillList);
	return SkillList;
end

function TitanPanelSkills_ShowRankCheck()
	TitanSetVar(TITAN_SKILLS_ID, "ShowLevel", nil);
	TitanSetVar(TITAN_SKILLS_ID, "ShowRank", 1);
	TitanPanelButton_UpdateButton(TITAN_SKILLS_ID);
end

function TitanPanelSkills_ShowLevelCheck()
	TitanSetVar(TITAN_SKILLS_ID, "ShowLevel", 1);
	TitanSetVar(TITAN_SKILLS_ID, "ShowRank", nil);
	TitanPanelButton_UpdateButton(TITAN_SKILLS_ID);
end


function TitanPanelSkills_ShowCharLevelToggle()
	if ( TitanGetVar(TITAN_SKILLS_ID, "ShowCharLevel") ) then
		TitanSetVar(TITAN_SKILLS_ID, "ShowCharLevel", nil);
	else
		TitanSetVar(TITAN_SKILLS_ID, "ShowCharLevel", 1);
	end
	
	TitanPanelButton_UpdateButton(TITAN_SKILLS_ID);
end

--
-- debug
-- 
function TitanPanelSkills_DisplayTheList(thelist)
	local i = 0;
	for i=1, table.getn(thelist) do
		TitanPanelSkills_ChatPrint(i..":"..thelist[i].name..":"..thelist[i].stype.."\n");
	end
end

function TitanPanelSkills_ChatPrint(msg)
        DEFAULT_CHAT_FRAME:AddMessage(msg);
end