--[[
TitanTradeCooldown
	By tsigo, goliathvt, and totalpackage
	Lists tradeskill cooldowns across all characters.

	Features
		- TitanPanel module
		- At-a-glance view of the status of tradeskills with cooldowns (Mooncloth/Transmutes)
	Thanks
		- WoW Wiki for the excellent resource

ChangeLog
v2.0.304  04/03/2007
- possible fix for possible doubling error

v2.0.303  03/31/2007
- ready alert nil bug fix

v2.0.302  03/27/2007
- more coding cleanup
- tradeskills no longer sort by character; it's sorted by time remaining
- added option to hide old tradeskill cooldowns (mooncloth, salt shaker)
- removed option to enable/disable alert; it's always enabled
- removed all nonimportant localization, such as options menu text (don't want to worry about these anymore)

v2.0.301b 02/02/2007
- 203 bug fix

v2.0.301b 02/02/2007
- support for new Tailoring cooldowns
- massive code cleanup (bugs awaiting)

...

]]--

local ID = "TradeCooldown"
local READY_COLOR = "|cff00FF00"
local NOTREADY_COLOR = "|cffFF0000"
local ALMOSTREADY_COLOR = "|cffF0F000"

--Time remaining to notify if the cooldown is almost ready
local NOTIFYTIME = 61

-- Localization
local BUTTON_LABEL = "TradeCooldown: "
local TOOLTIP_LABEL = "TradeCooldown"
local READY = "Ready!"
local MOONCLOTH = "Mooncloth"
local PRIMALMOONCLOTH = "Primal Mooncloth"
local SPELLCLOTH = "Spellcloth"
local SHADOWCLOTH = "Shadowcloth"
local SALTSHAKER = "Salt Shaker"
local REFINEDSALT = "Refined Deeprock Salt"
local SNOWMASTER = "SnowMaster 9000"
local SNOWBALL = "Snowball"
local EVERGREENPOUCH = "Evergreen Pouch"
local EVERGREENHERBCASING = "Evergreen Herb Casing"
local ELUNESLANTERN = "Elune's Lantern"
local ELUNESTONE = "Elune Stone"
local TRANSMUTE_MATCH = "Transmute"
local TRANSMUTES = "Transmutes"
local CLEARCONFIRM = "Are you sure you want to clear all data for Titan TradeCooldown?"
local NODATA = "No Data"
local CURRENTREALM = "Show Only Current Realm"
local SHOWEVERGREEN = "Enable Evergreen Pouch"
local CLEARALL = "Clear all data"
local CREATESEARCH = "You create (.*)."
local READYNOTIFY0 = "|cff00FF00TradeCooldown:|r %s for %s:%s is ready."
local READYNOTIFY1 = "|cff00FF00TradeCooldown:|r %s for %s:%s will be ready in %ss."
local WARN = "|cff00FF00TradeCooldown:|r Item created while tradeskill window is closed mooncloth/transmutes cooldown may not update until you reopen the window."
local ALCHEMY = "Alchemy"
local TAILORING = "Tailoring"

if GetLocale() == "deDE" then
	MOONCLOTH = "Mondstoff"
	PRIMALMOONCLOTH = "Urmondstoff"
	SPELLCLOTH = "Zauberstoff"
	SHADOWCLOTH = "Schattenstoff"
	SALTSHAKER = "Salzstreuer"
	REFINEDSALT = "Raffiniertes Tiefsteinsalz"
	SNOWMASTER = "Schneemeister 9000"
	SNOWBALL = "Schneeball"
	EVERGREENPOUCH = "Immergr\195\188nbeutel"
	EVERGREENHERBCASING = "Immergr\195\188nkrautgeh\195\164use"
	ELUNESLANTERN = "Elunes Laterne"
	ELUNESTONE = "Elunes Stein"
	TRANSMUTE_MATCH = "Transmutieren"
	TRANSMUTES = "Transmutationen"
	CREATESEARCH = "Ihr erschafft (.*)."
	ALCHEMY = "Alchimie"
	TAILORING = "Schneiderei"
	
elseif GetLocale() == "frFR" then
	MOONCLOTH = "Etoffe lunaire"
	PRIMALMOONCLOTH = "Etoffe lunaire primordiale"
	SPELLCLOTH = "Etoffe ensorcel\195\169e"
	SHADOWCLOTH = "Etoffe d'ombre"
	SALTSHAKER = "Tamis \195\160 sel"
	REFINEDSALT = "Sel"
	SNOWMASTER = "Ma\195\174treNeige 9000"
	SNOWBALL = "Neige"
	EVERGREENPOUCH = "Sacoche de printemps"
	EVERGREENHERBCASING = "Jardini\232res de printemps"
	ELUNESLANTERN = "Lanterne d'Elune"
	ELUNESTONE = "Pierre d'Elune"
	TRANSMUTE_MATCH = "Transmutation"
	TRANSMUTES = "Transmutations"
	CREATESEARCH = "Vous creez (.*)."
	ALCHEMY = "Alchimie"
	TAILORING = "Couture"
end

-- lookup table for tradeskills in tradeskill window
local tradeskills = {
	[TRANSMUTES] = true,
	[MOONCLOTH] = true,
	[PRIMALMOONCLOTH] = true,
	[SPELLCLOTH] = true,
	[SHADOWCLOTH] = true,
}

-- cd's that are useless in TBC basically
local oldstuff = {
	[MOONCLOTH] = true,
	[SALTSHAKER] = true,
}

-- professions where the cd is in the tradeskill window
local prof = {
	[TAILORING] = true,
	[ALCHEMY] = true,
}

TTC_Save = { }
local key
local rdyn, totn = 0, 0
local currealm
local insert, sort, getn = table.insert, table.sort, table.getn
local events = { }

function TitanPanelTradeCooldownButton_OnLoad()
	-- Titan Panel Registry
	this.registry = {
		id = ID,
		menuText = TOOLTIP_LABEL,
		buttonTextFunction = "TitanPanelTradeCooldownButton_GetButtonText",
		tooltipTitle = TOOLTIP_LABEL,
		tooltipTextFunction = "TitanPanelTradeCooldownButton_GetTooltipText",
		category = "Profession",
		icon = "Interface\\Icons\\Spell_Nature_TimeStop.blp",
		iconWidth = 16,
		savedVariables = {
			ShowIcon = 1,
			ShowLabelText = TITAN_NIL,
			ShowOnlyCurrentRealm = TITAN_NIL,
			ShowEvergreen = TITAN_NIL,
			ShowOld = 1,
		}
	}
    
	-- Events
	this:RegisterEvent("TRADE_SKILL_UPDATE")
	this:RegisterEvent("CHAT_MSG_SPELL_TRADESKILLS")
	this:RegisterEvent("CHAT_MSG_LOOT")
    
	this:SetScript("OnEvent", function()
		events[event]()
		TitanPanelButton_UpdateButton(ID)
		TitanPanelButton_UpdateTooltip(ID)
	end)
	this:SetScript("OnUpdate", TitanPanelTradeCooldownButton_OnUpdate)
    
	key = GetCVar("realmName").."|"..UnitName("player")
	currealm = GetCVar("realmName")
end

-- makes a new table entry if necessary and returns the reference
local function checkentry(value)
	if not TTC_Save[key] then
		TTC_Save[key] = { }
	end
	if not TTC_Save[key][value] then
		TTC_Save[key][value] = { }
	end
	return TTC_Save[key][value]
end

function events:TRADE_SKILL_UPDATE()
	-- if this is not Alchemy or Tailoring, don't waste time checking
	if not prof[GetTradeSkillLine()] then 
		return 
	end
	-- This is when the Save table is populated
	local numSkills = GetNumTradeSkills()
	for i=1, numSkills do
		local skillName = GetTradeSkillInfo(i)
		local cooldown = GetTradeSkillCooldown(i) or 0

		-- Condense all transmute entries into one, since they share a cooldown
		if strfind(skillName, TRANSMUTE_MATCH) then
			skillName = TRANSMUTES
		end

		if tradeskills[skillName] then
			local t = checkentry(skillName)
			t.Cooldown = cooldown
			if not (t.Cooldown <= 60 and t.IsReady == 1) then
				t.LastCheck = time()
				t.IsReady = 0 
			end
			
			-- check transmute only once
			if skillName == TRANSMUTES then break end
		end
	end
end
function events:CHAT_MSG_SPELL_TRADESKILLS()
	local _, _, created = strfind(arg1, CREATESEARCH)
	if created and strfind(created, SNOWBALL) then
		local t = checkentry(SNOWMASTER)
		t.Cooldown = 86400
		t.LastCheck = time()
		t.IsReady = 0
	elseif created and strfind(created, REFINEDSALT) then
		local t = checkentry(SALTSHAKER)
		t.Cooldown = 259200
		t.LastCheck = time()
		t.IsReady = 0 
	end
end
function events:CHAT_MSG_LOOT()
	if strfind(arg1, ELUNESTONE) then
		local t = checkentry(ELUNESLANTERN)
		t.Cooldown = 86400
		t.LastCheck = time()
		t.IsReady = 0
	elseif TitanGetVar(ID, "ShowEvergreen") == 1 and strfind(arg1, EVERGREENHERBCASING) then
		local t = checkentry(EVERGREENPOUCH)
		t.Cooldown = 600
		t.LastCheck = time()
		t.IsReady = 0
	end
end


function TitanPanelTradeCooldownButton_GetButtonText(id)
	return BUTTON_LABEL, format("%s%d/%d|r", (rdyn == 0 and NOTREADY_COLOR) or READY_COLOR, rdyn, totn)
end


local temp = { }
local sortThis = { }
local function GetTimeText(remaining)
	if remaining <= 0 then
		return "|cff00FF00Ready!|r"
	else
		local d,h,m,s = ChatFrame_TimeBreakDown(remaining)
		if d == 0 and h == 0 then
			return format("|cffF0F000%dd %02dh %02dm %02ds|r", d, h, m, s)
		else
			return format("|cffFF0000%dd %02dh %02dm %02ds|r", d, h, m, s)
		end
	end
end
local function ClearTable(t)
	if not t then return end
	for ind in pairs(t) do
		if type(t[ind]) == "table" then
			ClearTable(t[ind])
		end
		t[ind] = nil
	end
end
function TitanPanelTradeCooldownButton_GetTooltipText()
	local ttext = ""
	local dif = 0
	for k,v in pairs(TTC_Save) do
		local _,_,realm,player = strfind(k, "^(.+)\|(.+)$")
		for skill,t in pairs(v) do
			if (TitanGetVar(ID, "ShowOnlyCurrentRealm") ~= 1 or realm == currealm) and 
			   (TitanGetVar(ID, "ShowOld") == 1  or not oldstuff[skill]) then
				dif = dif + 0.0001
				local remaining = t.Cooldown + t.LastCheck - time() + dif
				insert(sortThis, remaining)
				temp[remaining] = { n = player, r = realm, s = skill, }
			end
		end
	end
	if getn(sortThis) > 0 then
		sort(sortThis)
		for _,value in ipairs(sortThis) do
			local t = temp[value]
			if TitanGetVar(ID, "ShowOnlyCurrentRealm") ~= 1 then
				ttext = format("%s%s:%s - |cffffffff%s|r\t%s\n", ttext, t.r, t.n, t.s, GetTimeText(value))
			else
				ttext = format("%s%s - |cffffffff%s|r\t%s\n", ttext, t.n, t.s, GetTimeText(value))
			end
		end
	end
	ClearTable(temp)
	ClearTable(sortThis)	
	if ttext == "" then
		return format("%s\n", NODATA)
	else
		return ttext
	end
end

-- metro function
local tcelapsed = 0
function TitanPanelTradeCooldownButton_OnUpdate()
	tcelapsed = tcelapsed + arg1
	if tcelapsed < 2 then return end
	tcelapsed = 0
	
	rdyn, totn = 0, 0
	for k,v in pairs(TTC_Save) do
		local _,_,realm,player = strfind(k, "^(.+)\|(.+)$")
		if TitanGetVar(ID, "ShowOnlyCurrentRealm") ~= 1 or realm == currealm then
			for s,t in pairs(v) do
				if TitanGetVar(ID, "ShowOld") == 1 or not oldstuff[s] then
					local remaining = t.Cooldown + t.LastCheck - time()
					totn = totn + 1
					if remaining <= 0 and t.IsReady ~= 1 then
						rdyn = rdyn + 1
						PlaySound("AuctionWindowOpen")
						t.IsReady = 1
						ChatFrame1:AddMessage(format(READYNOTIFY0, s, realm, player)) 
					elseif remaining < NOTIFYTIME and t.IsReady ~= 1 then
						PlaySound("AuctionWindowOpen")
						t.IsReady = 1
						ChatFrame1:AddMessage(format(READYNOTIFY1, s, realm, player, floor(remaining))) 
					elseif remaining <= 0 then
						rdyn = rdyn + 1
					end
				end
			end
		end
	end
	TitanPanelButton_UpdateButton(ID)
	TitanPanelButton_UpdateTooltip(ID)
end

-- dropdown menu functions
local function ToggleRealm()
	TitanToggleVar(ID, "ShowOnlyCurrentRealm")
	TitanPanelButton_UpdateButton(ID)
end
local function ToggleEvergreen()
	TitanToggleVar(ID, "ShowEvergreen")
	TitanPanelButton_UpdateButton(ID)
end
local function Clear()
	for index in pairs(TTC_Save) do
		TTC_Save[index] = nil
	end
	TitanPanelButton_UpdateButton(ID)
end
local function ClearConfirm()
	StaticPopupDialogs["TTC_CLEAR"] = {
		text = TEXT(CLEARCONFIRM),
		button1 = TEXT(OKAY),
		button2 = TEXT(CANCEL),
		OnAccept = Clear,
		timeout = 0,
		exclusive = 1,
	}
	StaticPopup_Show("TTC_CLEAR")
end

-- dropdown menu
function TitanPanelRightClickMenu_PrepareTradeCooldownMenu()
	TitanPanelRightClickMenu_AddTitle(TitanPlugins[ID].menuText)
	TitanPanelRightClickMenu_AddToggleIcon(ID)
	TitanPanelRightClickMenu_AddToggleLabelText(ID)

	TitanPanelRightClickMenu_AddSpacer()
	UIDropDownMenu_AddButton({
		text = CURRENTREALM,
		func = ToggleRealm,
		checked = TitanGetVar(ID, "ShowOnlyCurrentRealm"),
		keepShownOnClick = 1, })
	UIDropDownMenu_AddButton({
		text = "Show Old Skills",
		func = function() TitanToggleVar(ID, "ShowOld") end,
		checked = TitanGetVar(ID, "ShowOld"),
		keepShownOnClick = 1, })
	UIDropDownMenu_AddButton({
		text = SHOWEVERGREEN,
		func = ToggleEvergreen,
		checked = TitanGetVar(ID, "ShowEvergreen"),
		keepShownOnClick = 1, })

	TitanPanelRightClickMenu_AddSpacer()

	UIDropDownMenu_AddButton({
		text = CLEARALL,
		func = ClearConfirm, })
	
	TitanPanelRightClickMenu_AddSpacer()
	TitanPanelRightClickMenu_AddCommand(TITAN_PANEL_MENU_HIDE, ID, TITAN_PANEL_MENU_FUNC_HIDE)
end

